# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmCategoriesController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false


  protect_from_forgery with: :exception
  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_roles
  before_action :get_vm_version  
  before_action :clear_import_session
  before_action :clear_export_session

  # 権限チェック
  def check_roles
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    if [ 'create', 'modify', 'delete' ].include?(action_name) and !@sidfm_roles.include?(:admin) then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
  end

  def form_params
  end

  public

  def index
    @category_names = SidfmdCategory.all.select(:name).distinct(:name).collect {|x| x.name }
    filtered_params = params.fetch(:pages, {}).permit(:offset, :per_page)
    @categories = {}
    @categories[:per_page] = (filtered_params[:per_page] || 10).to_i
    @categories[:offset] = (filtered_params[:offset] || 0).to_i
    @categories[:content] = SidfmdCategory.order(:is_system, :id)
    if !@sidfm_roles.include?(:admin) then
      @categories[:content] = @categories[:content].where(:id => SidfmdCategoriesUser.where(:user_id => User.current.id).select(:sidfmd_category_id).distinct)
    end
    @categories[:content] = @categories[:content].offset(@categories[:offset]).limit(@categories[:per_page])
    @categories[:total_size] = @categories[:content].except(:order, :offset, :limit).count
    @users = User.where(:admin => false).where.not(:type => 'AnonymousUser').order(:lastname, :firstname)
  end

  def create
    filtered_params = params.fetch(:form, {}).permit(:func, :name, :desc)
    p_name = filtered_params['name']
    if p_name.nil? or p_name.size == 0 then
      redirect_to :action => :index, :notice => 'カテゴリ名が入力されていません。'
      return
    end
    p_name = p_name.strip
    if SidfmdCategory.where(:name => p_name).count > 0 then
      redirect_to :action => :index, :notice => '同名のカテゴリが存在します。'
      return
    end
    p_desc = filtered_params['desc']
    if p_desc.nil? or p_desc.size == 0 then
      redirect_to :action => :index, :notice => '概要が入力されていません。' and return
    end
    if p_name.size > 250 then
      redirect_to :action => :index, :notice => 'カテゴリ名が長すぎます。（最大250文字）' and return
    end
    if p_desc.size > 1000 then
      redirect_to :action => :index, :notice => '概要が長すぎます。（最大1000文字）' and return
    end
    begin
      SidfmdCategory.create(:name => p_name,
                            :desc => p_desc,
                            :is_system => false)
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'カテゴリの作成に失敗しました。' and return
    end
    redirect_to :action => :index
  end

  def modify
    filtered_params = params.fetch(:form, {}).permit(:id, :name, :desc)
    category = SidfmdCategory.find(filtered_params['id'].to_i)
    if category then
      opts = {}
      p_name = filtered_params['name']
      if p_name.nil? or p_name.size == 0 then
        redirect_to :action => :index, :notice => '名前が入力されていません。' and return
      end
      p_name = p_name.strip
      if (SidfmdCategory.where(:name => p_name) - [category]).size > 0 then
        redirect_to :action => :index, :notice => '変更予定のカテゴリ名について、同名のカテゴリが存在します。' and return
      end
      if category.name != p_name then
        opts[:name] = p_name
      end
      p_desc = filtered_params['desc']
      if p_desc.nil? or p_desc.size == 0 then
        redirect_to :action => :index, :notice => '概要が入力されていません。' and return
      end
      if category.desc != p_desc then
        opts[:desc] = p_desc
      end
      if p_name.size > 250 then
        redirect_to :action => :index, :notice => 'カテゴリ名が長すぎます。（最大250文字）' and return
      end
      if p_desc.size > 1000 then
        redirect_to :action => :index, :notice => '概要が長すぎます。（最大1000文字）' and return
      end
      begin
        SidfmdCategory.transaction do
          category.update_attributes(opts)
          category.save
        end
        Rails.cache.clear
      rescue Exception => e
        log_exception(e)
        redirect_to :action => :index, :notice => '更新に失敗しました。' and return
      end
    else
      redirect_to :action => :index, :notice => '更新対象のカテゴリが存在しません。' and return
    end
    redirect_to :action => :index
  end

  def delete
    #filtered_params = params.fetch(:formdel, {}).permit(:id, :rule_info)
    filtered_params = params.permit(:id, :rule_info)
    category_id = filtered_params['id'].to_i
    err_message = ''
    unless category_id then
      err_message = '不正な入力です。'
      render :layout => false, :json => {:err_message => err_message} and return  
    end
    unless category_id > 0 then
      err_message = '不正な入力です。'
      render :layout => false, :json => {:err_message => err_message} and return
    end
    begin
      rule_info = JSON.parse(filtered_params[:rule_info] || '{}').symbolize_keys
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = '不正な入力です。'
      render :layout => false, :json => {:err_message => err_message} and return
    end
    
    begin
      SidfmdCategory.transaction do
        category = SidfmdCategory.lock.find(category_id)
        if category and !category.is_system then
          SidfmdCategoriesHost.where(:sidfmd_category_id => category_id).delete_all
          SidfmdCategoriesUser.where(:sidfmd_category_id => category_id).delete_all
          category.destroy
        else
          err_message = '不正な入力です。'
          render :layout => false, :json => {:err_message => err_message} and return
        end
        #関連するルール・ルールの条件・操作の削除
        delete_rule_ids = rule_info[:delete_rule_ids]
        delete_cond_ids = rule_info[:delete_cond_ids]
        delete_action_ids = rule_info[:delete_action_ids]
        delete_category_cond_ids = rule_info[:delete_category_cond_ids]
        delete_host_cond_ids = rule_info[:delete_host_cond_ids]
        if delete_rule_ids && delete_rule_ids.size > 0 then 
          SidfmdRule.where(:id => delete_rule_ids).destroy_all
        end
        if delete_cond_ids && delete_cond_ids.size > 0 then
          SidfmdRuleCond.where(:id => delete_cond_ids).destroy_all
        end
        if delete_action_ids && delete_action_ids.size > 0 then
          SidfmdRuleAction.where(:id => delete_action_ids).destroy_all
        end
        if delete_category_cond_ids && delete_category_cond_ids.size > 0 then
          SidfmdRuleCmCategorySub.where(:id => delete_category_cond_ids).destroy_all
        end
        if delete_host_cond_ids && delete_host_cond_ids.size > 0 then
          SidfmdRuleCmHostSub.where(:id => delete_host_cond_ids).destroy_all
        end
      end

      #View更新のためのid取得
      host_ids = SidfmdHostView.where(
                   :id => SidfmdHvCategory.where(:sidfmd_category_id => category_id).select(:sidfmd_host_view_id)
                 ).pluck(:sidfmd_host_id)
      html_ids = SidfmdHostView.relate_htmlids(host_ids).uniq
    rescue Exception => e
      log_exception(e)
      err_message = 'カテゴリの削除に失敗しました。'
    end
    Rails.cache.clear
    render :layout => false, :json => { :host_ids => host_ids, :html_ids => html_ids, :err_message => err_message }
  end

  def update_view
    f_params = params.permit(:host_ids)
    host_ids = JSON.parse(f_params[:host_ids])
    target_host_ids = host_ids.shift(100)
    executed_hosts = target_host_ids.size
    target_html_ids = SidfmdHostView.relate_htmlids(target_host_ids)
    SidfmdHostView.sync_data(target_host_ids)
    SidfmdVulView.sync_data(target_html_ids.uniq)
    render :layout => false, :json => { :executed_hosts => executed_hosts, :unexecuted_host_ids => host_ids }
  end

end
