# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles

  before_action :get_view_state
  after_action :put_view_state

  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session

  # 権限チェック
  def check_roles
    @role_admin = @sidfm_roles.include?(:admin)
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    @user_category_ids = if @role_admin then
                           nil
                         else
                           SidfmdCategoriesUser.where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
                     end
    @cache_prefix = @role_admin ? 'dashboard/all' : "dashboard/categories/#{User.current.id}"
  end

  def create_hosts_all_content

    @data[:hosts] = {
      :all_content => SidfmdHost.create_all_content(category_id: 0, role_admin: @role_admin, user_host_ids: @user_host_ids, user_category_ids: @user_category_ids, corresp_required: @corresp_required)
    }
    @data[:hosts][:all_content].sort! {|x, y|
      (x[:status_ids].size > 0 ? 0 : 100) <=> (y[:status_ids].size > 0 ? 0 : 100) &&
      x[:hostname] <=> y[:hostname]
    }
  end

  def create_vuls_all_content
    @data[:vuls] = {
      :all_content => SidfmdIssue.create_all_content(category_id: 0, role_admin: @role_admin, user_host_ids: @user_host_ids, user_category_ids: @user_category_ids, corresp_required: @corresp_required),
    }
    @enable_host_ids = SidfmdHost.enable_hosts.collect {|x| x.id }
    @category_host_ids =
      if @user_category_ids then
        SidfmdCategoriesHost
          .where(:sidfmd_category_id => @user_category_ids)
          .select(:sidfmd_host_id).distinct.collect {|x| x.sidfmd_host_id }
      else
        SidfmdCategoriesHost.select(:sidfmd_host_id).distinct.collect {|x| x.sidfmd_host_id }
      end
    @data[:vuls][:all_content].sort! {|x, y|
      SidfmdIssue::SORT_PRIORITIES[x[:min_status_id]] <=> SidfmdIssue::SORT_PRIORITIES[y[:min_status_id]] ||
      y[:min_created_on][:value] <=> x[:min_created_on][:value]
    }

  end

  def parse_params
    #create_hosts_all_content
    #create_vuls_all_content
  end

  def create_data
    create_hosts_data
    create_vuls_data
  end

  def create_hosts_data
    @data[:hosts] = {}

    #権限によってデータが変わるため、SidfmdHostViewからのデータを使用する時は@host_viewを使う
    @host_view = SidfmdHostView.user_host_view(@role_admin, @user_host_ids)
    
    @data[:hosts][:all_host_size] = @host_view.count

    @data[:hosts][:corresp_required_host_size] = {}
    @data[:hosts][:fixed_corresp_required_host_size] = {}
    @data[:hosts][:not_fixed_corresp_required_host_size] = {}
  
    @data[:hosts][:corresp_required_host_size][:sri] = @host_view.where(:warning_sri => true).count
    @data[:hosts][:corresp_required_host_size][:cvss] = @host_view.where(:warning_cvss => true).count
 
    @data[:hosts][:fixed_corresp_required_host_size][:sri] = @host_view.where(:fixed_warning_sri => true).count  
    @data[:hosts][:fixed_corresp_required_host_size][:cvss] = @host_view.where(:fixed_warning_cvss => true).count

    @data[:hosts][:not_fixed_corresp_required_host_size][:sri] = @host_view.where(:not_fixed_warning_sri => true).count
    @data[:hosts][:not_fixed_corresp_required_host_size][:cvss] = @host_view.where(:not_fixed_warning_cvss => true).count
=begin
    @data[:hosts][:all_host_size] = @data[:hosts][:all_content].size

    @data[:hosts][:corresp_required_host_size] = {}
    @data[:hosts][:fixed_corresp_required_host_size] = {}
    @data[:hosts][:not_fixed_corresp_required_host_size] = {}
    [ :sri, :cvss ].each do |i|
      @data[:hosts][:corresp_required_host_size][i] =
        @data[:hosts][:all_content].inject(0) {|r, x|
        r += (x[:warning][@data[:idx]] ? 1 : 0)
      }
      @data[:hosts][:fixed_corresp_required_host_size][i] =
        @data[:hosts][:all_content].sum {|x| x[:fixed_warning][@data[:idx]] ? 1 : 0 }
      @data[:hosts][:not_fixed_corresp_required_host_size][i] =
        @data[:hosts][:all_content].sum {|x| x[:not_fixed_warning][@data[:idx]] ? 1 : 0 }
    end
=end
  end

  def create_vuls_data
    @enable_host_ids = SidfmdHost.enable_hosts.collect {|x| x.id }
    @category_host_ids =
      if @user_category_ids then
        SidfmdCategoriesHost.where(:sidfmd_category_id => @user_category_ids)
                            .select(:sidfmd_host_id).distinct.collect {|x| x.sidfmd_host_id }
      else
        SidfmdCategoriesHost.select(:sidfmd_host_id).distinct.collect {|x| x.sidfmd_host_id }
      end
  
    #権限によってデータが変わるため、SidfmdVulViewからのデータを使用する時は@vul_viewを使う
    @vul_view = SidfmdVulView.user_vul_view(@role_admin, @user_host_ids)
                             .order(:min_status_id)
                             .order(:min_created_on)
    @data[:vuls] = {}
    @data[:vuls][:all_vul_size] = @vul_view.count

    @data[:vuls][:corresp_required_vul_size] = {}
    @data[:vuls][:fixed_corresp_required_vul_size] = {}
    @data[:vuls][:not_fixed_corresp_required_vul_size] = {}
   
    @data[:vuls][:corresp_required_vul_size][:sri] = @vul_view.where(:warning_sri => true).count
    @data[:vuls][:corresp_required_vul_size][:cvss] = @vul_view.where(:warning_cvss => true).count

    @data[:vuls][:fixed_corresp_required_vul_size][:sri] = @vul_view.where(:fixed_warning_sri => true).count
    @data[:vuls][:fixed_corresp_required_vul_size][:cvss] = @vul_view.where(:fixed_warning_cvss => true).count

    @data[:vuls][:not_fixed_corresp_required_vul_size][:sri] = @vul_view.where(:not_fixed_warning_sri => true).count
    @data[:vuls][:not_fixed_corresp_required_vul_size][:cvss] = @vul_view.where(:not_fixed_warning_cvss => true).count
=begin
    @data[:vuls][:all_vul_size] = @data[:vuls][:all_content].size

    @data[:vuls][:corresp_required_vul_size] = {}
    @data[:vuls][:fixed_corresp_required_vul_size] = {}
    @data[:vuls][:not_fixed_corresp_required_vul_size] = {}

    [ :sri, :cvss ].each do |i|
      @data[:vuls][:corresp_required_vul_size][i] =
        @data[:vuls][:all_content].inject(0) {|r, x|
        r += (x[:warning][@data[:idx]] ? 1 : 0)
      }
      @data[:vuls][:fixed_corresp_required_vul_size][i] =
        @data[:vuls][:all_content].sum {|x| x[:fixed_warning][@data[:idx]] ? 1 : 0 }
      @data[:vuls][:not_fixed_corresp_required_vul_size][i] =
        @data[:vuls][:all_content].sum {|x| x[:not_fixed_warning][@data[:idx]] ? 1 : 0 }
    end
=end
  end

  public

  def index
    pages_params = params.fetch(:pages, {}).permit(:idx)
    @idx = (pages_params[:idx] || @view_state[:idx].to_s)
    @view_state[:idx] = @idx
  end

  def index_r
    pages_params = params.fetch(:pages, {}).permit(:idx)
    @data = {
      :sidfm_user => @sidfm_user,
      :sidfm_roles => @sidfm_roles,
    }
    @alerts = view_context.prepare_alerts
    @corresp_required = Sidfm.corresp_required
    @data[:idx] = (pages_params[:idx] || @view_state[:idx].to_s).to_sym
    @view_state[:idx] = @data[:idx]
    
    #login情報が無い場合にログイン情報を作成する。
    #SidfmdLoginHistoryテーブルが無いバージョンから、
    #ログインしたままSidfmdLoginHistoryがあるバージョンにアップデートするとログイン履歴が無いのでエラーが起きる
    #これを防ぐための処理
    user = User.current
    if !SidfmdLoginHistory.where(:user_id => user.id).exists? then
      SidfmdLoginHistory.create(:login_date => Time.now, :user_id => user.id, :login => user.login)
    end

    #parse_params
    create_data

    @data[:dist_cat_hosts] = {}

    #SidfmdCategory::LABELS.each do |label|
    #  @data[:dist_cat_hosts][label] = { 
    #    :count => 0, 
    #    :not_fixed_count => 0, 
    #    :max_sri => 0, 
    #    :max_cvss => { :label => :notreg, :score => nil }, 
    #    :elapsed_days => 0 
    #  }      
    #end
   
    scat_ids_with_names = SidfmdCategory.where(:is_system => true).map{|scat| [ scat.id, scat.name ]}.to_h 
    
    scat_ids_with_names.each do |scat_id, scat_name|
      label = SidfmdCategory.id2label(scat_id)
      count = @host_view.where(:system_category_id => scat_id).count

      target_hosts = if @data[:idx] == :sri then
                       @host_view.where(:system_category_id => scat_id).where(:warning_sri => true)
                     else
                       @host_view.where(:system_category_id => scat_id).where(:warning_cvss => true)
                     end
      if target_hosts.any?
        not_fixed_hosts = target_hosts.where(:min_status_id => SidfmdIssue::STATUS_UNFINISHED)
        min_dt = not_fixed_hosts.minimum(:min_elapsed_created_on)

        if @data[:idx] == :sri then
          max_sri_host = target_hosts.order(:max_sri).last
          max_sri = max_sri_host ? max_sri_host.max_sri : 0
          max_cvss = { :label => :notreg, :score => nil }
        else
          max_sri = 0
          max_cvss_host = target_hosts.order(:max_cvss_score).order("CASE max_cvss_label WHEN 'notreg' THEN 2 ELSE 1 END").last
          max_cvss = { :label => max_cvss_host.max_cvss_label.to_sym, :score => max_cvss_host.max_cvss_score }
        end
          
        @data[:dist_cat_hosts][label] = { 
          :count => count,
          :not_fixed_count => not_fixed_hosts.count,
          :max_sri => max_sri,
          :max_cvss => max_cvss,
          :elapsed_days => (min_dt ? ((Time.current - min_dt) / 24.0 / 60.0 / 60.0).to_i : 0)
        }
      else
        @data[:dist_cat_hosts][label] = {
          :count => count,
          :not_fixed_count => 0,
          :max_sri => 0,
          :max_cvss => { :label => :notreg, :score => nil },
          :elapsed_days => 0
        }
      end
    end

    @data[:recently_5_custom_categories] = []

    cat_hosts = if @role_admin then
                  SidfmdHvCategory.joins(:sidfmd_host_view)
                else
                  SidfmdHvCategory.where(:sidfmd_category_id => @user_category_ids).joins(:sidfmd_host_view)
                end
    cat_hosts.distinct.pluck(:sidfmd_category_id).each do |cat_id|
      target_data = cat_hosts.where(:sidfmd_category_id => cat_id)
      cat_data = target_data.first
      next unless cat_data
      min_dt = target_data.where(:sidfmd_host_views => {:min_status_id => SidfmdIssue::STATUS_UNFINISHED})
                          .minimum(:min_elapsed_created_on)
      max_sri = target_data.maximum(:max_sri)
      max_cvss_data = target_data.order('sidfmd_host_views.max_cvss_score')
                                 .order("CASE sidfmd_host_views.max_cvss_label WHEN 'notreg' THEN 2 ELSE 1 END")
                                 .select('sidfmd_host_views.max_cvss_label', 'sidfmd_host_views.max_cvss_score')
                                 .last
                                 
      max_cvss = { :label => max_cvss_data.max_cvss_label.to_sym, :score => max_cvss_data.max_cvss_score } 
                 
      item = {
        :id => cat_data.sidfmd_category_id,
        :name => cat_data.name,
        :host_count => target_data.distinct.count('sidfmd_host_views.id'),
        :max_sri => max_sri ? max_sri : 0,
        :max_cvss => max_cvss,
        :elapsed_days => (min_dt ? ((Time.current - min_dt) / 24.0 / 60.0 / 60.0).to_i : -1)
      }
      item[:not_fixed_count] = if @data[:idx] == :sri then 
                                 target_data.where(:sidfmd_host_views => {:warning_sri => true})
                                            .where(:sidfmd_host_views => {:min_status_id => SidfmdIssue::STATUS_UNFINISHED})
                                            .distinct
                                            .count('sidfmd_host_views.id')
                               else
                                 target_data.where(:sidfmd_host_views => {:warning_cvss => true})
                                            .where(:sidfmd_host_views => {:min_status_id => SidfmdIssue::STATUS_UNFINISHED})
                                            .distinct
                                            .count('sidfmd_host_views.id')
                               end
      @data[:recently_5_custom_categories] << item
      
    end
 
    @data[:recently_5_custom_categories].sort! {|x, y|
      temp_ev = proc {|x|
        [ x[:not_fixed_count], x[:max_cvss][:score] ? x[:max_cvss][:score] : 0.0 , x[:max_sri] ]
      }
      xx = temp_ev.call(x)
      yy = temp_ev.call(y)
      r = yy[2] <=> xx[2]
      if r == 0 then
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = yy[0] <=> xx[0]
        end
      end
      r
    }
    @data[:recently_5_custom_categories] = @data[:recently_5_custom_categories][0, 5]
    
    hosts = @host_view.where(:min_status_id => SidfmdIssue::STATUS_UNFINISHED)
                      .order(:max_sri => :desc)
                      .order(:max_cvss_score => :desc)
                      .order(:min_elapsed_created_on)
                      .limit(5)

    if hosts.size < 5 then
      hosts = hosts + 
                @host_view.where.not(:min_status_id => SidfmdIssue::STATUS_UNFINISHED)
                          .order(:max_sri => :desc)
                          .order(:max_cvss_score => :desc)
                          .order(:min_elapsed_created_on)
                          .limit(5 - hosts.size)
    end

    @data[:recently_5_hosts] = []

    hosts.each do |host|
      min_dt = host.min_elapsed_created_on
      @data[:recently_5_hosts] << { 
        :sidfmd_host_id => host.sidfmd_host_id,
        :hostname => host.hostname,
        :ext_name => host.ext_name,
        :display_name => host.display_name,
        :system_category => { :id => host.system_category_id, :name => host.system_category_name },
        :no_system_categories => if @role_admin then
                                   host.sidfmd_hv_categories.map{|cat|
                                     { :id => cat.sidfmd_category_id, :name => cat.name }
                                   }
                                 else
                                   host.sidfmd_hv_categories.where(:sidfmd_category_id => @user_category_ids).map{|cat|
                                     { :id => cat.sidfmd_category_id, :name => cat.name }
                                   }
                                 end,
        :min_status_id => host.min_status_id,
        :max_sri => host.max_sri,
        :max_cvss => { :label => host.max_cvss_label.to_sym, :score => host.max_cvss_score },
        :elapsed_days => (min_dt ? ((Time.current - min_dt) / 24.0 / 60.0 / 60.0).to_i : -1),
        :status_ids => host.sidfmd_hv_scsids.pluck(:status_id)
      }
    end
     
=begin
    @data[:hosts][:all_content].each do |host|
      syscat = host[:system_category]
      if syscat then
        label = SidfmdCategory.id2label(syscat[:id])
        if label then
          @data[:dist_cat_hosts][label][:count] += 1
          if host[:warning][@data[:idx]] and SidfmdIssue::STATUS_UNFINISHED.include?(host[:min_status_id]) then
            @data[:dist_cat_hosts][label][:not_fixed_count] += 1
            if @data[:dist_cat_hosts][label][:elapsed_days] < host[:elapsed_days] then
              @data[:dist_cat_hosts][label][:elapsed_days] = host[:elapsed_days]
            end
          end
          case @data[:idx]
          when :sri then
            if @data[:dist_cat_hosts][label][:max_sri] < host[:max_sri] then
              @data[:dist_cat_hosts][label][:max_sri] = host[:max_sri]
            end
          when :cvss then
            if host[:max_cvss][:score] then
              if (@data[:dist_cat_hosts][label][:max_cvss][:score] and @data[:dist_cat_hosts][label][:max_cvss][:score] < host[:max_cvss][:score]) or @data[:dist_cat_hosts][label][:max_cvss][:score].nil? then
                @data[:dist_cat_hosts][label][:max_cvss] = host[:max_cvss]
              end
            end
          end
        end
      end
    end

    temp_r5cc = {}
    @data[:hosts][:all_content].each do |host|
      host[:no_system_categories].each do |cat|
        catid = cat[:id]
        if temp_r5cc.has_key?(catid) then
          temp_r5cc[catid][:host_count] += 1
          if host[:warning][@data[:idx]] and host[:min_status_id] == SidfmdIssue::STATUS_NOTCONFIRMED then
            temp_r5cc[catid][:not_fixed_count] += 1
          end
          if (host[:max_sri] and temp_r5cc[catid][:max_sri] and temp_r5cc[catid][:max_sri] < host[:max_sri]) or
              temp_r5cc[catid][:max_sri].nil? then
            temp_r5cc[catid][:max_sri] = host[:max_sri]
          end
          if (host[:max_cvss][:score] and temp_r5cc[catid][:max_cvss][:score] and temp_r5cc[catid][:max_cvss][:score] < host[:max_cvss][:score]) or
              temp_r5cc[catid][:max_cvss][:score].nil? then
            temp_r5cc[catid][:max_cvss] = host[:max_cvss]
          end
          
          if (host[:elapsed_days] > temp_r5cc[catid][:elapsed_days]) && 
             ((SidfmdIssue::STATUS_UNFINISHED - host[:status_ids]).size != SidfmdIssue::STATUS_UNFINISHED.size)  then
            temp_r5cc[catid][:elapsed_days] = host[:elapsed_days]
          end
        else
          temp_r5cc[catid] = {
            :id => catid,
            :name => cat[:name],
            :host_count => 1,
            :not_fixed_count => (host[:warning][@data[:idx]] and host[:min_status_id] == SidfmdIssue::STATUS_NOTCONFIRMED) ? 1 : 0,
            :max_sri => host[:max_sri],
            :max_cvss => host[:max_cvss],
            :elapsed_days => ((SidfmdIssue::STATUS_UNFINISHED - host[:status_ids]).size != SidfmdIssue::STATUS_UNFINISHED.size) ? host[:elapsed_days] : -1
          }
        end
      end
    end
    @data[:recently_5_custom_categories] = temp_r5cc.keys.collect {|x| temp_r5cc[x] }
    @data[:recently_5_custom_categories].sort! {|x, y|
      temp_ev = proc {|x|
        [ x[:not_fixed_count], x[:max_cvss][:score] ? x[:max_cvss][:score] : 0.0 , x[:max_sri] ]
      }
      xx = temp_ev.call(x)
      yy = temp_ev.call(y)
      r = yy[2] <=> xx[2]
      if r == 0 then
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = yy[0] <=> xx[0]
        end
      end
      r
    }
    @data[:recently_5_custom_categories] = @data[:recently_5_custom_categories][0, 5]

    #@data[:recently_5_custom_categories].each do |cat|
    #  min_created_on =
    #    SidfmdIssue
    #    .enable_hosts
    #    .where(:sidfmd_host_id =>
    #           SidfmdCategoriesHost
    #             .where(:sidfmd_category_id => cat[:id])
    #             .select(:sidfmd_host_id))
    #    .joins(:issue)
    #    .where(:issues => { :status_id => SidfmdIssue::STATUS_UNFINISHED })
    #    .select(:created_on)
    #    .minimum(:created_on)
    #  cat[:elapsed_days] = min_created_on ? ((Time.current - min_created_on) / 24.0 / 60.0 / 60.0).to_i : -1
    #end

    temp_ev = proc {|x|
      [ SidfmdIssue::STATUS_UNFINISHED.include?(x[:min_status_id]) ? 100 : 1000,
        x[:max_sri] ? x[:max_sri] : 0,
        x[:max_cvss][:score] ? x[:max_cvss][:score] : 0.0,
        x[:elapsed_days], ]
    }
    @data[:recently_5_hosts] = @data[:hosts][:all_content].sort {|x, y|
      xx = temp_ev.call(x)
      yy = temp_ev.call(y)
      r = xx[0] <=> yy[0]
      if r == 0 then
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = yy[2] <=> xx[2]
          if r == 0 then
            r = yy[3] <=> xx[3]
          end
        end
      end
      r
    }[0, 5]

    @data[:recently_5_vuls] = @data[:vuls][:all_content].sort {|x, y|
      temp_ev = proc {|x|
        [ x[:warning][@data[:idx]] ? 100 : 1000, x[:max_sri] ? x[:max_sri] : 0, x[:cvss][:score] ? x[:cvss][:score] : 0.0, x[:not_fixed_hosts], x[:min_created_on][:display] ]
      }
      xx = temp_ev.call(x)
      yy = temp_ev.call(y)
      r = xx[0] <=> yy[0]
      if r == 0 then
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = yy[2] <=> xx[2]
          if r == 0 then
            r = yy[3] <=> xx[3]
            if r == 0 then
              r = yy[4] <=> xx[4]
            end
          end
        end
      end
      r
    }[0, 5]
=end

=begin    @data[:recently_5_vuls] = @vul_view.order("warning_#{@data[:idx]} DESC")
                                       .order('CASE max_sri WHEN NULL THEN 0 ELSE max_sri END DESC')
                                       .order('CASE cvss_score WHEN NULL THEN -1 ELSE cvss_score END DESC')
                                       .order(:not_fixed_hosts => :desc)
                                       .order(:min_elapsed_created_on_display => :desc)
                                       .limit(5)
=end
   
    @data[:recently_5_vuls] = @vul_view.sort {|x, y|
      temp_ev = if @data[:idx] == :sri then 
                  proc {|x|
                    [ x[:warning_sri] ? 100 : 1000, x[:max_sri] ? x[:max_sri] : 0, x[:cvss_score] ? x[:cvss_score] : 0.0, x[:not_fixed_hosts], x[:min_created_on_display] ]
                  }
                else
                  proc {|x|
                    [ x[:warning_cvss] ? 100 : 1000, x[:max_sri] ? x[:max_sri] : 0, x[:cvss_score] ? x[:cvss_score] : 0.0, x[:not_fixed_hosts], x[:min_created_on_display] ]
                  }
                end
      xx = temp_ev.call(x)
      yy = temp_ev.call(y)
      r = xx[0] <=> yy[0]
      if r == 0 then
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = yy[2] <=> xx[2]
          if r == 0 then
            r = yy[3] <=> xx[3]
            if r == 0 then
              r = yy[4] <=> xx[4]
            end
          end
        end
      end
      r
    }[0, 5]
    @data[:recently_5_journals] = Journal
      .joins(:issue)
      .joins(:details)
      .joins(:issue => :sidfmd_issue)
      .joins(:issue => { :sidfmd_issue => :sidfmd_host })
      .where(:private_notes => false)
      .where(:sidfmd_hosts => { :flg_delete => false })
    if @user_category_ids then
        @data[:recently_5_journals] = @data[:recently_5_journals].where(:sidfmd_issues => {:sidfmd_host_id => SidfmdCategoriesHost.all.where(:sidfmd_category_id => @user_category_ids).select(:sidfmd_host_id).distinct})
    end
    @data[:recently_5_journals] = @data[:recently_5_journals]
      .where(:journal_details => { :prop_key => [ 'status_id', 'due_date' ] })
      .order(:created_on => :desc)
      .limit(5)
  end

end
