# -*- coding: utf-8 -*-

require 'sidfmdlm_api'
require 'sidfm_controller_support'

class SidfmInformationsController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles
  
  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session

  # 権限チェック
  def check_roles
    redirect_to home_path, :notice => '許可されていない操作です。' unless @sidfm_user
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           SidfmdCategoriesUser.where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
                     end
    @cache_prefix = @role_admin ? '/infos/admin/0' : "/infos/user/#{@user_category_ids.join(",")}"
  end

  def escape_special_chars(text)
    text = text.gsub(/&/,'&amp;')
    text = text.gsub(/<|>|"|'/, '<' => '&lt;', '>' => '&gt;', '"' => '&quot;', "'" => '&#39;', '`' => '&#x60;')
  end

  def get_local_vuln_data(local_vuln)
    vuln = {}
    vuln[:id] = local_vuln.html_id
    vuln[:title] = local_vuln.title
    vuln[:area_id] = local_vuln.area_id
    vuln[:create_datetime] = local_vuln.created_at
    vuln[:update_datetime] = local_vuln.updated_at
    vuln[:risk_id] = local_vuln.risk_id
    #影響を受ける製品
    affect_products = []
    affect_versions = {}
    affect_lproducts = []
    affect_lversions = {}
    pushed_pid = []
    pushed_lpid = []
    local_vuln.sidfmd_local_vul_apps.each do |app|
      pid = app.pid
      vid = app.vid
      prod = SidfmdProduct.find_by(:pid => pid)
      next unless prod
      if !pushed_pid.include?(pid) then
        affect_products << {
          :product => {:id => pid, :name => prod.name},
          :create_date => prod.create_datetime,
          :update_date => prod.update_datetime
        }
        pushed_pid << pid
      end
      if vid != 0 then #vid != 0 はバージョン
        version = SidfmdVersion.find_by(:vid => vid)
        next unless version
        affect_versions[pid] = [] unless affect_versions[pid]
        affect_versions[pid] << version.version
      end
    end
    local_vuln.sidfmd_local_vul_lapps.each do |lapp|
      pid = lapp.sidfmd_lproduct_id
      vid = lapp.sidfmd_lversion_id
      lprod = SidfmdLproduct.find_by(:id => pid)
      next if !lprod || !lprod.flg_enable
      if !pushed_lpid.include?(pid) then
        affect_lproducts << {
          :product => {:lproduct_id => pid, :name => lprod.name},
          :create_date => lprod.created_at,
          :update_date => lprod.updated_at
        }
        pushed_lpid << pid
      end
      if vid != 0 then #vid != 0 はバージョン
        version = SidfmdLversion.find_by(:id => vid)
        next unless version
        affect_lversions[pid] = [] unless affect_lversions[pid]
        affect_lversions[pid] << version.version
      end
    end
    vuln[:affect_products] = (affect_products + affect_lproducts).sort_by{|p| p[:product][:name].downcase}
    #content
    vuln[:content] = {}
    #影響を受ける製品
    if vuln[:affect_products].any? then
      affect_prods_elm = '<ul>'
      vuln[:affect_products].each do |p|
        version_names = []
        if p[:product][:id] then
          prod_versions = affect_versions[p[:product][:id]]
          version_names = prod_versions if prod_versions
        else
          lprod_versions = affect_lversions[p[:product][:lproduct_id]]
          version_names = lprod_versions if lprod_versions
        end
        product = '<li class=\"d1\">' + "\n" + '<p><b>' + p[:product][:name] + "</b></p>\n"
        versions = ''
        version_names.each do |v_name|
          versions += '<p>' + v_name + "</p>\n"
        end
        product += versions + "</li>\n"
        affect_prods_elm += product
      end
      affect_prods_elm += '</ul>'
    else
      affect_prods_elm = ''
    end
    vuln[:content][:affect] = affect_prods_elm
    #概要
    description = escape_special_chars(local_vuln.description)
    description = SidfmdLocalVul.get_link_converted_text(description)
    if description[:err] then #エラーは登録時に確認するため、通常は発生しない
      logger.info('converting link is failed: ' + description[:err])
      description = local_vuln.description 
    else
      description = description[:text]
    end
    vuln[:content][:description] = '<p>' + description + '</p>'
    #影響などの確認方法
    verification  = escape_special_chars(local_vuln.confirmation)
    verification  = SidfmdLocalVul.get_link_converted_text(verification)
    if verification[:err] then
      logger.info('converting link is failed: ' + verification[:err])
      verification = local_vuln.confirmation
    else
      verification = verification[:text]
    end
    vuln[:content][:verification] = '<p>' + verification + '</p>'
    #対処方法
    solution = escape_special_chars(local_vuln.workaround)
    solution = SidfmdLocalVul.get_link_converted_text(solution)
    if solution[:err] then
      logger.info('converting link is failed: ' + solution[:err]) 
      solution = local_vuln.workaround
    else
      solution = solution[:text]
    end
    vuln[:content][:solution] = solution
    #関連情報
    related_info = escape_special_chars(local_vuln.related)
    related_info = SidfmdLocalVul.get_link_converted_text(related_info)
    if related_info[:err] then
      logger.info('converting link is failed: ' + related_info[:err])
      related_info = local_vuln.related
    else
      related_info = related_info[:text]
    end
    vuln[:content][:link] = related_info
    #更新履歴
    logs = local_vuln.sidfmd_local_vul_logs.order(:id)
    if logs.any? then 
      history = '<ul>'
      logs.each do |log|
        conv_log = escape_special_chars(log.mes)
        conv_log = SidfmdLocalVul.get_link_converted_text(conv_log)
        log = (conv_log[:err] ? log : conv_log[:text])
        history += '<li>' + "\n" + '<p><font color=\"#000000\">' + log + "</font></p>\n" + "</li>\n" 
      end
      history += '</ul>'
    else
      history = ''
    end
    vuln[:content][:history] = history
    #CVES
    cves = []
    local_vuln.sidfmd_local_vul_cves.each do |cve|
      cves << {
        :cve_id => cve.cve_id,
        :cvss3 => {
          :vector => cve.c3_vector,
          :base_score => cve.c3_base_score,
          :create_datetime => cve.created_at,
          :update_datetime => cve.updated_at,
          :exploitable => cve.c3_exploitable
        }
      }
    end
    vuln[:cves] = cves
    #sris
    sris = []
    local_vuln.sidfmd_local_vul_sris.order(:category_id).each do |sri|
      sris << {
        :category_id => sri.category_id,
        :risk_id => sri.risk_id,
        :memo => (sri.memo.size > 0 ? sri.memo : nil)
      }
    end
    vuln[:sris] = sris 
    return vuln
  end

  def get_local_prod
    lprod = SidfmdLproduct.find_by_id(@id)
    if !lprod || !lprod.flg_enable then
      return nil
    end
    lprod_info = {}
    lprod_info[:id] = lprod.id
    lprod_info[:name] = lprod.name
    lprod_info[:is_support_end] = true
    lprod_info[:category] = 'ローカルプロダクト'
    lprod_info[:support_date] = nil
    lprod_info[:create_datetime] = lprod.created_at
    lprod_info[:update_datetime] = lprod.updated_at
    versions = lprod.sidfmd_lversions.order(:version)
    formated_versions = []
    eols = []
    versions_info = {}
    versions.each_with_index do |v, i|
      formated_versions << {
        :id => v.id,
        :fullname => v.fullname,
        :version => v.version,
        :is_support_end => true,
        :position => i,
        :release_date => nil,
        :support_date => nil,
        :create_datetime => v.created_at,
        :update_datetime => v.updated_at
      }
      eols << {
         :name => v.fullname,
         :date => nil,
         :url => nil
      }
      versions_info[v.id] = v
    end
    lprod_info[:versions] = formated_versions
    vulns = get_local_prod_vulns(lprod)
    affect_hosts = get_local_prod_hosts(lprod, versions_info)
    return {:lprod_info => lprod_info, :eols => eols, :vulns => vulns, :affect_hosts => affect_hosts}
  end

  def get_local_prod_vulns(lproduct)
    local_vul_lapps = SidfmdLocalVulLapp.where(:sidfmd_lproduct_id => @id)
    vulns = SidfmdLocalVul.where(:id => local_vul_lapps.map(&:sidfmd_local_vul_id)).where(:flg_enable => true)
    ret = []
    vulns.each do |vul|
      r = {}
      r[:id] = vul.html_id
      r[:title] = vul.title
      r[:area_id] = vul.area_id
      r[:create_datetime] = vul.created_at
      r[:update_datetime] = vul.updated_at
      r[:risk_id] = vul.risk_id
      r[:affect_products] = [
        {
          :product => { :id => lproduct.id, :name => lproduct.name },
          :create_datetime => lproduct.created_at,
          :update_datetime => lproduct.updated_at
        }
      ]
      cves = vul.sidfmd_local_vul_cves
      r[:cves] = cves.map{|cve|
        {
          :cve_id => cve.cve_id,
          :cvss3 => {
            :vector => cve.c3_vector,
            :base_score => cve.c3_base_score,
            :create_datetime => cve.created_at,
            :update_datetime => cve.updated_at,
            :exploitable => cve.c3_exploitable
          }
        }
      }
      ret << r
    end
    return ret
  end

  def get_local_prod_hosts(lproduct, versions) #local productのレコードと、keyがidでvalueがversionのハッシュが引数
    user_host_ids_role = if @role_admin then
                           SidfmdHost
                             .joins(:sidfmd_categories_hosts)
                             .joins(:sidfmd_categories)
                             .where(:flg_delete => false)
                             .where(:sidfmd_categories => { :is_system => true })
                             .select(:id)
                             .distinct
                         else
                           @user_host_ids
                         end
    ret = []
    hostlapps = SidfmdHostLapp
      .joins(:sidfmd_host)
      .includes(:sidfmd_host)
      .where(:sidfmd_host_id => user_host_ids_role)
      .where("(sidfmd_host_lapps.sidfmd_lproduct_id = ? OR sidfmd_host_lapps.sidfmd_lversion_id IN (?))", @id, versions.keys)
      .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
      .order('display_name')
    hostlapps.each do |hostlapp|
      temp_ver = versions[hostlapp.sidfmd_lversion_id]
      if temp_ver then
        temp_ret = {
          :type => :version,
          #:sidfmd_hostapp_id => hostlapp.id,
          :sidfmd_host_id => hostlapp.sidfmd_host_id,
          :hostname => hostlapp.sidfmd_host.hostname,
          :product_id => @id,
          :name => lproduct.name,
          :version_id => hostlapp.sidfmd_lversion_id,
          :version => temp_ver.version,
          :full_name => temp_ver.fullname,
          :is_support_end => true,
          :release_date =>  nil,
          :support_date => nil,
          :eol_date => nil,
          :ext_name => hostlapp.sidfmd_host.ext_name,
          :display_name => (hostlapp.sidfmd_host.ext_name ? hostlapp.sidfmd_host.ext_name : hostlapp.sidfmd_host.hostname)
        }
        ret << temp_ret
      else
        temp_ret = {
          :type => :product,
          #:sidfmd_hostapp_id => hostapp.id,
          :sidfmd_host_id => hostlapp.sidfmd_host_id,
          :hostname => hostlapp.sidfmd_host.hostname,
          :product_id => @id,
          :name => lproduct.name,
          :is_support_end => true,
          :support_date => nil,
          :ext_name => hostlapp.sidfmd_host.ext_name,
          :display_name => (hostlapp.sidfmd_host.ext_name ? hostlapp.sidfmd_host.ext_name : hostlapp.sidfmd_host.hostname)
        }
        ret << temp_ret 
      end
    end
    return ret
  end

  public

  def index
    redirect_to :action => :vuln_list
  end

  def vuln_list
  end

  def vuln
    @id = params[:id].to_i
    if @id <= 0 then
      redirect_to :action => :vuln_list, :notice => 'idが不正です。' and return
    end
    local_vuln = SidfmdLocalVul.find_by_html_id(@id)
    if local_vuln then
      @local_vuln = get_local_vuln_data(local_vuln)
      @areas = SidfmdArea.order(:area_id).map{|area| {:id => area.area_id, :name => area.name} } 
      
      @risks = SidfmdRisk.order(:risk_id).map{|risk| {:id => risk.risk_id, :name => risk.name, :name_en => risk.name_en} }
      
      cat_labels = SidfmdCategory::LABELS
      @sri_categories = SidfmdCategory.where(:is_system => true).order(:id).map.with_index{|c, idx|
        {:id => c.id, :name => c.name, :key => cat_labels[idx] }
      }
    else
      @local_vuln = nil
    end
  end

  def product_index
  end

  def product
    id = params[:id]
    if id.index('lproduct') then
      @id = id.slice(/[0-9]+/).to_i
      if @id <= 0 then
        redirect_to :action => :product_index, :notice => 'idが不正です。' and return
      end
      data = get_local_prod
      if !data then
        redirect_to :action => :product_index, :notice => 'プロダクトが見つかりませんでした。' and return
      end
      @local_prod = data[:lprod_info]
      @local_prod_eols = data[:eols]
      @local_prod_vulns = data[:vulns]
      @local_prod_affect_hosts = data[:affect_hosts]
      @areas = SidfmdArea.order(:area_id).map{|area| {:id => area.area_id, :name => area.name} }
      @risks = SidfmdRisk.order(:risk_id).map{|risk| {:id => risk.risk_id, :name => risk.name, :name_en => risk.name_en} }
    else 
      @id = id.to_i
      if @id <= 0 then
        redirect_to :action => :product_index, :notice => 'idが不正です。' and return
      end
      @local_prod = nil
    end
  end

  def news
  end

  # プロダクトIDから登録済ホストを取得する
  # input: product_id (GET)
  # output: [ { :sidfmd_host_id => 123, :hostname => 'xxx' }, ... ]
  def api_prd2hosts
    product_id = params[:id].to_i
    if product_id <= 0 then
      render :layout => false, :json => [], :status => 400
      return
    end
    product = SidfmdProduct.find_by_pid(product_id)
    unless product then
      render :layout => false, :json => [], :status => 400
      return
    end
    versions = product.sidfmd_versions
    version_ids = versions.collect {|x| x.vid }
    user_host_ids_role = if @role_admin then
                           SidfmdHost
                             .joins(:sidfmd_categories_hosts)
                             .joins(:sidfmd_categories)
                             .where(:flg_delete => false)
                             .where(:sidfmd_categories => { :is_system => true })
                             .select(:id)
                             .distinct
                         else
                           @user_host_ids
                         end
    ret = []
    hostapps = SidfmdHostApp
      .joins(:sidfmd_host)
      .includes(:sidfmd_host)
      .where(:sidfmd_host_id => user_host_ids_role)
      .where("(sidfmd_host_apps.product_id = ? OR sidfmd_host_apps.version_id IN (?))", product_id, version_ids)
      .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
      .order('display_name')
      #.order("sidfmd_hosts.hostname")
    hostapps.each do |hostapp|
      temp_prod = SidfmdProduct.find_by_pid(hostapp.product_id)
      temp_ver = SidfmdVersion.find_by_vid(hostapp.version_id)
      if temp_ver and temp_ver.sidfmd_product then
        temp_ret = {
          :type => :version,
          :sidfmd_hostapp_id => hostapp.id,
          :sidfmd_host_id => hostapp.sidfmd_host_id,
          :hostname => hostapp.sidfmd_host.hostname,
          :product_id => temp_ver.sidfmd_product_id,
          :name => temp_ver.sidfmd_product.name,
          :version_id => hostapp.version_id,
          :version => temp_ver.version,
          :full_name => temp_ver.fullname,
          :is_support_end => temp_ver.is_support_end,
          :release_date => temp_ver.release_date,
          :support_date => temp_ver.support_date,
          :eol_date => temp_ver.eol_date,
          :ext_name => hostapp.sidfmd_host.ext_name,
          :display_name => (hostapp.sidfmd_host.ext_name ? hostapp.sidfmd_host.ext_name : hostapp.sidfmd_host.hostname)
        }
        ret << temp_ret
      elsif temp_prod then
        temp_ret = {
          :type => :product,
          :sidfmd_hostapp_id => hostapp.id,
          :sidfmd_host_id => hostapp.sidfmd_host_id,
          :hostname => hostapp.sidfmd_host.hostname,
          :product_id => hostapp.product_id,
          :name => temp_prod.name,
          :is_support_end => temp_prod.is_support_end,
          :support_date => temp_prod.support_date,
          :ext_name => hostapp.sidfmd_host.ext_name,
          :display_name => (hostapp.sidfmd_host.ext_name ? hostapp.sidfmd_host.ext_name : hostapp.sidfmd_host.hostname)
        }
        ret << temp_ret
      else
        # skip: invalid data
      end
    end
    # see vm_daemon
    queue_ids = SidfmdHost
      .where(:flg_delete => false)
      .where(:id => user_host_ids_role)
      .select("(SELECT id FROM sidfmd_queues WHERE sidfmd_hosts.id=sidfmd_queues.sidfmd_host_id ORDER BY created_at DESC LIMIT 1) AS lastest_queue_id")
    case product.name
    when 'Apache Struts' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Apache Struts')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Tomcat' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Apache Tomcat')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'PHP' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'PHP')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Ruby' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'ruby')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        unless temp_version then
          if queue_app.version =~ /(\d+\.\d+\.\d+)/ then
            temp_version = product.sidfmd_versions.where(:version => Regexp.last_match(1)).first
          end
        end
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Oracle Java SE SDK', 'Oracle Java SE JDK', 'Oracle Java SE JRE' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Oracle Java')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        unless temp_version then
          if queue_app.version =~ /(\d+\.\d+\.\d+)/ then
            temp_version = product.sidfmd_version.where(:version => Regexp.last_match(1)).first
          end
        end
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path, 
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    end
    ret.sort! {|x, y| x[:display_name] <=> y[:display_name] }
    render :layout => false, :json => ret
  end

  # 指定したhtml_idの配列から脆弱性詳細の有無を返す
  # input:  html_ids=JSON.stringify([ 1, 2, 3, ... ]) (POST)
  # output: { "1" => true, "2" => false, }
  # FIXME: under constructions
  def api_htmlids2issue
    htmlids = params[:html_ids]
    ret = {}
    unless htmlids then
      htmlids = []
    end
    unless htmlids.respond_to?(:collect) then
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    begin
      htmlids = htmlids.collect {|x| x.to_i }.sort.uniq
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    if !htmlids.respond_to?(:each) or !htmlids.respond_to?(:'[]') then
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    ret = SidfmdIssue
      .joins(:issue)
      .where(:sidfmd_host_id => if @role_admin then
                                  SidfmdHost
                                    .joins(:sidfmd_categories_hosts)
                                    .joins(:sidfmd_categories)
                                    .where(:flg_delete => false)
                                    .where(:sidfmd_categories => { :is_system => true })
                                    .select(:id)
                                    .distinct
                                else
                                  @user_host_ids
                                end)
      .where(:html_id => htmlids)
      .select(:html_id)
      .group(:html_id)
      .count
    render :layout => false, :json => ret.to_json
  end

end
