#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class Session < ActiveRecord::Base
  # セッションタイムアウトの処理
  # この処理ではcreated_atとupdated_atを1年前に変更して、続くtrim sessionの
  # 処理で実際にセッションを削除する形になっていることに注意。
  # なお、ユーザidの
  def self.expire
    self.transaction do
      expires = self.lock.where('updated_at < ?', Time.current.ago(Sidfm.session_timeout.minutes))
      Rails.logger.info("expire session:")
      expires_count = 0
      expires.each do |expire_session|
        expire_session.update_attribute :updated_at, Time.current.ago(1.year)
        expires_count += 1
        begin
          data = JSON.parse(expire_session.data || '{}')
          if data["value"] and data["value"]["user_id"] then
            user = User.where(:id => data["value"]["user_id"])
            if user.count > 0 then
              user = user.first
              Rails.logger.info(" expire user session: user_id=#{data["value"]["user_id"]}(login: #{user.login})")
            else
              Rails.logger.info(" expire user session: user_id=#{data["value"]["user_id"]}")
            end
          end
        rescue JSON::ParserError
        end
      end
      Rails.logger.info(" expired #{expires_count} sessions")
    end
  end
end
