# -*- coding: utf-8 -*-

class SidfmdRuleAction < ActiveRecord::Base
  # target(操作対象)の値
  # 脆弱性一覧(default)
  TARGET_VULS = 0
  # ホスト一覧
  TARGET_HOSTS = 1
  # 脆弱性同期時に新規登録された脆弱性一覧
  TARGET_ADD = 2
  # 脆弱性同期時に値が変更された脆弱性一覧
  TARGET_MODIFY = 3
  # 脆弱性同期時に対策済みに変更された脆弱性一覧
  TARGET_REMOVE = 4
  # ADD/MODIFY/REMOVE全て（脆弱性一覧）
  TARGET_CHANGE = 5
  TARGETS = [ TARGET_VULS, TARGET_HOSTS, TARGET_ADD, TARGET_MODIFY, TARGET_REMOVE, TARGET_CHANGE ].freeze

  belongs_to :sidfmd_rule

  # どれか一つのみ。
  has_one :sidfmd_rule_a_mail_category, dependent: :delete
  has_one :sidfmd_rule_a_mail_user, dependent: :delete
  has_one :sidfmd_rule_a_status_change, dependent: :delete
  has_one :sidfmd_rule_a_duedate_change, dependent: :delete
  has_one :sidfmd_rule_a_send_syslog, dependent: :delete

  def detail
    case 
    when sidfmd_rule_a_mail_category  then sidfmd_rule_a_mail_category
    when sidfmd_rule_a_mail_user      then sidfmd_rule_a_mail_user
    when sidfmd_rule_a_status_change  then sidfmd_rule_a_status_change
    when sidfmd_rule_a_duedate_change then sidfmd_rule_a_duedate_change
    when sidfmd_rule_a_send_syslog    then sidfmd_rule_a_send_syslog
    else nil
    end
  end

  # ログを追加する
  def log(type, message)
    begin
      self.with_lock do
        self.update_attribute :logs, SidfmdRule.log_append(self.logs, type, SidfmdRule::LOG_ACTION, message)
      end
    rescue ActiveRecord::StatementInvalid => e
      logger.error(e.message)
      if e.backtrace then
        e.backtrace.each do |i|
          logger.error(i)
        end
      end
    end
  end

  def log_info(message)
    log(SidfmdRule::LOG_INFO, message)
  end

  def log_warn(message)
    log(SidfmdRule::LOG_WARN, message)
  end

  def log_err(message)
    log(SidfmdRule::LOG_ERR, message)
  end

end
