# -*- coding: utf-8 -*-
require 'date'
require 'sidfm_controller_support'

# 内部的に使用するAPIをまとめたもの
# 基本的にviewのJavaScriptから利用する。
class SidfmInternalApiController < ApplicationController
  unloadable

  layout false

  self.main_menu = false

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at, :except => [ :api_get_history_vuls, :api_get_journal_hosts ]
  before_action :get_sidfm_roles
  before_action :check_roles
  protect_from_forgery with: :exception

  # 権限チェック
  def check_roles
    @role_admin = @sidfm_roles.include?(:admin)
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    @user_category_ids = if @role_admin then
                           nil
                         else
                           SidfmdCategoriesUser.where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
                     end
    @cache_prefix = @role_admin ? 'internalapi/all' : "internalapi/categories/#{User.current.id}"
  end

  def get_hosts(keyword)
    if keyword and keyword.size >= 2 then
      #keyword = '%' + keyword.gsub(/\%/, '').downcase + '%'
      hosts =
        SidfmdHost
        .enable_hosts
        .select(:id, :hostname, :created_at, :updated_at, :is_agent, :ext_name)
        .where("LOWER(hostname) LIKE ? OR LOWER(ext_name) LIKE ?", '%' + ActiveRecord::Base.send(:sanitize_sql_like, keyword) + '%', '%' + ActiveRecord::Base.send(:sanitize_sql_like, keyword) + '%')
        #.where('LOWER(hostname) LIKE ?', '%' + keyword.gsub(/\%/, '').downcase + '%')
      if !@sidfm_roles.include?(:admin) then
        user_category_ids = SidfmdCategoriesUser
          .where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
        user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
        hosts = hosts
          .where(:id => user_host_ids)
      end
      hosts = hosts
        .limit(100)
    end
  end

  public

  # JavaScriptでのエラーをアプリケーションログに記録する
  def api_logger
    f_params = params.fetch(:params, {}).permit(:msg, :file, :line, :column, :err)
    msg = f_params[:msg]
    file = f_params[:file]
    line = f_params[:line]
    column = f_params[:column]
    err = f_params[:err]
    logger.info('---JavaScript error - begin---')
    logger.info('msg: ' + msg)
    logger.info('file: ' + file)
    logger.info('line: ' + line)
    logger.info('column: ' + column)
    logger.info('err: ' + err.inspect)
    logger.info('---JavaScript error - end---')
    render :layout => false, :json => {}.to_json
  end

  # ホスト名のキーワード検索
  # suggestで使うことを想定。
  def api_hosts
    f_params = params.fetch(:params, {}).permit(:keyword)
=begin    ret = []
    if f_params[:keyword] and f_params[:keyword].size >= 2 then
      hosts =
        SidfmdHost
        .enable_hosts
        .select(:id, :hostname, :created_at, :updated_at, :is_agent)
        .where('LOWER(hostname) LIKE ?', '%' + f_params[:keyword].gsub(/\%/, '').downcase + '%')
      if !@sidfm_roles.include?(:admin) then
        user_category_ids = SidfmdCategoriesUser
          .where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
        user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
        hosts = hosts
          .where(:id => user_host_ids)
      end
      hosts = hosts
        .order(:hostname)
        .limit(100)
    end
=end
    kwd = f_params[:keyword]
    hosts = get_hosts(kwd)
    if hosts then
      get_hit_name = ->(host) { (host.ext_name && host.ext_name.downcase.match(Regexp.new(Regexp.escape(kwd.downcase)))) ? host.ext_name.downcase : host.hostname.downcase }
      hosts = hosts.sort{|a, b| 
        get_hit_name.(a) <=> get_hit_name.(b)
      }
      logger.info(hosts.inspect)
    end
    render :layout => false, :json => hosts.to_json
  end

  # 脆弱性のキーワード検索
  # suggestで使うことを想定。
  def api_vulnerabilities
    f_params = params.fetch(:params, {}).permit(:keyword)
    ret = nil
    if f_params[:keyword] and (f_params[:keyword].size >= 2 or /\A\d\z/ =~ f_params[:keyword]) then
      keyword = f_params[:keyword].gsub(/[\'"]/, '')
      cve_keyword = nil
      # CVE IDっぽい文字列を見つけたら、整形してcve_idの検索に入れる。
      if /\Acve(\D)?(\d{1,4})(\D)?(\d+)?\z/i =~ keyword then
        year = Regexp.last_match(2)
        num = Regexp.last_match(4)
        if year and num then
          cve_keyword = "cve-#{year}-#{num}"
        end
      end
      sidfmid_keyword = nil
      # 数値ならSIDfmIDとして検索
      if /\A(\d+)\z/ =~ keyword then
        sidfmd_keyword = Regexp.last_match(1)
      end
      # KB数値ならMicrosoftKBとして検索
      kb_keyword = nil
      if /\AKB(\d+)\z/i =~ keyword then
        kb_keyword = Regexp.last_match(1).to_i
      end
      part_sql = [ 'LOWER(sidfmd_issues.title) LIKE ?' ]
      part_arg = [ '%' + ActiveRecord::Base.send(:sanitize_sql_like, keyword.downcase) + '%' ]
      if cve_keyword then
        part_sql << 'LOWER(sidfmd_issue_cves.cve_id) LIKE ?'
        part_arg << '%' + ActiveRecord::Base.send(:sanitize_sql_like, cve_keyword) + '%'
      end
      if sidfmd_keyword then
        part_sql << 'sidfmd_issues.html_id = ?'
        part_arg << sidfmd_keyword.to_i
      end
      if kb_keyword then
        part_sql << 'sidfmd_issues.html_id = ?'
        part_arg << -kb_keyword
      end
      logger.info(part_sql.inspect)
      logger.info(part_arg.inspect)
      issues = SidfmdIssue
        .enable_hosts
        .categorized
      if !@sidfm_roles.include?(:admin) then
        user_category_ids = SidfmdCategoriesUser
          .where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
        user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
        issues = issues
          .where(:sidfmd_host_id => user_host_ids)
      end
      issues = issues
        .limit(100)
        .joins("LEFT OUTER JOIN sidfmd_issue_cves ON sidfmd_issue_cves.sidfmd_issue_id=sidfmd_issues.id")
        .where(part_sql.join(' OR '), *part_arg)
        .group(:html_id, :title)
      logger.info(issues.to_sql)
      issues = issues.count
      ret = issues.keys.collect {|x| { :html_id => x[0], :title => x[1] } }
    end
    render :layout => false, :json => ret.to_json
  end

  # 指定された期間に登録された脆弱性のIDとタイトル、日付、関連するホスト数を返す。
  def api_vulnerabilities_period
    f_params = params.fetch(:params, {}).permit(:unit, :num)
    ret = []
    unit = f_params[:unit]
    unless [ 'days', 'weeks', 'months' ].include?(unit) then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    num = f_params[:num]
    unless num then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    num = num.to_i
    unless num > 0 then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end

    issues = SidfmdIssue
      .enable_hosts
      .categorized
    if !@sidfm_roles.include?(:admin) then
        user_category_ids = SidfmdCategoriesUser
          .where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
        user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
        issues = issues
          .where(:sidfmd_host_id => user_host_ids)
    end
    issues = issues
      .joins(:issue)
      .where('issues.created_on >= ?', Time.current.ago(num.send(unit.to_sym)))
      .group(:html_id, :title)
      .count
      .keys

    issues.each do |issue|
      item = {
        :html_id => issue[0],
        :title => issue[1],
        :created_on => '',
        :hosts => [],
      }

      times_hosts = SidfmdIssue
        .joins(:issue)
        .where(:html_id => item[:html_id])
        .group(:created_on, :sidfmd_host_id)
        .count
        .keys

      created_on = Time.current
      times_hosts.each do |th|
        t = th[0]
        sidfmd_host_id = th[1]
        sidfmd_host = SidfmdHost.find(sidfmd_host_id)
        created_on = t if created_on > t
        item[:hosts] << {
          :id => sidfmd_host_id,
          :hostname => sidfmd_host.hostname,
        }
      end
      item[:created_on] = created_on.localtime.strftime('%Y/%m/%d')
      item[:hosts].sort! {|x, y| x[:id] <=> y[:id] }
      ret << item
    end

    render :layout => false, :json => ret.to_json
  end

  # 指定された期間に登録され、かつ、指定の危険度の脆弱性を抽出し、
  # html_idとタイトル、日付、関連するホスト数を返す。
  def api_vulnerabilities_risk_period
    f_params = params.fetch(:params, {}).permit(:risk_type, :risk_level, :unit, :num)
    ret = []
    risk_type = f_params[:risk_type]
    unless [ 'sri', 'cvss' ].include?(risk_type) then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    risk_level = f_params[:risk_level]
    if
        !(risk_type == 'sri' and
          SidfmdIssueSri::VALUES.include?(risk_level.to_i)) and
        !(risk_type == 'cvss' and
          SidfmdIssueCve::LABELS.include?(risk_level.to_sym))
    then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    unit = f_params[:unit]
    unless [ 'days', 'weeks', 'months' ].include?(unit) then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    num = f_params[:num]
    unless num then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    num = num.to_i
    unless num > 0 then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end

    case risk_type
    when 'sri' then
      issues = SidfmdIssue
        .enable_hosts
        .categorized
        .joins(:issue)
        .where('issues.created_on >= ?', Time.current.ago(num.send(unit.to_sym)))
        .group(:html_id, :title)
        .count
        .keys
      issues.each do |issue|
        item = {
          :html_id => issue[0],
          :title => issue[1],
          :created_on => '',
          :hosts => [],
        }
        times_hosts = SidfmdIssue
          .joins(:issue)
          .where(:html_id => item[:html_id])
          .group(:created_on, :sidfmd_host_id)
          .count
          .keys
        created_on = Time.current
        times_hosts.each do |th|
          t = th[0]
          sidfmd_host_id = th[1]
          sidfmd_host = SidfmdHost.find(sidfmd_host_id)
          created_on = t if created_on > t
          sri = SidfmdIssueSri
            .joins(:sidfmd_issue)
            .where(:sidfmd_issues => { :sidfmd_host_id => sidfmd_host_id })
            .where(:category_id => sidfmd_host.system_category.id)
            .first
          risk_id = if sri then
                      sri.risk_id
                    else
                      0
                    end
          item[:hosts] << {
            :id => sidfmd_host_id,
            :hostname => sidfmd_host.hostname,
            :created_on => t,
            :risk_id => risk_id,
          }
        end
        item[:created_on] = created_on.localtime.strftime('%Y/%m/%d')
        item[:hosts].sort! {|x, y| x[:id] <=> y[:id] }
        ret << item
      end
      risk_level_i = risk_level.to_i
      logger.info("before #{ret.size}")
      logger.info(ret.inspect)
      ret.delete_if {|x|
        x[:hosts].delete_if {|y| y[:risk_id] != risk_level_i }
        x[:hosts].size == 0
      }
      logger.info("after #{ret.size}")
      logger.info(ret.inspect)
    when 'cvss' then
      issues = SidfmdIssue
        .enable_hosts
        .categorized
        .joins(:issue)
        .where('issues.created_on >= ?', Time.current.ago(num.send(unit.to_sym)))
        .group(:html_id, :title)
        .count
        .keys
      issues.each do |issue|
        item = {
          :html_id => issue[0],
          :title => issue[1],
          :created_on => '',
          :hosts => [],
          :temp_score => 0.0,
        }

        cves = SidfmdIssue
          .where(:html_id => item[:html_id])
          .first
          .sidfmd_issue_cves
          .select('MAX(COALESCE(c3_base_score,c2_base_score)) AS max_score')
        item[:temp_score] = cves[0].max_score

        times_hosts = SidfmdIssue
          .joins(:issue)
          .where(:html_id => item[:html_id])
          .group(:created_on, :sidfmd_host_id)
          .count
          .keys

        created_on = Time.current
        times_hosts.each do |th|
          t = th[0]
          sidfmd_host_id = th[1]
          sidfmd_host = SidfmdHost.find(sidfmd_host_id)
          created_on = t if created_on > t
          item[:hosts] << {
            :id => sidfmd_host_id,
            :hostname => sidfmd_host.hostname,
          }
        end
        item[:created_on] = created_on.localtime.strftime('%Y/%m/%d')
        item[:hosts].sort! {|x, y| x[:id] <=> y[:id] }
        ret << item
      end
      case risk_level
      when 'notreg' then
        ret.delete_if {|x| !x[:temp_score].nil? }
      when 'd4' then
        ret.delete_if {|x| x[:temp_score].nil? or x[:temp_score] < 9.0 }
      when 'd3' then
        ret.delete_if {|x| x[:temp_score].nil? or
          (x[:temp_score] >= 9.0 or x[:temp_score] < 7.0 ) }
      when 'd2' then
        ret.delete_if {|x| x[:temp_score].nil? or
          (x[:temp_score] >= 7.0 or x[:temp_score] < 4.0 ) }
      when 'd1' then
        ret.delete_if {|x| x[:temp_score].nil? or x[:temp_score] >= 4.0 }
      end
      ret.each do |i|
        i.delete(:temp_score)
      end
    end

    render :layout => false, :json => ret.to_json
  end

  # sidfmd_hosts.idの配列から、対象ホスト、関連するカテゴリ、
  # カテゴリに関連づけられたユーザ、ユーザに関連づけられたメール
  # アドレスの情報を返す。
  def api_hosts_categories_users_emails
    f_params = params.fetch(:params, {}).permit(:hostids)
    hostids = JSON.parse(f_params[:hostids])

    hosts = SidfmdHost.enable_hosts.where(:id => hostids).select(:id, :hostname, :flg_delete, :is_agent, :ext_name)

    categories_hosts = SidfmdCategoriesHost.where(:sidfmd_host_id => hosts.collect {|x| x.id })

    if @role_admin then
      categories = SidfmdCategory.joins(:sidfmd_categories_hosts).where(:sidfmd_categories_hosts => { :sidfmd_host_id => hosts.collect {|x| x.id } }).select(:id, :name, :desc, :user_id, :is_system)
    else
      categories = SidfmdCategory.where(:id => @user_category_ids).joins(:sidfmd_categories_hosts).where(:sidfmd_categories_hosts => { :sidfmd_host_id => hosts.collect {|x| x.id } }).select(:id, :name, :desc, :user_id, :is_system)
    end
    categories_users = SidfmdCategoriesUser.where(:sidfmd_category_id => categories.collect {|x| x.id })

    users = User.where(:id => categories_users.collect {|x| x.user_id }).select(:id, :login, :firstname, :lastname, :admin, :status, :language, :type, :identity_url, :mail_notification)


    emails = EmailAddress.where(:user_id => users.collect {|x| x.id }).select(:id, :user_id, :address, :is_default, :notify)

    r = {
      :hosts => hosts,
      :categories => categories.uniq,
      :categories_users => categories_users,
      :categories_hosts => categories_hosts,
      :users => users,
      :emails => emails,
    }

    render :layout => false, :json => r.to_json
  end

  # sidfmd_hosts.idの配列から、対象ホストとそれに関連づけられたissuesの
  # due_dateの一覧を返す。
  def api_hosts_due_dates
    f_params = params.fetch(:params, {}).permit(:hostids)
    hostids = JSON.parse(f_params[:hostids])
    if !@role_admin then
      if (hostids - @user_host_ids).size > 0 then
        r = {
          :hosts => [],
          :issues => [],
          :statuses => {},
        }
        render :layout => false, :json => r
      end
    end
    hosts = SidfmdHost.enable_hosts.where(:id => hostids).select(:id, :hostname, :flg_delete, :is_agent)
    issues = Issue.joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => hostids }).where(:status_id => [ 1, 2 ]).select(:sidfmd_host_id, :due_date).distinct

    r = {
      :hosts => hosts,
      :issues => issues,
    }
    
    render :layout => false, :json => r
  end


  # sidfmd_hosts.idの配列から、対象ホストとそれに関連づけられたissuesの
  # status_idの一覧を返す。
  def api_hosts_statuses
    f_params = params.fetch(:params, {}).permit(:hostids)
    hostids = JSON.parse(f_params[:hostids])

    if !@role_admin then
      if (hostids - @user_host_ids).size > 0 then
        r = {
          :hosts => [],
          :issues => [],
          :statuses => {},
        }
        render :layout => false, :json => r
      end
    end
    hosts = SidfmdHost.enable_hosts.where(:id => hostids).order(:hostname).select(:id, :hostname, :flg_delete, :is_agent)
    issues = Issue.joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => hostids }).select(:sidfmd_host_id, :status_id).distinct
    statuses = {}
    hosts.each do |host|
      statuses[host.id] = host.status_count
    end

    r = {
      :hosts => hosts,
      :issues => issues,
      :statuses => statuses,
    }
    
    render :layout => false, :json => r
  end

  # html_idの配列から、関連づけられた脆弱性とissuesのstatus_idの一覧を返す。
  # 利用:
  #   sidfm_vulnerbilities
  def api_htmlids_statuses
    f_params = params.fetch(:params, {}).permit(:htmlids)
    htmlids = JSON.parse(f_params[:htmlids])
    ret = {}

    issues = SidfmdIssue
      .joins(:sidfmd_host)
      .select(:html_id, :title)
      .select("ARRAY_TO_STRING(ARRAY(
SELECT
 CAST(issues.status_id AS TEXT) || ':' || CAST(COUNT(issues.status_id) AS TEXT)
FROM sidfmd_issues AS t_sidfmd_issues
INNER JOIN issues
 ON t_sidfmd_issues.issue_id=issues.id
WHERE t_sidfmd_issues.html_id=sidfmd_issues.html_id
      #{@role_admin ? '' : "AND t_sidfmd_issues.sidfmd_host_id IN (" + @user_host_ids.join(',') + ")" }
GROUP BY issues.status_id
ORDER BY issues.status_id
),',') AS status_ids")
      .where(:html_id => htmlids)
      .where(:sidfmd_hosts => { :flg_delete => false })
    unless @role_admin then
      issues = issues
        .where(:sidfmd_host_id => @user_host_ids)
    end
    issues = issues
      .group(:html_id, :title)
    status2color = [ '', :danger, :warning, :info, :success ]
    issues.each do |issue|
      unless ret.has_key?(issue[:html_id]) then
        ret[issue[:html_id]] = {
          :title => issue[:title],
          :status => [],
        }
        status_ids = issue[:status_ids] ? issue[:status_ids].split(/,/) : []
        status_ids.each do |status_id|
          item = status_id.split(/:/)
          id = item.first.to_i
          cnt = item.last.to_i
          ret[issue[:html_id]][:status] << {
            :id => issue[:html_id],
            :value => cnt,
            :name => SidfmdIssue::STATUS_NAMES[id],
            :color => ('status_' + SidfmdIssue::STATUS_LABELS[id].to_s).to_sym,
            :placement => 'top',
          }
        end

      end
    end

    render :layout => false, :json => ret.to_json
  end

  # html_idの配列から、関連づけられた脆弱性とissuesのdue_datesの一覧を返す。
  # 利用:
  #   sidfm_vulnerbilities
  def api_htmlids_due_dates
    f_params = params.fetch(:params, {}).permit(:htmlids)
    htmlids = JSON.parse(f_params[:htmlids])

    issues = SidfmdIssue
      .joins(:sidfmd_host)
      .joins(:issue)
      .where(:sidfmd_hosts => { :flg_delete => false })
      .where(:html_id => htmlids)
    unless @role_admin then
      issues = issues
        .where(:sidfmd_host_id => @user_host_ids)
    end
    issues = issues
      .group(:html_id, :title, :due_date)
      .count
      .keys

    ret = {}

    issues.each do |issue|
      html_id = issue[0]
      title = issue[1]
      due_date = issue[2] ? issue[2].strftime('%Y/%m/%d') : nil
      if ret.has_key?(html_id) then
        ret[html_id][:due_dates] << due_date
      else
        ret[html_id] = {
          :title => title,
          :due_dates => [ due_date ],
        }
      end
    end
    ret.keys.each do |html_id|
      ret[html_id][:due_dates].uniq!
      ret[html_id][:due_dates].sort! {|x, y| (x.nil? ? '' : x) <=> (y.nil? ? '' : y) }
    end

    render :layout => false, :json => ret.to_json
  end

  # sidfmd_host_idとhtml_idの配列から、関連付けられたissuesのdue_dateを返す。
  def api_host_htmlids_due_dates
    f_params = params.fetch(:params, {}).permit(:hostid, :htmlids)
    unless f_params[:hostid] or f_params[:htmlids] then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    hostid = f_params[:hostid].to_i
    htmlids = JSON.parse(f_params[:htmlids])

    issues = SidfmdIssue
      .enable_hosts
      .categorized
      .joins(:issue)
      .where(:sidfmd_host_id => hostid)
      .where(:html_id => htmlids)
      .group(:html_id, :title, :due_date)
      .count
      .keys

    ret = []

    issues.each do |issue|
      ret << {
        :html_id => issue[0],
        :title => issue[1],
        :due_date => issue[2] ? issue[2].strftime('%Y/%m/%d') : issue[2],
      }
    end

    render :layout => false, :json => ret.to_json
  end

  # sidfmd_host_idとhtml_idの配列から、関連付けられたissuesのstatus_idの一覧を返す。
  def api_host_htmlids_statuses
    f_params = params.fetch(:params, {}).permit(:hostid, :htmlids)
    unless f_params[:hostid] or f_params[:htmlids] then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    hostid = f_params[:hostid].to_i
    htmlids = JSON.parse(f_params[:htmlids])

    issues = SidfmdIssue
      .enable_hosts
      .categorized
      .joins(:issue)
      .where(:sidfmd_host_id => hostid)
      .where(:html_id => htmlids)
      .group(:html_id, :title, :status_id)
      .count
      .keys

    ret = []

    issues.each do |issue|
      ret << {
        :html_id => issue[0],
        :title => issue[1],
        :status_id => issue[2],
      }
    end

    render :layout => false, :json => ret.to_json
  end

  # html_idとsidfmd_hosts.idの配列から、対象ホストとそれに関連づけられ
  # たissuesのstatus_idの一覧を返す。
  def api_htmlid_hosts_statuses
    f_params = params.fetch(:params, {}).permit(:htmlid, :hostids)
    unless f_params[:htmlid] or f_params[:hostids] then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    htmlid = f_params[:htmlid].to_i
    if htmlid == 0 then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    hostids = nil
    begin
      hostids = JSON.parse(f_params[:hostids])
    rescue JSON::ParserError => e
      log_exception(e)
    end
    unless hostids then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end

    hosts = SidfmdHost
      .enable_hosts
      .categorized
      .joins(:sidfmd_issues)
      .joins(:sidfmd_issues => [ :issue ])
      .where(:id => hostids)
      .where(:sidfmd_issues => { :html_id => htmlid })
      .select(:id, :hostname, :status_id)

    ret = []
    hosts.each do |host|
      ret << {
        :id => host[:id],
        :hostname => host[:hostname],
        :status_id => host[:status_id],
      }
    end
    render :layout => false, :json => ret.to_json
  end

  # htmlidとsidfmd_hosts.idの配列から、対象ホストとそれに関連づけられ
  # たissuesのdue_dateの一覧を返す。
  def api_htmlid_hosts_due_dates
    f_params = params.fetch(:params, {}).permit(:htmlid, :hostids)
    unless f_params[:htmlid] or f_params[:hostids] then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    htmlid = f_params[:htmlid].to_i
    if htmlid == 0 then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    hostids = nil
    begin
      hostids = JSON.parse(f_params[:hostids])
    rescue JSON::ParserError => e
      log_exception(e)
    end
    unless hostids then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end

    hosts = SidfmdHost
      .enable_hosts
      .categorized
      .joins(:sidfmd_issues)
      .joins(:sidfmd_issues => [ :issue ])
      .where(:id => hostids)
      .where(:sidfmd_issues => { :html_id => htmlid })
      .select(:id, :hostname, :due_date)

    ret = []
    hosts.each do |host|
      ret << {
        :id => host[:id],
        :hostname => host[:hostname],
        :due_date => host[:due_date] ? host[:due_date].strftime('%Y/%m/%d') : host[:due_date],
      }
    end

    render :layout => false, :json => ret.to_json
  end


  # html_idの配列から、関連づけられた脆弱性と脆弱性に関連したホストの内、
  # ユーザが定義されたカテゴリに属するユーザの情報を返す。
  # 利用:
  #   sidfm_vulnerbilities
  def api_htmlids_emails
    f_params = params.fetch(:params, {}).permit(:htmlids)
    htmlids = JSON.parse(f_params[:htmlids])
    ret = { :users => {}, :htmls => [] }
    htmls = SidfmdIssue
      .joins(:sidfmd_host)
      .select(:html_id, :title)
      .select("ARRAY_TO_STRING(ARRAY(
SELECT DISTINCT(t_sidfmd_hosts.id)
FROM sidfmd_hosts AS t_sidfmd_hosts
INNER JOIN sidfmd_issues AS t_sidfmd_issues
 ON t_sidfmd_issues.sidfmd_host_id=t_sidfmd_hosts.id
WHERE sidfmd_issues.html_id=t_sidfmd_issues.html_id AND
     t_sidfmd_hosts.flg_delete IS FALSE
     #{@role_admin ? '' : 'AND t_sidfmd_hosts.id IN(' + @user_host_ids.join(',') + ')'}
ORDER BY t_sidfmd_hosts.id
),',') AS hostids")
      .where(:sidfmd_hosts => { :flg_delete => false })
      .where(:sidfmd_host_id => SidfmdCategoriesHost.joins(:sidfmd_category).where(:sidfmd_categories => { :is_system => false }).select(:sidfmd_host_id).distinct)
    unless @role_admin then
      htmls = htmls.where(:sidfmd_host_id => @user_host_ids)
    end
    htmls = htmls
      .where(:html_id => htmlids)
      .group(:html_id, :title)
    hostids = []
    htmls.each do |html|
      ret[:htmls] << {
        :id => html[:html_id],
        :title => html[:title],
      }
      hostids += html[:hostids].split(/,/).collect {|x| x.to_i }
    end
    hostids.uniq!
    hostids.compact!
    hostids &= @user_host_ids unless @role_admin

    categories = SidfmdCategoriesHost
      .joins(:sidfmd_category)
      .where(:sidfmd_host_id => hostids)
      .where(:sidfmd_categories => { :is_system => false })
    unless @role_admin then
      categories = categories
        .where(:sidfmd_category_id => @user_category_ids)
    end
    categories = categories
      .select(:sidfmd_category_id)
      .distinct

    users = SidfmdCategoriesUser
      .joins(:sidfmd_category)
      .joins(:user)
      .select(:user_id)
      .select(:login)
      .select(:firstname)
      .select(:lastname)
      .select(:sidfmd_category_id)
      .select(:name)
      .where(:sidfmd_categories => { :is_system => false })
      .where(:sidfmd_category_id => categories)
    unless @role_admin then
      users = users
        .where(:sidfmd_category_id => @user_category_ids)
    end
    users.each do |user|
      unless ret[:users].has_key?(user[:user_id]) then
        ret[:users][user[:user_id]] = {
          :login => user[:login],
          :firstname => user[:firstname],
          :lastname => user[:lastname],
          :emails => user.user.email_addresses.collect {|x|
            { :id => x.id, :address => x.address, }
          } ,
          :categories => [],
          :hosts => [],
        }
      end
      if @role_admin or @user_category_ids.include?(user[:sidfmd_category_id]) then
        ret[:users][user[:user_id]][:categories] << { :id => user[:sidfmd_category_id], :name => user[:name] }
        ret[:users][user[:user_id]][:categories].uniq!
        cat_hosts = SidfmdCategoriesHost
          .joins(:sidfmd_host)
          .select(:sidfmd_host_id)
          .select(:hostname)
          .select(:ext_name)
          .where(:sidfmd_hosts => { :flg_delete => false })
          .where(:sidfmd_host_id => hostids)
          .where(:sidfmd_category_id => user[:sidfmd_category_id])
          .where(:sidfmd_category_id => categories)
          .group(:sidfmd_host_id, :hostname, :ext_name)
        cat_hosts.each do |host|
          ret[:users][user[:user_id]][:hosts] << { 
            :id => host[:sidfmd_host_id], 
            :hostname => host[:hostname], 
            :ext_name => host[:ext_name]
          }
        end
        ret[:users][user[:user_id]][:categories].uniq!
        ret[:users][user[:user_id]][:hosts].uniq!
      end
    end

    render :layout => false, :json => ret.to_json
  end

  # html_idの配列から、関連づけられた脆弱性と脆弱性に関連したホストの内、
  # ユーザが定義されたカテゴリに属するユーザの情報を返す。
  # api_htmlids_emailsとほぼ同じ処理だが、ホスト詳細画面専用で、ホストが
  # 固定されているため、カテゴリもホストに付与されたものが採用される。
  # 利用:
  #   sidfm_host
  def api_host_htmlids_emails
    f_params = params.fetch(:params, {}).permit(:htmlids, :hostid)
    htmlids = JSON.parse(f_params[:htmlids])
    hostid = f_params[:hostid].to_i
    host = SidfmdHost.find(hostid)
    unless host then
      render :layout => false, :json => '{}'.to_json, :status => 500
      return
    end

    ret = { :users => {}, :htmls => [] }
    htmls = SidfmdIssue
      .joins(:sidfmd_host)
      .select(:html_id, :title)
      .select("ARRAY_TO_STRING(ARRAY(
SELECT DISTINCT(t_sidfmd_hosts.id)
FROM sidfmd_hosts AS t_sidfmd_hosts
INNER JOIN sidfmd_issues AS t_sidfmd_issues
 ON t_sidfmd_issues.sidfmd_host_id=t_sidfmd_hosts.id
WHERE sidfmd_issues.html_id=t_sidfmd_issues.html_id AND
     t_sidfmd_hosts.flg_delete IS FALSE
     #{@role_admin ? '' : 'AND t_sidfmd_hosts.id IN(' + @user_host_ids.join(',') + ')'}
ORDER BY t_sidfmd_hosts.id
),',') AS hostids")
      .where(:sidfmd_hosts => { :flg_delete => false })
      .where(:sidfmd_host_id => SidfmdCategoriesHost.joins(:sidfmd_category).where(:sidfmd_categories => { :is_system => false }).select(:sidfmd_host_id).distinct)
    unless @role_admin then
      htmls = htmls.where(:sidfmd_host_id => @user_host_ids)
    end
    htmls = htmls
      .where(:html_id => htmlids)
      .group(:html_id, :title)
    
    hostids = [ hostid ]
    htmls.each do |html|
      ret[:htmls] << {
        :id => html[:html_id],
        :title => html[:title],
      }
    end
    hostids &= @user_host_ids unless @role_admin

    categories = SidfmdCategoriesHost
      .joins(:sidfmd_category)
      .where(:sidfmd_host_id => hostids)
      .where(:sidfmd_categories => { :is_system => false })
    unless @role_admin then
      categories = categories
        .where(:sidfmd_category_id => @user_category_ids)
    end
    categories = categories
      .select(:sidfmd_category_id)
      .distinct

    users = SidfmdCategoriesUser
      .joins(:sidfmd_category)
      .joins(:user)
      .select(:user_id)
      .select(:login)
      .select(:firstname)
      .select(:lastname)
      .select(:sidfmd_category_id)
      .select(:name)
      .where(:sidfmd_categories => { :is_system => false })
      .where(:sidfmd_category_id => categories)
    unless @role_admin then
      users = users
        .where(:sidfmd_category_id => @user_category_ids)
    end
    users.each do |user|
      unless ret[:users].has_key?(user[:user_id]) then
        ret[:users][user[:user_id]] = {
          :login => user[:login],
          :firstname => user[:firstname],
          :lastname => user[:lastname],
          :emails => user.user.email_addresses.collect {|x|
            { :id => x.id, :address => x.address, }
          } ,
          :categories => [],
          :hosts => [],
        }
      end
      if @role_admin or @user_category_ids.include?(user[:sidfmd_category_id]) then
        ret[:users][user[:user_id]][:categories] << { :id => user[:sidfmd_category_id], :name => user[:name] }
        ret[:users][user[:user_id]][:categories].uniq!
        cat_hosts = SidfmdCategoriesHost
          .joins(:sidfmd_host)
          .select(:sidfmd_host_id)
          .select(:hostname)
          .select(:ext_name)
          .where(:sidfmd_hosts => { :flg_delete => false })
          .where(:sidfmd_host_id => hostids)
          .where(:sidfmd_category_id => user[:sidfmd_category_id])
          .where(:sidfmd_category_id => categories)
          .group(:sidfmd_host_id, :hostname, :ext_name)
        cat_hosts.each do |host|
          ret[:users][user[:user_id]][:hosts] << { 
            :id => host[:sidfmd_host_id],
            :hostname => host[:hostname],
            :ext_name => host[:ext_name]
          }
        end
        ret[:users][user[:user_id]][:categories].uniq!
        ret[:users][user[:user_id]][:hosts].uniq!
      end
    end

    render :layout => false, :json => ret.to_json
  end

  # セキュリティニュース情報（SIDfm情報）で使用するAPI
  def api_info_find_news_topics
    f_params = params.permit(:begin_date, :end_date, :search_text, :limit, :offset, kinds: [])
    ret = { 'topics' => [] }
    Sidfm.sidapi do |o|
      ret = o.find_news_topics(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報（SIDfm情報）で使用するAPI
  def api_info_find_products
    f_params = params.permit(:search_text)
    ret = { 'products' => [] }
    Sidfm.sidapi do |o|
      ret = o.find_products(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報（SIDfm情報）で使用するAPI
  def api_info_recent_product_eols
    ret = { 'product_eols' => [] }
    Sidfm.sidapi do |o|
      ret = o.recent_product_eols
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報（SIDfm情報）で使用するAPI
  def api_info_products
    # FIXME: datetime_typeはobsolete
    f_params = params.permit(:datetime_type, :time_field, :begin_datetime, :end_datetime)
    ret = { 'products' => [] }
    Sidfm.sidapi do |o|
      ret = o.products2(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_product
    f_params = params.permit(:id)
    ret = { 'product' => [] }
    Sidfm.sidapi do |o|
      ret = o.product(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_product_eols
    f_params = params.permit(:product_id)
    ret = { 'product_eols' => [] }
    Sidfm.sidapi do |o|
      ret = o.product_eols(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_risks
    ret = { 'risks' => [] }
    Sidfm.sidapi do |o|
      ret = o.risks
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_areas
    ret = { 'areas' => [] }
    Sidfm.sidapi do |o|
      ret = o.areas
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_product_vulns
    f_params = params.permit(:product_id, :begin_datetime, :end_datetime)
    ret = { 'product_vulns' => [] }
    Sidfm.sidapi do |o|
      ret = o.product_vulns(params)
    end
    render :layout => false, :json => ret.to_json
  end

  # セキュリティホール情報（SIDfm情報）で使用するAPI
  def api_info_find_vulns
    f_params = params.permit(:begin_date, :end_date, :cvss_base_score, :search_text, :limit, :offset)
    ret = { 'vulns' => [] }
    Sidfm.sidapi do |o|
      ret = o.find_vulns(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # セキュリティホール情報詳細（SIDfm情報）で使用するAPI
  def api_info_sri_categories
    ret = { 'sri_categories' => [] }
    Sidfm.sidapi do |o|
      ret = o.sri_categories
    end
    render :layout => false, :json => ret.to_json
  end

  # セキュリティホール情報詳細（SIDfm情報）で使用するAPI
  def api_info_vuln
    f_params = params.permit(:id)
    ret = { 'vuln' => {} }
    Sidfm.sidapi do |o|
      ret = o.vuln2(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # KBでセキュリティホール情報を検索し、その結果を返す。
  def api_find_vulns_by_mskb
    ret = []
    f_params = params.fetch(:params, {}).permit(:kb)
    kb = (f_params[:kb] || '0').to_i
    Sidfm.sidapi do |o|
      ret = o.find_vulns_by_mskb(kb)
    end
    render :layout => false, :json => ret.to_json
  end

  # ホスト名から、アクセス可能なホストか否かを返す。
  # ホスト管理画面で使用
  def api_check_hostname
    ret = { :exist => false, :accessable => false }
    f_params = params.fetch(:params, {}).permit(:hostname)
    begin
      host =
        SidfmdHost
        .where(:flg_delete => false)
        .where(:hostname => (f_params[:hostname] || '')).first
      ret[:exist] = !host.nil?
      if @sidfm_roles.include?(:admin) then
        ret[:accessable] = true
      elsif @sidfm_roles.include?(:use) then
        if ret[:exist] then
          host =
            SidfmdHost
            .joins(:sidfmd_categories_hosts)
            .where(:flg_delete => false)
            .where(:sidfmd_categories_hosts =>
                   { :sidfmd_category_id =>
                     SidfmdCategoriesUser
                       .where(:user_id => User.current.id)
                       .select(:sidfmd_category_id)
                       .distinct.collect {|x| x[:sidfmd_category_id] } })
            .where(:hostname => (f_params[:hostname] || '')).first
          ret[:accessable] = !host.nil?
        else
          ret[:accessable] = true
        end
      end
    rescue Exception => e
      log_exception(e)
    end

    render :layout => false, :json => ret.to_json
  end
  
  #履歴表示のためのデータ取得
  def api_get_journals
    f_params = params.permit(:html_id, :host_id)
    data = []
    html_id = f_params[:html_id].to_i
    host_id = f_params[:host_id].to_i
    issue = SidfmdIssue.where(:html_id => html_id).where(:sidfmd_host_id => host_id)
    if issue.length > 0 then
      issue_id = issue.map{|i| i.issue_id}
      journal = Journal.where(:journalized_id => issue_id).map{|j| [j.id, j.user_id, j.notes, j.created_on.strftime('%Y/%m/%d %H:%M:%S')]}
      users = {}
      User.where(:id => journal.map{|j| j[1]}).each do |user|
        users[user.id] = if user.anonymous? then
                           ['', '削除済み', 'ユーザ']
                         else
                           [user.mail, user.lastname, user.firstname]
                         end
      end
      journal.each  do |j|
        j_details = []
        get_j_details = JournalDetail.where(:journal_id => j[0])
        if get_j_details.length > 0 then
          get_j_details.order(:id).each do |detail|
            case detail[:prop_key] 
            when 'status_id' then
              j_detail = ['status', SidfmdIssue.status_id_to_s(detail[:old_value].to_i), SidfmdIssue.status_id_to_s(detail[:value].to_i)]
            when 'due_date' then
              j_detail = ['due_date', SidfmdIssue.due_date_to_s(detail[:old_value]), SidfmdIssue.due_date_to_s(detail[:value])]
            else
              j_detail = ['other']
            end
            j_details.push(j_detail)
          end
        end
        j.push(users[j[1]])
        j.push(j_details)
        data.push(j)
      end
    end
    render :layout => false, :json => data.to_json
  end

  #履歴のコメント投稿時に履歴作成
  def api_post_journal_comment
    f_params =  params.permit(:html_id, :host_id, :comment, :user_id)
    html_id = f_params[:html_id].to_i 
    host_id = f_params[:host_id].to_i
    comment = f_params[:comment]
    user_id = f_params[:user_id].to_i
    pass_flag = 0
    if !@sidfm_roles.include?(:use) then
      render status: 401, json: { status: 401, message: 'Unauthorized' } and return
    end
    if @sidfm_roles.include?(:admin) then
      pass_flag = 1
    else
      if !@sidfm_roles.include?(:admin) then
        host_categories = SidfmdHost.find(host_id).sidfmd_categories.map{|c|c.id}
        user_categories = User.current.sidfmd_categories.map{|c|c.id}
        user_categories.each do |c|
          if host_categories.include?(c) then
            pass_flag = 1
            break
          end
        end
      end
    end
    if pass_flag == 1 then
      session[:confirming_journal] = nil
      begin 
        issue_id = SidfmdIssue.where(:html_id => html_id).where(:sidfmd_host_id => host_id).first
        if issue_id then
          Journal.transaction do
            j_comment = Journal.new(journalized_id: issue_id.issue_id, journalized_type: "Issue", user_id: user_id, notes: comment) 
            j_comment.save
          end
        end
          created_journal = Journal.last
          journal_content = [created_journal.created_on.strftime('%Y/%m/%d %H:%M:%S'), created_journal.notes]
          user = User.find(user_id)
          user_email = user.mail
          user_lastname = user.lastname
          user_firstname = user.firstname
          journal_content.push([user_email, user_lastname, user_firstname])
      rescue Exception => e
        log_exception(e)
      end
      render :layout => false, :json => journal_content
    else
      render status: 401, json: { status: 401, message: 'Unauthorized' }
    end
  end

  #ページロード時の履歴作成
  def api_get_journal_hosts
    if @sidfm_roles.include?(:admin) then 
      category_ids = SidfmdCategory.all.map{|c|c.id}
    else
      category_ids = @user_category_ids
    end
    if !session['first_history_time'] then
      session['first_history_time'] = Time.current.to_time
    end
    host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => category_ids).map{|s| s.sidfmd_host_id}.uniq
    journals = Journal.joins(:issue)
                      .joins('LEFT OUTER JOIN sidfmd_issues ON sidfmd_issues.issue_id = issues.id')
                      .where('sidfmd_issues.sidfmd_host_id IN (?)', host_ids)
                      .where(:created_on => NOTIFY_HISTORY_DATE..Time.current.to_time)
    #履歴のリンクからぺーじを遷移した時に、確認済みの履歴にその履歴を追加
    session[:confirmed_journal_ids] = [] unless session[:confirmed_journal_ids]
    if session[:confirming_journal] then
      hostid = session[:confirming_journal][:host_id]
      htmlid = session[:confirming_journal][:html_id]
      j_ids = journals.where('sidfmd_issues.sidfmd_host_id = ?', hostid)
                      .where('sidfmd_issues.html_id = ?', htmlid)
                      .map(&:id)
      session[:confirmed_journal_ids].push(*j_ids).uniq!
      session[:confirming_journal] = nil
    end
    #確認済み以外の新しい履歴を取得して、新しい履歴に関係ないホストには新着マークを付けない
    #確認済みの履歴のホストは新しい履歴から除く
    new_journals = journals.where(:created_on => session['first_history_time']..Time.current.to_time)
                           .where.not(:id => session[:confirmed_journal_ids])
    new_journals_amount = new_journals.size
 
    new_journals_hostids = new_journals.distinct('sidfmd_issues.sidfmd_host_id')
                                       .select('sidfmd_issues.sidfmd_host_id AS issue_host_id')
                                       .map{|j| j.issue_host_id}.uniq
    #１週間の履歴に関連するホスト
    journals_hostids = journals.distinct('sidfmd_issues.issue_id')
                               .select(
                                 'sidfmd_issues.sidfmd_host_id AS hostid',
                                 'sidfmd_issues.html_id AS htmlid',
                               )
    hosts = {}
    journals_hostids.each do |row|
      host_id = row.hostid
      target = hosts[host_id]
      if target then
        hosts[host_id][:total_vuls] += 1
      else 
        host = SidfmdHost.find(host_id)
        hostname = host.hostname
        ext_name = host.ext_name
        hosts[host_id] = { 
          :host_id => host_id, 
          :hostname => hostname,
          :ext_name => ext_name,
          :total_vuls => 1,
        }
      end
    end
    new_journals_hostids.each do |host_id|
      hosts[host_id][:is_new] = true
    end
    sorted_hosts = hosts.values.sort{|a, b| 
      r = (a[:is_new] ? 1 : 10) <=> (b[:is_new] ? 1 : 10)
      if r == 0 then
        r = (a[:ext_name] ? a[:ext_name] : a[:hostname]) <=> (b[:ext_name] ? b[:ext_name] : b[:hostname])
        if r == 0 then
          a[:hostname] <=> b[:hostname]
        end 
      end
      r
    }
    render :layout => false, :json => { :hosts => sorted_hosts, :new_journals_amount => new_journals_amount }
  end

  def api_get_history_vuls
    f_params = params.permit(:host_id)
    host_id = f_params[:host_id].to_i
    if @role_admin || @user_host_ids.include?(host_id) then
      journals = Journal.joins(:issue)
                        .joins('LEFT OUTER JOIN sidfmd_issues ON sidfmd_issues.issue_id = issues.id')
                        .where('sidfmd_issues.sidfmd_host_id = ?', host_id)
                        .where(:created_on => NOTIFY_HISTORY_DATE..Time.current.to_time)
      new_journal_html_ids = journals.where(:created_on => session['first_history_time']..Time.current.to_time)
                                     .where.not(:id => session[:confirmed_journal_ids])
                                     .select('sidfmd_issues.html_id')
                                     .map{|i| i.html_id}
      target_vuls = journals.distinct('sidfmd_issues.issue_id')
                            .select('sidfmd_issues.html_id', 'sidfmd_issues.title')
      vuls = {}
      target_vuls.each do |vul|
        vuls[vul.html_id] = {:html_id => vul.html_id, :title => vul.title}
      end 
      new_journal_html_ids.each do |html_id|
        vuls[html_id][:is_new] = true
      end
      sorted_vuls = vuls.values.sort{|a, b|
        r = (a[:is_new] ? 1 : 10) <=> (b[:is_new] ? 1: 10)
        if r == 0 then
          r = a[:title] <=> b[:title]
          if r == 0 then
            r = a[:html_id] <=> b[:html_id]
          end
        end
        r
      }
      render :layout => false, :json => sorted_vuls
    end
  end

  #操作したサマリー表のtrに色を付ける機能で使用
  def api_store_selected_id
    data = params.permit(:hostid_with_htmlid, :selected_id, :page_name)
    page_name = data[:page_name]
    if data[:hostid_with_htmlid] then
      data = JSON.parse(data[:hostid_with_htmlid])
      hostid = data[0].to_s
      htmlid = data[1].to_s
      if !session['hostid_with_htmlid'] then
        session['hostid_with_htmlid'] = {}
      end
      if !session['hostid_with_htmlid'][page_name]
        session['hostid_with_htmlid'][page_name] = {}
      end
      if page_name == 'host' then
        session['hostid_with_htmlid'][page_name][hostid] = [htmlid, false]
        render :layout => false, :json => {}.to_json
      elsif page_name == 'vul' then
        session['hostid_with_htmlid'][page_name][htmlid] = [hostid, false]
        render :layout => false, :json => {}.to_json
      end
    elsif data[:selected_id] then
      if page_name == 'host' then
        host_id = data[:selected_id].to_s
        selected_htmlid = []
        if session['hostid_with_htmlid'] && session['hostid_with_htmlid'][page_name] then
          if session['hostid_with_htmlid'][page_name][host_id] then
            object = session['hostid_with_htmlid'][page_name][host_id]
            if object[1] then
              selected_htmlid.push(object[0])
              session['hostid_with_htmlid'][page_name].delete(host_id)
            end
          end
        end
        render :layout => false, :json => selected_htmlid.to_json
      elsif page_name == 'vul' then
        html_id = data[:selected_id].to_s
        selected_hostid = []
        if session['hostid_with_htmlid'] && session['hostid_with_htmlid'][page_name] then
          if session['hostid_with_htmlid'][page_name][html_id] then
            object = session['hostid_with_htmlid'][page_name][html_id]
            if object[1] then
              selected_hostid.push(object[0])
              session['hostid_with_htmlid'][page_name].delete(html_id)
            end
          end
        end
        render :layout => false, :json => selected_hostid.to_json
      end
    end
  end

  #def api_check_browser_back
  #  data = params.permit(:page_name)
  #  page_name = data[:page_name]
  #  if !session['chkbok_ids'][page_name]['is_browser_back'] then
  #    render :layout => false, :text => 'normal_load'
  #  else
  #    render :layout => false, :text => 'browser_back'
  #  end
  #  session['chkbok_ids'][page_name]['is_browser_back'] = true
  #end

  #ルール設定のメール設定で必要なデータ(カテゴリ名・ユーザ名)を取得
  def api_get_data_for_mail_setting
    data = params.permit(:info_name, :ids)
    info_name = data[:info_name]
    ids = JSON.parse(data[:ids])
    ccids = User.current.sidfmd_categories.where(:is_system => false).map{|c| c.id}
    pass = true
    names = []
    if info_name == 'get_users_by_category_id' then
      if ids[0] == 'all'
        ids = SidfmdCategory.where(:is_system => false).map{|c| c.id}
      end
      if !@sidfm_roles.include?(:admin) then
        ids.each do |id|
          if !ccids.include?(id.to_i) then
            pass = false
          end
        end
      end
      userids = SidfmdCategoriesUser.where(:sidfmd_category_id => ids)
                                    .map{|c| c.user_id}
      User.where(:id => userids).order(:login => :asc).each do |user|
        fullname = user.lastname + user.firstname
        address = user.mail
        id = user.id
        names.push([id, fullname, address])
      end
    elsif info_name.include?('user') then
      if !@sidfm_roles.include?(:admin) then
        userids = SidfmdCategoriesUser.where(:sidfmd_category_id => ccids).map{|c| c.user_id}
        accessible_user_ids = []
        none_accessible_user_ids = []
        ids.each do |id|
          if userids.include?(id.to_i) then
            accessible_user_ids << id.to_i
          else
            none_accessible_user_ids << id.to_i
          end  
        end
        if accessible_user_ids.size > 0 then
          User.where(:id => accessible_user_ids).each do |user|
            fullname = user.lastname + user.firstname
            names << fullname
          end
          if none_accessible_user_ids.size > 0 then
            names << "他#{none_accessible_user_ids.size}名"
          end
        else
          names << "表示できないユーザです"
        end
      else
        User.where(:id => ids).each do |user|
          fullname = user.lastname + user.firstname
          names << fullname
        end 
      end
    elsif info_name.include?('category') then
      if !@sidfm_roles.include?(:admin) then
        accessible_category_ids = []
        none_accessible_category_ids = []
        ids.each do |id|
          if ccids.include?(id.to_i) then
            accessible_category_ids << id.to_i
          else
            none_accessible_category_ids << id.to_i
          end
        end
        if accessible_category_ids.size > 0 then
          SidfmdCategory.where(:id => accessible_category_ids).where(:is_system => false)
          .each do |cat|
            names << cat.name
          end
          if none_accessible_category_ids.size > 0 then
            names << "他#{none_accessible_category_ids.size}カテゴリ"
          end
        else
          names << "表示できないカテゴリです"
        end
      else
        if ids[0] == 'all' then
          cats = SidfmdCategory.where(:is_system => false)
        else
          cats = SidfmdCategory.where(:id => ids).where(:is_system => false)
        end
        cats.each do |cat|
          names.push(cat.name)
        end 
      end
    end
    if pass then
      render :layput => false, :json => names.to_json
    else
      render status: 400, json: { status: 400, message: 'Wrong Parameter' } 
    end 
  end

  def api_get_hosts_by_category_id 
    data = params.permit(:keyword, :category_id)
    keyword = data[:keyword]
    category_ids = JSON.parse(data[:category_id])
    if category_ids[0] == 'all' then
      category_ids = SidfmdCategory.where(:is_system => false)
                                  .map{|c| c.id}
    end
    if keyword and keyword.size >= 2 then
      hosts = get_hosts(keyword)
      host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => category_ids).map{|c| c.sidfmd_host_id}
      hosts = hosts.where(:id => host_ids)
      get_hit_name = ->(host) { ((host.ext_name && host.ext_name.downcase.match(Regexp.new(Regexp.escape(keyword.downcase)))) ? host.ext_name : host.hostname).downcase }
      hosts = hosts.sort{|a, b|
        get_hit_name.(a) <=> get_hit_name.(b)
      }
    end
    render :layout => false, :json => hosts.to_json
  end

  #同名ルールがあるかどうか、ルールの変更があるかどうか、ルールの変更がフラグのみかどうかを返す
  def api_check_rule
    f_params = params.permit(:rule_id, :rule_name, :prev_rule_info, :new_rule_info)#, :updated_at)
    rule_id = (f_params[:rule_id] || 0).to_i
    ret = { :is_same_rule_name => false, :is_updated => false, :is_rule_changed => true, :only_flg_changed => false }
    if SidfmdRule.where.not(:id => rule_id).where(:name => f_params[:rule_name]).first then
      ret[:is_same_rule_name] = true
    end
    #if rule_id != 0 then
    #  if SidfmdRule.where(:id => rule_id).where('updated_at > ?', f_params[:updated_at].in_time_zone + 1).present? then
    #    ret[:is_updated] = true
    #  end
    #end
    if rule_id != 0 then #0は新規登録
      prev_rule_info = JSON.parse(f_params[:prev_rule_info])
      new_rule_info = JSON.parse(f_params[:new_rule_info])
      prev_rule_info.delete('rule_updated_at')
      prev_rule_info['category'] = prev_rule_info['category'].to_s
      new_rule_info['category'] = new_rule_info['category'].to_s
      if prev_rule_info == new_rule_info then
        ret[:is_rule_changed] = false
      else
        prev_rule_flg = prev_rule_info['is_enable']
        new_rule_flg = new_rule_info['is_enable']
        prev_rule_info.delete('is_enable')
        new_rule_info.delete('is_enable')
        if (prev_rule_flg != new_rule_flg) and (prev_rule_info == new_rule_info) then
          ret[:only_flg_changed] = true
        end
      end
    end
    render :layout => false, :json => ret
  end
 
  #全プロダクトを返す。ホストに登録されているものにはホスト名を付ける
  def api_get_products_with_hostname
    data = params.permit(:keyword)
    keyword = data[:keyword]
    ret = {}
    if keyword and keyword.size >= 2 then
      hosts = SidfmdHost.where(:flg_delete => false)
      hosts_versions = {}
      hosts.each do |host|
        hostname = host.hostname
        host.sidfmd_host_apps.each do |app|
          v_id = app.version_id
          hosts_versions[v_id] = [] unless hosts_versions[v_id]
          hosts_versions[v_id].push(hostname)
        end
        host.get_agent_products.each do |prod|
          v_id = prod["version_id"]
          hosts_versions[v_id] = [] unless hosts_versions[v_id]
          hosts_versions[v_id].push(hostname) if v_id
        end  
      end
      products = SidfmdProduct
        .where('LOWER(name) LIKE ? OR LOWER(name) LIKE ?',
               ActiveRecord::Base.send(:sanitize_sql_like, keyword.downcase) + '%',
               '% ' + ActiveRecord::Base.send(:sanitize_sql_like, keyword.downcase) + '%')
        .order(:category)
      products.each do |product|
        ret[product.pid] = { 'product' => product }
      end
      versions = SidfmdVersion
        .where.not(:version => 'ALL')
        .where('LOWER(fullname) LIKE ? OR LOWER(fullname) LIKE ?',
               ActiveRecord::Base.send(:sanitize_sql_like, keyword.downcase) + '%',
               '% ' + ActiveRecord::Base.send(:sanitize_sql_like, keyword.downcase) + '%')
        .order(:position => :desc)
      versions.each do |version|
        product = version.sidfmd_product
        if !ret.has_key?(product.pid) then
          ret[product.pid] = { 'product' => product }
        end
        ret[product.pid]['versions'] = [] unless ret[product.pid].has_key?('versions')
        ret[product.pid]['versions'] << {
                                          "data" => version, 
                                          "related-host" => hosts_versions[version.vid]
                                        }
      end
    end
    render :layout => false, :json => ret.to_json
  end

  #idからversionもしくはproductの名前を返す
  def api_get_products_and_versions_by_ids
    f_param = params.permit(:product_ids, :version_ids)
    prod_ids = JSON.parse(f_param[:product_ids]) if f_param[:product_ids]
    ver_ids = JSON.parse(f_param[:version_ids]) if f_param[:version_ids]
    data = {}
    products = []
    versions = []
    SidfmdProduct.where(:id => prod_ids)
                 .each do |p|
                   products.push(
                     { "id" => p.id, "name" =>  p.name }
                   )
                 end
    SidfmdVersion.where(:id => ver_ids)
                 .each do |v| 
                   versions.push(
                     { "id" => v.id, "name" => v.fullname }
                   )
                 end
    data[:products] = products
    data[:versions] = versions
    render :layout => false, :json => data.to_json
  end
   
  #ルール設定画面のメール送信画面上にあるキーワードを変換ボタンを押した時にキーワードを変換した文字列を返す
  #出力されるデータは全てサンプル
  def api_replace_mail_keywords
    f_param = params.permit(:title,:message)
    opts = {}
    mail_title = f_param[:title]
    message = f_param[:message]
    vulsync_info_titles = {
      11287 => "GNU Wget の HTTP リダイレクトの処理に任意のファイルを上書きされる問題",
      15766 => "FreeType の BDF フォントの処理にサービスを妨害される問題",
      19285 => "GNU Bash の環境変数の処理に任意のコマンドを実行される問題",
      26981 => "Linux Kernel の Bluetooth の処理に任意のコードを実行される問題",
      15602 => "CUPS に特権を奪われる問題"
    }
    risk_rate_description = ->(c_percent, h_percent, m_percent, l_percent, n_percent, f_percent, risk_index){
      return ret = if risk_index == "sri" then
        "重大:#{c_percent >= 10 ? c_percent.round(1) : (c_percent >= 1 ? c_percent.round(2) : c_percent.round(3))}%," +
        "高:#{h_percent >= 10 ? h_percent.round(1) : (h_percent >= 1 ? h_percent.round(2) : h_percent.round(3))}%," +
        "中:#{m_percent >= 10 ? m_percent.round(1) : (m_percent >= 1 ? m_percent.round(2) : m_percent.round(3))}%," +
        "低:#{l_percent >= 10 ? l_percent.round(1) : (l_percent >= 1 ? l_percent.round(2) : l_percent.round(3))}%," +
        "無:#{n_percent >= 10 ? n_percent.round(1) : (n_percent >= 1 ? n_percent.round(2) : n_percent.round(3))}%," +
        "済:#{f_percent >= 10 ? f_percent.round(1) : (f_percent >= 1 ? f_percent.round(2) : f_percent.round(3))}%"
      else
        "未登録:#{n_percent >= 10 ? n_percent.round(1) : (n_percent >= 1 ? n_percent.round(2) : n_percent.round(3))}%," +
        "10.0～9.0:#{c_percent >= 10 ? c_percent.round(1) : (c_percent >= 1 ? c_percent.round(2) : c_percent.round(3))}%," +
        "8.9～7.0:#{h_percent >= 10 ? h_percent.round(1) : (h_percent >= 1 ? h_percent.round(2) : h_percent.round(3))}%," +
        "6.9～4.0:#{m_percent >= 10 ? m_percent.round(1) : (m_percent >= 1 ? m_percent.round(2) : m_percent.round(3))}%," +
        "3.9～0.0#{l_percent >= 10 ? l_percent.round(1) : (l_percent >= 1 ? l_percent.round(2) : l_percent.round(3))}%," +
        "済:#{f_percent >= 10 ? f_percent.round(1) : (f_percent >= 1 ? f_percent.round(2) : f_percent.round(3))}%"
      end
    }
    handling_rate_description = ->(c_percent, g_percent, f_percent, n_percent){
      "未対策:#{c_percent >= 10 ? c_percent.round(1) : (c_percent >= 1 ? c_percent.round(2) : c_percent.round(3))}%," +
      "対策中:#{g_percent >= 10 ? g_percent.round(1) : (g_percent >= 1 ? g_percent.round(2) : g_percent.round(3))}%," +
      "対策済:#{f_percent >= 10 ? f_percent.round(1) : (f_percent >= 1 ? f_percent.round(2) : f_percent.round(3))}%," +
      "影響無:#{n_percent >= 10 ? n_percent.round(1) : (n_percent >= 1 ? n_percent.round(2) : n_percent.round(3))}%"
    }
    filter_list_data = ->(type, data_num, content, content_size, no_data_message, join_type, limit){
      return ret = if type == "default" then
        content_size > 0 ? content.slice(0..limit - 1).join(join_type) : no_data_message
      elsif type == "case"
        data_num.nil? ? content_size : (content_size + data_num > 0 ? content_size + data_num : 0)
      elsif type == "top" then
        content_size > 0 ? content.slice(0..data_num - 1).join(join_type) : no_data_message
      else
        content_size > 0 ? content.reverse.slice(0..data_num - 1).join(join_type) : no_data_message
      end
    }
    messages_keywords = SidfmdMailer.get_keywords(message)
    titles_keywords = SidfmdMailer.get_keywords(mail_title)
    limit_num = SidfmdMailer::MAIL_LIMIT_DATA_NUM
    keywords = messages_keywords.merge(titles_keywords)
    keywords.each do |filtered_keyword, data|
      keyword = data[:keyword] 
      type = data[:type]
      data_num = data[:necessary_data_num] 
      case keyword 
      when "%RISK_RATE_SRI_ALL%" then
        opts[filtered_keyword] = risk_rate_description.(4.08, 29.3, 41.0, 24.1, 1.2, 0.298, "sri")
      when "%UNHANDLED_RATE_SRI_ALL%" then
        opts[filtered_keyword] = "38%"
      when "%RISK_RATE_SRI_MATCH%" then
        opts[filtered_keyword] = risk_rate_description.(5.08, 19.3, 41.0, 24.1, 2.2, 10.298, "sri")
      when "%UNHANDLED_RATE_SRI_MATCH%" then
        opts[filtered_keyword] = "28%"
      when "%RISK_RATE_CVSS_ALL%" then
        opts[filtered_keyword] = risk_rate_description.(15.08, 19.3, 32.0, 23.1, 3.2, 8.298, "cvss")
      when "%UNHANDLED_RATE_CVSS_ALL%"
        opts[filtered_keyword] = "20%"
      when "%RISK_RATE_CVSS_MATCH%"
        opts[filtered_keyword] = risk_rate_description.(25.08, 29.3, 52.0, 3.1, 1.2, 10.298, "cvss")
      when "%UNHANDLED_RATE_CVSS_MATCH%" then
        opts[filtered_keyword] = "30%"
      when "%HANDLING_RATE_ALL%" then
        opts[filtered_keyword] = handling_rate_description.(38.33, 32.67, 15.33, 14.77)
      when "%HANDLING_RATE_MATCH%" then
        opts[filtered_keyword] = handling_rate_description.(58.33, 22.67, 5.33, 14.77)
      when "%URL%" then
        opts[filtered_keyword] = sidfm_rule_setting_index_url
      when "%TITLE%" then
        opts[filtered_keyword] = SidfmdRule::PAGE_TITLE
      when "%OS_RATE_ALL%" then
        opts[filtered_keyword] = "Ubuntu 18.04 LTS: 33.3%, CentOS 7: 33.3%, （不明）: 33.3%"
      when "%OS_RATE_MATCH%" then
        opts[filtered_keyword] = "Ubuntu 18.04 LTS: 50.0%, CentOS 7: 50.0%"
      when "%OS_ALL%" then
        all_os_names = ["Ubuntu 18.04 LTS", "CentOS 7"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, all_os_names, all_os_names.size, "対象のOSはありません。", ', ', limit_num)
      when "%OS_MATCH%" then
        match_all_os_names = ["Ubuntu 18.04 LTS", "CentOS 7"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_all_os_names, match_all_os_names.size, "対象のOSはありません。", ', ', limit_num)
      when "%HOST_ALL%" then
        hostnames = ["host-a", "host-b", "host-c"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, hostnames, hostnames.size, "対象のホストはありません。", ', ', limit_num)
      when "%HOST_MATCH%" then
        match_hostnames = ["host-a", "host-c"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_hostnames, match_hostnames.size, "対象のホストはありません。", ', ', limit_num)
      when "%DISPLAY_NAME_ALL%" then
        display_names = ["host-a", "host-b", "サンプルホスト"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, display_names, display_names.size, "対象のホストはありません。", ', ', limit_num)
      when "%DISPLAY_NAME_MATCH%" then
        match_display_names = ["host-a", "サンプルホスト"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_display_names, match_display_names.size, "対象のホストはありません。", ', ', limit_num)
      when "%HOSTINFO_ALL%" then
        hosts_info = [
          "host-a\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10000)}\n" + 
          "OS:Ubuntu 18.04 LTS\n" +
          "システムカテゴリ:内部サーバ   カスタムカテゴリ:サンプルA, サンプルB\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:高   最大CVSS値:9.8   影響範囲:リモート   攻撃コード:あり\n" +
          "対応状況: 未対策101件  対策中60件 対策済20件  影響無1件\n" +
          "対策予定日設定済:0.0%   対処率:77.3%\n" +
          "検出パッケージ数:100   検出アプリケーション数:5   手動登録プロダクト数:1   ローカルプロダクト数:1\n" +
          "同期状態:同期中   エージェントの使用状態:使用\n" +
          "最終確認日時:2020/05/11 20:38:54\n" +
          "メモ:サンプル1",
          "host-b\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10001)}\n" +   
          "OS:---\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルC\n" +
          "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
          "最大SRI値:中   最大CVSS値: 5.8   影響範囲:リモート   攻撃コード:なし\n" +
          "対応状況: 未対策11件  対策中50件 対策済120件  影響無10件\n" +
          "対策予定日設定済:10.0%   対処率:90.0%\n" +
          "検出パッケージ数:0   検出アプリケーション数:0   手動登録プロダクト数:1   ローカルプロダクト数:0\n" +
          "同期状態:非同期   エージェントの使用状態:不使用\n" +
          "最終確認日時:2020/03/11 12:31:50\n" +
          "メモ:サンプル2",
          "host-c\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10002)}\n" +   
          "OS:CentOS 7\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA, サンプルB, サンプルC\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:重大   最大CVSS値:9.7   影響範囲:クライアント   攻撃コード:あり\n" +
          "対応状況: 未対策211件  対策中10件 対策済10件  影響無10件\n" +
          "対策予定日設定済:20.0%   対処率:80.0%\n" +
          "検出パッケージ数:200   検出アプリケーション数:10   手動登録プロダクト数:3   ローカルプロダクト数:2\n" +
          "同期状態:同期中   エージェントの使用状態:使用\n" + 
          "最終確認日時:2020/01/11 11:11:00\n" +
          "メモ:サンプル3"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, hosts_info, hosts_info.size, "対象のホストはありません。", "\n\n", limit_num)
      when "%HOSTINFO_MATCH%" then
        match_hosts_info = [
          "host-a\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10000)}\n" +
          "OS:Ubuntu 18.04 LTS\n" +
          "システムカテゴリ:内部サーバ   カスタムカテゴリ:サンプルA, サンプルB\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:高   最大CVSS値:9.8   影響範囲:リモート   攻撃コード:あり\n" +
          "対応状況: 未対策101件  対策中60件 対策済20件  影響無1件\n" +
          "対策予定日設定済:0.0%   対処率:77.3%\n" +
          "検出パッケージ数:100   検出アプリケーション数:5   手動登録プロダクト数:1   ローカルプロダクト数:1\n" +
          "同期状態:同期中   エージェントの使用状態:使用\n" +
          "最終確認日時:2020/05/11 20:38:54\n" +
          "メモ:サンプル1",
          "host-c\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10002)}\n" +
          "OS:CentOS 7\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA, サンプルB, サンプルC\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:重大   最大CVSS値:9.7   影響範囲:クライアント   攻撃コード:あり\n" +
          "対応状況: 未対策211件  対策中10件 対策済10件  影響無10件\n" +
          "対策予定日設定済:20.0%   対処率:80.0%\n" +
          "検出パッケージ数:200   検出アプリケーション数:10   手動登録プロダクト数:3   ローカルプロダクト数:2\n" +
          "同期状態:同期中   エージェントの使用状態:使用\n" +
          "最終確認日時:2020/01/11 11:11:00\n" +
          "メモ:サンプル3"
        ] 
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_hosts_info, match_hosts_info.size, "対象のホストはありません。", "\n\n", limit_num) 
      when "%S_CATEGORY_ALL%" then
        s_categories = ["公開サーバ", "内部サーバ"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, s_categories, s_categories.size, "対象のシステムカテゴリはありません。", ', ', limit_num)
      when "%S_CATEGORY_MATCH%" then
        match_s_categories = ["公開サーバ", "内部サーバ"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_s_categories, match_s_categories.size, "対象のシステムカテゴリはありません。", ', ', limit_num)
      when "%C_CATEGORY_ALL%" then
        c_categories = ["サンプルA", "サンプルB", "サンプルC"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, c_categories, c_categories.size, "対象のカスタムカテゴリはありません。", ', ', limit_num)
      when "%C_CATEGORY_MATCH%" then
        match_c_categories = ["サンプルA", "サンプルB", "サンプルC"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_c_categories, match_c_categories.size, "対象のカスタムカテゴリはありません。", ', ', limit_num)
      when "%HOST_LINKS_ALL%" then
        hosts_links = [
          "host-a\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10000)}",
          "host-b\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10001)}",
          "host-c\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10002)}"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, hosts_links, hosts_links.size, "対象のホストはありません。", "\n\n", limit_num)
      when "%HOST_LINKS_MATCH%" then
        match_hosts_links = [
          "host-a\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10000)}",
          "host-c\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10002)}"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_hosts_links, match_hosts_links.size, "対象のホストはありません。", "\n\n", limit_num)
      when "%VULINFO_ALL%" then
        vuls_info = [
          "[23355]Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)\n" +
          "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => 23355)}\n" +
          "関連するホスト:host-a, host-b\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA\n" +
          "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
          "最大SRI値:中   最大CVSS値:7.8   影響範囲:ローカル   攻撃コード:なし\n" +
          "対応状況: 未対策2件  対策中0件 対策済0件   影響無0件\n" +
          "検出日:2020/01/18   更新日:2021/07/28   検出後最長経過日数:116   対策予定日設定済:50.0%   対処率:0.0%\n" +
          "CVSS:\n" +
          "CVE-2010-2252: CVSS V2: AV:N/AC:M/Au:N/C:P/I:P/A:P\n" +
          "CVE-2019-19604: CVSS V3: AV:L/AC:L/PR:N/UI:R/S:U/C:H/I:H/A:H, CVSS V2: AV:N/AC:M/Au:N/C:C/I:C/A:C\n" +
          "CVE-2021-34506: CVSS V3: AV:N/AC:L/PR:N/UI:R/S:U/C:L/I:L/A:N\n" +
          "SIDfmID:23355\n" +
          "脆弱性情報の詳細ページへのリンク:\n" +
          "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 23355)}",
          "[KB2984943]Microsoft Office 2007 suites (KB2984943)   のセキュリティ更新プログラム\n" +
          "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => -2984943)}\n" +
          "関連するホスト:hostb. host-c\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルC\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:高   最大CVSS値:n/a   影響範囲:未定義   攻撃コード:n/a\n" +
          "対応状況: 未対策2件  対策中0件 対策済0件   影響無0件\n" +
          "検出日:2020/01/14   更新日:2021/07/28   検出後最長経過日数:119   対策予定日設定済:50.0%   対処率:0.0%\n" +
          "KB番号:KB2984943\n" +
          "Microsoft Update カタログリンク:\n" +
          "https://www.catalog.update.microsoft.com/Search.aspx?q=KB2984943\n" +
          "SIDfm情報:\n" +
          "23348 Microsoft Office の Office ファイルの処理に任意のコードを実行されるなど複数の問題 (MS16-054)\n" +
          "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 23348)}"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, vuls_info, vuls_info.size, "対象の脆弱性はありません。", "\n\n", limit_num)
      when "%VULINFO_MATCH%" then
        match_vuls_info = [
          "[23355]Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)\n" +
          "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => 23355)}\n" +
          "関連するホスト:host-a, host-b\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA\n" +
          "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
          "最大SRI値:中   最大CVSS値:7.8   影響範囲:ローカル   攻撃コード:なし\n" +
          "対応状況: 未対策2件  対策中0件 対策済0件   影響無0件\n" +
          "検出日:2020/01/18   更新日:2021/07/28   検出後最長経過日数:116   対策予定日設定済:50.0%   対処率:0.0%\n" +
          "CVSS:\n" +
          "CVE-2010-2252: CVSS V2: AV:N/AC:M/Au:N/C:P/I:P/A:P\n" +
          "CVE-2019-19604: CVSS V3: AV:L/AC:L/PR:N/UI:R/S:U/C:H/I:H/A:H, CVSS V2: AV:N/AC:M/Au:N/C:C/I:C/A:C\n" +
          "CVE-2021-34506: CVSS V3: AV:N/AC:L/PR:N/UI:R/S:U/C:L/I:L/A:N\n" +
          "SIDfmID:23355\n" +
          "脆弱性情報の詳細ページへのリンク:\n" +
          "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 23355)}",
          "[KB2984943]Microsoft Office 2007 suites (KB2984943)   のセキュリティ更新プログラム\n" +
          "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => -2984943)}\n" +
          "関連するホスト:hostb. host-c\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルC\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:高   最大CVSS値:n/a   影響範囲:未定義   攻撃コード:n/a\n" +
          "対応状況: 未対策2件  対策中0件 対策済0件   影響無0件\n" +
          "検出日:2020/01/14   更新日:2021/07/28   検出後最長経過日数:119   対策予定日設定済:50.0%   対処率:0.0%\n" +
          "KB番号:KB2984943\n" +
          "Microsoft Update カタログリンク:\n" +
          "https://www.catalog.update.microsoft.com/Search.aspx?q=KB2984943\n" +
          "SIDfm情報:\n" +
          "23348 Microsoft Office の Office ファイルの処理に任意のコードを実行されるなど複数の問題 (MS16-054)\n" +
          "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 23348)}" 
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_vuls_info, match_vuls_info.size, "対象の脆弱性はありません。", "\n\n", limit_num)
      when "%VULTITLE_ALL%" then
        vul_titles = [
          "Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)",
          "Adobe Flash Player に任意のコードを実行される問題",
          "Microsoft Windows に Kernel API の処理に権限を昇格されるなど複数の問題 (MS16-111)",
          "CentOS 6 の samba に複数のセキュリティホール",
          "Microsoft Windows の Adobe フォントドライバの処理に任意のコードを実行される複数の問題 (MS15-021)",
          "Red Hat Enterprise Linux 6 の ntp に複数のセキュリティホール",
          "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
          "CentOS の firefox に複数のセキュリティホール"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, vul_titles, vul_titles.size, "対象の脆弱性はありません。", ",\n", limit_num)
      when "%VULTITLE_MATCH%" then
        match_vul_titles = [
          "Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)",
          "Adobe Flash Player に任意のコードを実行される問題",
          "CentOS 6 の samba に複数のセキュリティホール",
          "Microsoft Windows に Kernel API の処理に権限を昇格されるなど複数の問題 (MS16-111)",
          "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
          
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_vul_titles, match_vul_titles.size, "対象の脆弱性はありません。", ",\n", limit_num)
      when "%VUL_LINKS_ALL%" then
        vulns = {
          23355 => "Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)",
          15685 => "Adobe Flash Player に任意のコードを実行される問題",
          24346 => "Microsoft Windows に Kernel API の処理に権限を昇格されるなど複数の問題 (MS16-111)",
          22373 => "CentOS 6 の samba に複数のセキュリティホール",
          20294 => "Microsoft Windows の Adobe フォントドライバの処理に任意のコードを実行される複数の問題 (MS15-021)",
          23396 => "Red Hat Enterprise Linux 6 の ntp に複数のセキュリティホール",
          19767 => "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
          21520 => "CentOS の firefox に複数のセキュリティホール", 
        }
        vuls_links = vulns.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}   
        opts[filtered_keyword] = filter_list_data.(type, data_num, vuls_links, vuls_links.size, "対象の脆弱性はありません。", "\n\n", limit_num)
      when "%VUL_LINKS_MATCH%" then
        vulns = {
          23355 => "Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)",
          15685 => "Adobe Flash Player に任意のコードを実行される問題",
          19767 => "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
        }
        match_vuls_links = vulns.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_vuls_links, match_vuls_links.size, "対象の脆弱性はありません。", "\n\n", limit_num)
      when "%SIDfmID_ALL%" then
        sidfm_ids = [15685, 22373, 23396, 19767, 21520]
        opts[filtered_keyword] = filter_list_data.(type, data_num, sidfm_ids, sidfm_ids.size, "関連するSIDfmIDはありません。", ", ", limit_num)
      when "%SIDfmID_MATCH%" then
        match_sidfm_ids = [15685, 19767]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_sidfm_ids, match_sidfm_ids.size, "関連するSIDfmIDはありません。", ", ", limit_num)
      when "%SIDfm_LINKS_ALL%" then
        vulns = {
          15685 => "Adobe Flash Player に任意のコードを実行される問題",
          22373 => "CentOS 6 の samba に複数のセキュリティホール",
          23396 => "Red Hat Enterprise Linux 6 の ntp に複数のセキュリティホール",
          19767 => "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
          21520 => "CentOS の firefox に複数のセキュリティホール",
        }
        sidfm_info_links = vulns.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => id)}"} 
        opts[filtered_keyword] = filter_list_data.(type, data_num, sidfm_info_links, sidfm_info_links.size, "関連する「脆弱性情報の詳細」ページはありません。", "\n\n", limit_num)
      when "%SIDfm_LINKS_MATCH%" then
        vulns = {
          15685 => "Adobe Flash Player に任意のコードを実行される問題", 
          19767 => "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
        }
        match_sidfm_info_links = vulns.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_sidfm_info_links, match_sidfm_info_links.size, "関連する「脆弱性情報の詳細」ページはありません。", "\n\n", limit_num)
      when "%MS_LINKS_ALL%" then
        kbs = {
          2984943 => "Microsoft Office 2007 suites (KB2984943)   のセキュリティ更新プログラム",
          2825645 => "Microsoft Office 2007 suites (KB2825645)   のセキュリティ更新プログラム",
          2899473 => "Microsoft Office 2007 suites (KB2899473)   のセキュリティ更新プログラム"
        }
        ms_links = kbs.map{|kb, title| "#{title}\nhttps://www.catalog.update.microsoft.com/Search.aspx?q=KB#{kb}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, ms_links, ms_links.size, "関連する「Microsoft Update カタログ」はありません。", "\n\n", limit_num)
      when "%MS_LINKS_MATCH%" then
        kbs = {
          2984943 => "Microsoft Office 2007 suites (KB2984943)   のセキュリティ更新プログラム", 
          2825645 => "Microsoft Office 2007 suites (KB2825645)   のセキュリティ更新プログラム",
        }
        match_ms_links = kbs.map{|kb, title| "#{title}\nhttps://www.catalog.update.microsoft.com/Search.aspx?q=KB#{kb}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_ms_links, match_ms_links.size, "関連する「Microsoft Update カタログ」はありません。", "\n\n", limit_num)
      when "%KB_ALL%" then
        kb_nums = [2984943, 2825645, 2899473]
        opts[filtered_keyword] = filter_list_data.(type, data_num, kb_nums, kb_nums.size, "関連するKB番号無し", ", ", limit_num)
      when "%KB_MATCH%" then
        match_kb_nums = [2984943, 2825645]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_kb_nums, match_kb_nums.size, "関連するKB番号無し", ", ", limit_num)
      when "%APP_ALL%" then
        all_apps = [
          "Apache Struts 1.3.9\n/home/softek/agenttools/dummy/struts/struts-1.3.9/struts-core-1.3.9.jar",
          "Apache Struts 2.5.16\n/home/softek/agenttools/dummy/struts/struts-2.5.16/struts2-core-2.5.16.jar",
          "Apache Tomcat 7.0.85\n/home/softek/agenttools/dummy/tomcat7/apache-tomcat-7.0.85/catalina.jar"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, all_apps, all_apps.size, "検出アプリケーションはありません。", "\n\n", limit_num)
      when "%APP_MATCH%" then
        match_all_apps = [
          "Apache Struts 1.3.9\n/home/softek/agenttools/dummy/struts/struts-1.3.9/struts-core-1.3.9.jar",
          "Apache Struts 2.5.16\n/home/softek/agenttools/dummy/struts/struts-2.5.16/struts2-core-2.5.16.jar",
          "Apache Tomcat 7.0.85\n/home/softek/agenttools/dummy/tomcat7/apache-tomcat-7.0.85/catalina.jar"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_all_apps, match_all_apps.size, "検出アプリケーションはありません。", "\n\n", limit_num)
      when "%PRODUCT_ALL%" then
        all_lapps = [
          "CentOS 7", "Microsoft Windows 8", "Microsoft Windows Media Player 12"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, all_lapps, all_lapps.size, "手動登録プロダクトはありません。", ", ", limit_num)
      when "%PRODUCT_MATCH%" then
        match_all_lapps = [
          "CentOS 7", "Microsoft Windows 8", "Microsoft Windows Media Player 12"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_all_lapps, match_all_lapps.size, "手動登録プロダクトはありません。", ", ", limit_num)
      when "%LPRODUCT_ALL%" then
        all_lproducts = [ "sample_product", "sample_product2", "sample_version1", "sample_version2", "sample_ver_3"].sort()
        opts[filtered_keyword] = filter_list_data.(type, data_num, all_lproducts, all_lproducts.size, "ローカルプロダクトはありません。", ", ", limit_num)
      when "%LPRODUCT_MATCH%" then
        match_all_lproducts = [ "sample_product", "sample_version1", "sample_ver_3"].sort()
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_all_lproducts, match_all_lproducts.size, "ローカルプロダクトはありません。", ", ", limit_num)
      when "%VULSYNC_REGI_ALL%" then
        new_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, new_vuls, new_vuls.size, "追加された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_RENEW_ALL%" then
        mod_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, mod_vuls, mod_vuls.size, "更新された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_DEL_ALL%" then
        del_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, del_vuls, del_vuls.size, "削除(ステータスが「影響無」に変更)された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_REGI_MATCH%" then
        new_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, new_vuls, new_vuls.size, "追加された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_RENEW_MATCH%" then
        mod_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, mod_vuls, mod_vuls.size, "更新された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_DEL_MATCH%" then
        del_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, del_vuls, del_vuls.size, "削除(ステータスが「影響無」に変更)された脆弱性はありません。", "\n\n", limit_num)
      when '%VULSYNC_INFO_REGI_ALL%', '%VULSYNC_INFO_RENEW_ALL%', '%VULSYNC_INFO_DEL_ALL%', '%VULSYNC_INFO_REGI_MATCH%', '%VULSYNC_INFO_RENEW_MATCH%', '%VULSYNC_INFO_DEL_MATCH%' then
        list = 
          if [ '%VULSYNC_INFO_REGI_ALL%', '%VULSYNC_INFO_RENEW_ALL%', '%VULSYNC_INFO_DEL_ALL%' ].include?(keyword) then
            [
              "[15436]Microsoft IIS の FTP サービスに情報漏洩など複数の問題 (MS12-073)\n" + 
              "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => 15436)}\n" +
              "関連するホスト:host-a, host-b\n" +
              "システムカテゴリ:内部サーバ, 公開サーバ   カスタムカテゴリ:サンプルA\n" +
              "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
              "最大SRI値:低   最大CVSS値:5.0   影響範囲:リモート   攻撃コード:なし\n" +
              "対応状況: 未対策1件  対策中0件 対策済0件   影響無0件\n" +
              "検出日:2021/07/12   更新日:2021/07/28   検出後最長経過日数:0   対策予定日設定済:0.0%   対処率:0.0%\n" +
              "CVSS:\n" +
              "CVE-2012-2531: CVSS V2: AV:L/AC:L/Au:N/C:P/I:N/A:N\n" +
              "CVE-2012-2532: CVSS V2: AV:N/AC:L/Au:N/C:P/I:N/A:N\n" +
              "SIDfmID:15436\n" +
              "脆弱性情報の詳細ページへのリンク:\n" +
              "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 15436)}",
              "[15580]Microsoft Windows の OpenType フォントおよび TrueType フォントの処理に任意のコードを実行される問題 (MS12-078)\n" +
              "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => 15580)}\n" +
              "関連するホスト:hosta. host-c\n" +
              "システムカテゴリ:内部サーバ, 公開サーバ  カスタムカテゴリ:サンプルA, サンプルB, サンプルC\n" +
              "対応:CVSS値を元にした場合、要。SRI値を元にした場合、不要。\n" +
              "最大SRI値:中   最大CVSS値:10.0   影響範囲:クライアント   攻撃コード:なし\n" +
              "対応状況: 未対策1件  対策中0件 対策済0件   影響無0件\n" +
              "検出日:2021/07/12   更新日:2021/07/28   検出後最長経過日数:0   対策予定日設定済:0.0%   対処率:0.0%\n" +
              "CVSS:\n" +
              "CVE-2012-2556: CVSS V2: AV:N/AC:M/Au:N/C:C/I:C/A:C\n" +
              "CVE-2012-4786: CVSS V2: AV:N/AC:L/Au:N/C:C/I:C/A:C\n" +
              "SIDfmID:15580\n" +
              "脆弱性情報の詳細ページへのリンク:\n" +
              "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 15580)}"
            ]
          else
            [
              "[18418]QEMU の IDE デバイスの処理に QEMU プロセスの権限を奪われる問題\n" +
              "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => 18418)}\n" +
              "関連するホスト:host-c\n" +
              "システムカテゴリ:公開サーバ   カスタムカテゴリ: サンプルD\n" +
              "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
              "最大SRI値:無   最大CVSS値:7.2   影響範囲:ローカル   攻撃コード:なし\n" +
              "対応状況: 未対策0件  対策中0件 対策済1件   影響無0件\n" +
              "検出日:2021/07/12   更新日:2021/07/28   検出後最長経過日数:---   対策予定日設定済:---   対処率:100.0%\n" +
              "CVSS:\n" +
              "CVE-2014-2894: CVSS V2: AV:L/AC:L/Au:N/C:C/I:C/A:C\n" +
              "SIDfmID:18418\n" +
              "脆弱性情報の詳細ページへのリンク:\n" +
              "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 18418)}",
              "[18696]Red Hat Enterprise Linux 7 の kernel に複数のセキュリティホール\n" +
              "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => 18696)}\n" +
              "関連するホスト:host-a, host-d\n" +
              "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA, サンプルD\n" +
              "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
              "最大SRI値:無   最大CVSS値:7.2   影響範囲:ローカル   攻撃コード:あり\n" +
              "対応状況: 未対策0件  対策中0件 対策済1件   影響無0件\n" +
              "検出日:2021/07/12   更新日:2021/07/28   検出後最長経過日数:---   対策予定日設定済:---   対処率:100.0%\n" +
              "CVSS:\n" +
              "CVE-2014-0206: CVSS V2: AV:L/AC:L/Au:N/C:P/I:N/A:N\n" +
              "CVE-2014-1737: CVSS V2: AV:L/AC:L/Au:N/C:C/I:C/A:C\n" +
              "CVE-2014-1738: CVSS V2: AV:L/AC:L/Au:N/C:P/I:N/A:N\n" +
              "CVE-2014-2568: CVSS V2: AV:A/AC:M/Au:N/C:P/I:N/A:N\n" +
              "CVE-2014-2851: CVSS V2: AV:L/AC:M/Au:N/C:C/I:C/A:C\n" +
              "CVE-2014-3144: CVSS V2: AV:L/AC:L/Au:N/C:N/I:N/A:C\n" +
              "CVE-2014-3145: CVSS V2: AV:L/AC:L/Au:N/C:N/I:N/A:C\n" +
              "CVE-2014-3153: CVSS V2: AV:L/AC:L/Au:N/C:C/I:C/A:C\n" +
              "SIDfmID:18696\n" +
              "脆弱性情報の詳細ページへのリンク:\n" +
              "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 18696)}"
            ]
          end
        empty_message = if keyword == '%VULSYNC_INFO_REGI_ALL%' then
                          "新たに登録された脆弱性はありません。"
                        elsif keyword == '%VULSYNC_INFO_RENEW_ALL%' then
                          "更新された脆弱性はありません。"
                        else
                          "削除(ステータスが「影響無」に変更された脆弱性はありません。"
                        end
        opts[filtered_keyword] = filter_list_data.(type, data_num, list, list.size, empty_message, "\n\n", limit_num)
      when "%VULSYNC_REGI_HOST_ALL%", "%VULSYNC_REGI_HOST_MATCH%", "%VULSYNC_RENEW_HOST_ALL%", "%VULSYNC_RENEW_HOST_MATCH%" , "%VULSYNC_DEL_HOST_ALL%", "%VULSYNC_DEL_HOST_MATCH%" then
        hostnames = case keyword
                    when "%VULSYNC_REGI_HOST_ALL%", "%VULSYNC_RENEW_HOST_ALL%", "%VULSYNC_DEL_HOST_ALL%" then
                      [ 'host-a', 'host-b', 'host-e' ]
                    when "%VULSYNC_REGI_HOST_MATCH%", "%VULSYNC_RENEW_HOST_MATCH%" , "%VULSYNC_RENEW_HOST_MATCH%" ,"%VULSYNC_DEL_HOST_MATCH%" then
                      [ 'host-c', 'host-d' ]
                    end
        empty_message = case keyword
                        when "%VULSYNC_REGI_HOST_ALL%", "%VULSYNC_RENEW_HOST_ALL%" then
                          "新たに登録された脆弱性のホストはありません。"
                        when "%VULSYNC_RENEW_HOST_ALL%", "%VULSYNC_RENEW_HOST_MATCH%" then
                          "更新された脆弱性のホストはありません。"
                        else
                          "削除(ステータスが「影響無」に変更)された脆弱性のホストはありません。"
                        end
        opts[filtered_keyword] = filter_list_data.(type, data_num, hostnames, hostnames.size, empty_message, ", ", limit_num)
      when "%VULSYNC_REGI_DISPLAY_NAME_ALL%", "%VULSYNC_REGI_DISPLAY_NAME_MATCH%", "%VULSYNC_RENEW_DISPLAY_NAME_ALL%", "%VULSYNC_RENEW_DISPLAY_NAME_MATCH%", "%VULSYNC_DEL_DISPLAY_NAME_ALL%", "%VULSYNC_DEL_DISPLAY_NAME_MATCH%" then
        display_names = case keyword
                    when "%VULSYNC_REGI_DISPLAY_NAME_ALL%", "%VULSYNC_RENEW_DISPLAY_NAME_ALL%", "%VULSYNC_DEL_DISPLAY_NAME_ALL%" then
                      [ 'サンプルホストA', 'サンプルホストB', 'サンプルホストE' ]
                    when "%VULSYNC_REGI_DISPLAY_NAME_MATCH%", "%VULSYNC_RENEW_DISPLAY_NAME_MATCH%", "%VULSYNC_DEL_DISPLAY_NAME_MATCH%" then
                      [ 'サンプルホストC', 'サンプルホストD' ]
                    end
        empty_message = case keyword
                        when "%VULSYNC_REGI_DISPLAY_NAME_ALL%", "%VULSYNC_REGI_DISPLAY_NAME_MATCH%" then
                          "新たに登録された脆弱性のホストはありません。"
                        when "%VULSYNC_RENEW_DISPLAY_NAME_ALL%", "%VULSYNC_RENEW_DISPLAY_NAME_MATCH%" then
                          "更新された脆弱性のホストはありません。"
                        else
                          "削除(ステータスが「影響無」に変更)された脆弱性のホストはありません。"
                        end
        opts[filtered_keyword] = filter_list_data.(type, data_num, display_names, display_names.size, empty_message, ", ", limit_num)
      when "%VULSYNC_ALL_HOST%" then
        hostnames = [ 'host-a', 'host-b', 'host-c' ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, hostnames, hostnames.size, "同期対象となったホストはありません。", ", ", limit_num)
      when "%VULSYNC_ALL_DISPLAY_NAME%" then
        display_names = [ 'サンプルホストA', 'サンプルホストB', 'サンプルホストC' ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, display_names, display_names.size, "同期対象となったホストはありません。", ", ", limit_num) 
      when "%RULE_NAME%" then
        opts[filtered_keyword] = "要対応の脆弱性を検知(サンプル)"
      when "%RULE_CATEGORY%" then
        opts[filtered_keyword] = "サンプルA"
      when "%RULE_TIMING%" then
        opts[filtered_keyword] = "毎日実行-時間指定(実行時間: 1時)"
      when "%RULE_COND%" then
        opts[filtered_keyword] = "脆弱性同期時(登録):脆弱性の登録が10件以上,\n" +
                                 "SRI指定値以上:対象の脆弱性の危険度が高以上,\n" +
                                 "カスタムカテゴリ限定: サンプルA"
      when "%RULE_ACTION%" then
        opts[filtered_keyword] = "メール送信(ユーザ),\n" +
                                 "対策予定日設定:対策予定日を7日後に設定する(上書き設定: 上書き)"
      when "%RULE_DATE%" then
        opts[filtered_keyword] = Date.today.strftime("%Y年%m月%d日")
      end
    end
    mail_body = SidfmdMailer.replace_keywords(message, opts)
    mail_title = SidfmdMailer.replace_keywords(mail_title, opts)
    data = {}
    data[:title] = mail_title
    data[:message] = mail_body
    render :layout => false, :json => data
  end

  #ルールのカテゴリ変更時にアラートを表示する
  def check_change_rule_category
    rule_params = params.permit(:new_category_name, :new_category_id, :trigger, :conds, :actions)
    new_category_name = rule_params[:new_category_name]
    new_category_id = rule_params[:new_category_id].to_s
    trigger = JSON.parse(rule_params[:trigger])
    conds = JSON.parse(rule_params[:conds])
    actions = JSON.parse(rule_params[:actions])
    all_cond_names = SidfmdRule::RULE_COND_NAMES
    all_action_names = SidfmdRule::RULE_ACTION_NAMES
    is_trigger_vulsync = (trigger["period"] == "vulsync")
    new_param = {}
    message = ""
    conf_cond_names = {}
    conf_action_names = {}  
    remove_users = []
    if is_trigger_vulsync then
      if trigger["hostname"] != "" then
        message += "タイミング設定に設定されているホストが削除されます。"
        trigger["hostname"] = ""
      end
      trigger["category"] = new_category_id
    end
    del_category_message = ""
    modify_category_items = []
    conds.each_key do |param_name|
      if(param_name == "category-match" || param_name == "category-unmatch" || param_name == "host-match" || param_name == "host-unmatch") &&
        new_category_id != "all"
      then
        if param_name == "category-match" || param_name == "category-unmatch" then
          if is_trigger_vulsync && trigger["hostname"] == '' && trigger["category"] == 'all' then
            #タイミングが脆弱性同期時で且つカテゴリが「全体」で且つホストが設定されていなければ
            #カテゴリ限定・カテゴリ除外は削除対象としない
            next
          else
            if !is_trigger_vulsync then
              system_cats = SidfmdCategory.where(:id => conds[param_name]).where(:is_system => true)
              if system_cats.any? then
                #タイミングが脆弱性同期時でなく、システムカテゴリがある場合は、カスタムカテゴリのみを削除
                if SidfmdCategory.where(:id => conds[param_name]).where(:is_system => false).any? then
                  del_category_message += "「" + all_cond_names[param_name] + "」" + "から、カスタムカテゴリが削除されます。"
                  modify_category_items << param_name
                end
                conds[param_name] = system_cats.order(:id).pluck(:id)
                next
              end
            end
          end
        end
        conf_cond_names[param_name] = "「" + all_cond_names[param_name] + "」"
        conds.delete(param_name)
      end
    end
    if conf_cond_names.size > 0 then
      message += "また、" if message != ""
      message += "条件に設定されている " + conf_cond_names.values.join(", ") + " が削除されます。" 
    end
    if del_category_message.size > 0 then
      if !message.index("条件に設定されている") then
        message += "また、" if message != ""
        message += "条件に設定されている "
      end
      message += del_category_message
    end
    actions.each_key do |param_name|
      if param_name == "send-mail-category" || param_name == "send-mail-user" then
        mail_data = actions[param_name]
        delete_flg = false 
        if mail_data["sender"]["userid"] && new_category_id != "all" then
          user_cats = User.find(mail_data["sender"]["userid"]).sidfmd_categories.map(&:id)
          if !user_cats.include?(new_category_id.to_i) then 
            conf_action_names[param_name] = "「" + all_action_names[param_name] + "」"
            actions.delete(param_name)
            delete_flg = true
          end
        end
        if param_name == "send-mail-user" && !delete_flg && new_category_id != "all" && mail_data["sendby"]["ids"] then
          ids = mail_data["sendby"]["ids"]
          if ids.size == 1 then
            user_cats = User.find(ids[0]).sidfmd_categories.map(&:id)
            if !user_cats.include?(new_category_id.to_i) then
              conf_action_names[param_name] = "「" + all_action_names[param_name] + "」"
              actions.delete(param_name)
            end
          elsif ids.size > 1 then
            new_category_id = new_category_id.to_i
            User.where(:id => ids).each do |user|
              user_cats = user.sidfmd_categories.map{ |c| c.id }
              if !user_cats.include?(new_category_id) then
                remove_users << user.id 
              end
            end
            if remove_users.size == ids.size then
              conf_action_names[param_name] = "「" + all_action_names[param_name] + "」"
              actions.delete(param_name)
              remove_users = []
            elsif remove_users.size > 0 then
              actions[param_name]["sendby"]["ids"] = ids - remove_users
            end
          end 
        end
      end
    end
    if conf_action_names.size > 0 then
      message += "加えて、" if message != ""
      message += "操作に設定されている" + conf_action_names.values.join(", ") + "が削除されます。"
    end
    if remove_users.size > 0 then
      message += " #{all_action_names['send-mail-user']}の操作の送信先に設定されている#{remove_users.size}名のユーザは、変更するカテゴリに属さないユーザのため、送信先から削除されます。"
    end
    if message != "" then
      message = "このカテゴリの選択に関連する項目が設定されています。変更に進むと、" + message + "よろしいですか?"
      new_param[:message] = message
      new_param[:trigger] = trigger
      new_param[:cond] = conds
      new_param[:action] = actions
      new_param[:del_conds] = conf_cond_names.keys
      new_param[:del_actions] = conf_action_names.keys
      new_param[:is_alert] = true
      new_param[:remove_users] = remove_users if remove_users.size > 0
      new_param[:modify_category_items] = modify_category_items
    else
      new_param[:is_alert] = false
      new_param[:trigger] = trigger
    end
    render :layout => false, :json => new_param
  end

  #カテゴリ設定・ホスト設定・ユーザ管理画面で、カテゴリ・ホスト・ユーザ削除時にルールをチェックし、アラート表示+ルールの内容を変更
  def api_check_related_rules 
    f_params = params.permit(:page_name, :delete_id, :action_idx, :user_ids, :host_ids, :hostnames, :category_ids, :hostid)
    page_name = f_params[:page_name]
    delete_id = (f_params[:delete_id] || 0).to_i
    trigger_names = SidfmdRule::RULE_TRIGGER_NAMES
    cond_names = SidfmdRule::RULE_COND_NAMES
    action_names = SidfmdRule::RULE_ACTION_NAMES
    delete_rule_ids = []
    delete_cond_ids = []
    delete_action_ids = []
    rule_messages = {}
    if page_name == "hostadmin" then #ホスト設定ページでのチェック
      action_idx = f_params[:action_idx]
      if action_idx == "category_change" || action_idx == "bulk_category_change" || action_idx == "category_change_by_import" then #ホストのカテゴリ変更時
        get_del_info = ->(host_ids, category_ids, delete_rule_ids, delete_cond_ids, host_cond_ids, del_cond_names, host_cond_hostnames){
          #triggerにホストが設定されているかチェック
          #ホストのカテゴリがなくなるならルールを削除
          if category_ids.empty? then
            SidfmdRuleTVulsync.where(:sidfmd_host_id => host_ids).each do |vulsync_trigger|
              rule = vulsync_trigger.sidfmd_rule_trigger.sidfmd_rule
              delete_rule_ids << rule.id
              rule_messages[rule.name] = {
                :item_name => "ルール", 
                :content => "ルールが削除されます。",
                :reason => "カスタムカテゴリに属さないホストをタイミングに設定できないため。"
              }
            end
          else
            SidfmdRuleTVulsync.where(:sidfmd_host_id => host_ids).where.not(:sidfmd_category_id => nil).each do |vulsync_trigger|
              #変更するカテゴリが設定されているカテゴリに含まれていなければルール削除
              if !category_ids.include?(vulsync_trigger.sidfmd_category_id) then
                rule = vulsync_trigger.sidfmd_rule_trigger.sidfmd_rule
                if !delete_rule_ids.include?(rule.id) then 
                  delete_rule_ids << rule.id
                  rule_messages[rule.name] = {
                    :item_name => "ルール", 
                    :content => "ルールが削除されます。",
                    :reason => "タイミングに設定しているホストが、タイミングの対象カテゴリに属さなくなるため。"
                  }
                end
              end
            end
          end
          #ホスト限定・除外の条件にホストが設定されているかチェック
          SidfmdRuleCmHostSub.where(:sidfmd_host_id => host_ids).each do |host_cond|
            sub_cond = host_cond.sidfmd_rule_c_match_host
            cond = sub_cond.sidfmd_rule_cond
            rule = cond.sidfmd_rule
            if !delete_rule_ids.include?(rule.id) then
              #ルールのtriggerがvulsyncで且つカテゴリが全体でない、またはカスタムカテゴリ限定の条件があり、
              #変更するカテゴリが設定されているカテゴリに含まれていなければホストを条件から外す
              rule_conds = rule.sidfmd_rule_conds
              rule_match_cat_conds_cat_ids = []
              is_match_category_cond = false
              rule_conds.each do |c|
                detail = c.detail
                if detail.class.table_name == "sidfmd_rule_c_match_categories" &&
                  detail.inv == false
                then
                  is_match_category_cond = true
                  sub_cat_ids = detail.sidfmd_rule_cm_category_subs.map{|sub_c|
                                  sub_c.sidfmd_category_id
                                } 
                  rule_match_cat_conds_cat_ids = sub_cat_ids.select{|c_id| category_ids.include?(c_id)}
                end
              end
              if (rule.sidfmd_category_id && !category_ids.include?(rule.sidfmd_category_id))||
                 (!rule.sidfmd_category_id && is_match_category_cond && rule_match_cat_conds_cat_ids.size == 0)||
                 (category_ids.size == 0)
              then
                sub_cond_ids = SidfmdRuleCmHostSub.where(:sidfmd_rule_c_match_host_id => sub_cond.id).map{|sub_c| sub_c.id}
                host_cond_name = (sub_cond.inv ? cond_names['host-unmatch'] : cond_names['host-match'])
                #ホストが１つだけだったら条件を削除の対象にする  
                if sub_cond_ids.count == 1 then
                  #この条件を削除すると条件がなくなるならルールを削除
                  if rule_conds.size == 1 then
                    delete_rule_ids << rule.id
                    rule_messages[rule.name] = {
                      :item_name => "ルール", 
                      :content => "ルールが削除されます。",
                      :reason => "ルールの条件が「#{host_cond_name}」のみで、「#{host_cond_name}」の条件に設定されているホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                    }
                  else
                    delete_cond_ids << cond.id
                    del_cond_names[rule.name] = [] unless del_cond_names[rule.name]
                    del_cond_names[rule.name] << "「#{host_cond_name}」"
                  end
                else
                  host_cond_ids << host_cond.id
                  host_cond_hostnames[rule.name] = {} unless host_cond_hostnames[rule.name]
                  host_cond_hostnames[rule.name][host_cond_name] = [] unless host_cond_hostnames[rule.name][host_cond_name] 
                  t_host = SidfmdHost.find(host_cond.sidfmd_host_id)
                  host_cond_hostnames[rule.name][host_cond_name] << '"' + "#{t_host.ext_name ? (t_host.ext_name + '(ホスト名: ' + t_host.hostname + ')') : t_host.hostname}" + '"'
                  if (sub_cond_ids - host_cond_ids).size == 0 then
                    #全てのホストが条件から外れるなら条件を削除
                    delete_cond_ids << cond.id
                    host_cond_ids = host_cond_ids - sub_cond_ids
                    host_cond_hostnames.delete(rule.name)
                    del_cond_names[rule.name] = [] unless del_cond_names[rule.name]
                    del_cond_names[rule.name] << "「#{host_cond_name}」"
                  end 
                end
              end
            end
          end
          r = {
            :delete_rule_ids => delete_rule_ids,
            :delete_cond_ids => delete_cond_ids,
            :host_cond_ids => host_cond_ids,
            :del_cond_names => del_cond_names,
            :host_cond_hostnames => host_cond_hostnames
          }
        }
        host_cond_ids = []
        del_cond_names = {}
        host_cond_hostnames = {}
        if action_idx == "category_change" then
          host_ids = JSON.parse(f_params[:host_ids])
          category_ids = JSON.parse(f_params[:category_ids])
          del_info = get_del_info.(host_ids, category_ids, delete_rule_ids, delete_cond_ids, host_cond_ids, del_cond_names, host_cond_hostnames)
          delete_rule_ids = del_info[:delete_rule_ids]
          delete_cond_ids = del_info[:delete_cond_ids]
          host_cond_ids = del_info[:host_cond_ids]
          del_cond_names = del_info[:del_cond_names]
          host_cond_hostnames = del_info[:host_cond_hostnames]
        elsif action_idx == "category_change_by_import" then
          hostname_with_categorynames = JSON.parse(f_params[:hostnames])
          hostname_with_categorynames.each do |hostname, category_names| 
            host_id = SidfmdHost.where(:hostname => hostname).map{|h| h.id}#get_del_infoに渡す引数が配列
            category_ids = SidfmdCategory.where(:name => category_names).where(:is_system => false).map(&:id)
            get_del_info.(host_id, category_ids, delete_rule_ids, delete_cond_ids, host_cond_ids, del_cond_names, host_cond_hostnames)
          end  
        else
          hostnames = JSON.parse(f_params[:hostnames])
          host_ids = SidfmdHost.where(:hostname => hostnames).map{|h| h.id}
          category_ids = JSON.parse(f_params[:category_ids])
          del_info = get_del_info.(host_ids, category_ids, delete_rule_ids, delete_cond_ids, host_cond_ids, del_cond_names, host_cond_hostnames)
          delete_rule_ids = del_info[:delete_rule_ids]
          delete_cond_ids = del_info[:delete_cond_ids]
          host_cond_ids = del_info[:host_cond_ids]
          del_cond_names = del_info[:del_cond_names]
          host_cond_hostnames = del_info[:host_cond_hostnames]
        end
        match_name = cond_names['host-match']
        unmatch_name = cond_names['host-unmatch']
        host_cond_hostnames.each do |rule_name, cond_data|
          message = ""
          match_hostnames = cond_data[match_name]
          unmatch_hostnames = cond_data[unmatch_name]
          if match_hostnames then 
            message += "「#{match_name}」から #{match_hostnames.join(', ')} が削除されます。"
          end
          if unmatch_hostnames then
            message += "「#{unmatch_name}」から #{unmatch_hostnames.join(', ')} が削除されます。"
          end
          rule_messages[rule_name] = {
            :item_name => "条件",
            :content => message,
            :reason => "ホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
          }
        end
        #カテゴリ一括変更時にチェック。条件が全てなくなるならルール削除
        if action_idx == "bulk_category_change" || action_idx == "category_change_by_import" then
          checked_rule_ids = []
          delete_cond_ids.each do |cond_id|
            rule = SidfmdRuleCond.find(cond_id).sidfmd_rule
            rule_id = rule.id
            if !checked_rule_ids.include?(rule_id) then
              checked_rule_ids << rule_id
              cond_ids = rule.sidfmd_rule_conds.map{|c| c.id}
              if (cond_ids - delete_cond_ids).size == 0 then
                delete_rule_ids << rule_id
                delete_cond_ids = delete_cond_ids - cond_ids
                del_cond_names.delete(rule.name)
                rule_messages[rule.name] = {
                  :item_name => "ルール", 
                  :content => "ルールが削除されます。",
                  :reason => "「#{cond_names['host-unmatch']}」または「#{cond_names['host-match']}」に設定されている全てのホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                }
              end
            end
          end
        end
        del_cond_names.each do |rule_name, cond_names|
          if rule_messages[rule_name] then
            content = rule_messages[rule_name][:content] + "\n#{cond_names.join("と")}が削除されます。"
          else
            content = "#{cond_names.join("と")}が削除されます。"
          end
          rule_messages[rule_name] = {
            :item_name => "条件",
            :content => content,
            :reason => "ホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
          }
        end
        render :layout => false,
               :json => {
                 :delete_rule_ids => delete_rule_ids,
                 :delete_cond_ids => delete_cond_ids,
                 :delete_host_cond_ids => host_cond_ids,
                 :rule_messages => rule_messages,
                 :hostid => f_params[:hostid]
               } 
      else
        #ホスト削除時
        get_del_info = ->(delete_id, delete_rule_ids, host_cond_ids, host_cond_hostnames, rule_messages){
          #脆弱性同期時実行のタイミングで、対象ホストに削除するホストが設定されていれば、そのルールを削除の対象にする
          SidfmdRuleTVulsync.where(:sidfmd_host_id => delete_id).each do |vulsync_trigger|
            rule = vulsync_trigger.sidfmd_rule_trigger.sidfmd_rule
            delete_rule_ids << rule.id
            rule_messages[rule.name] = {
              :item_name => "ルール",
              :content => "ルールが削除されます。",
              :reason => "タイミングにホストが設定されているため。"
            }
          end
          #ホスト限定・除外に削除するホストが設定されていればそのホストを外す
          SidfmdRuleCmHostSub.where(:sidfmd_host_id => delete_id).each do |host_cond|
            sub_cond = host_cond.sidfmd_rule_c_match_host
            cond = sub_cond.sidfmd_rule_cond
            rule = cond.sidfmd_rule
            if !delete_rule_ids.include?(rule.id) then
              host_cond_name = (sub_cond.inv ? cond_names['host-unmatch'] : cond_names['host-match'])
              #ホストが１つだけだったら条件を削除の対象にする
              if SidfmdRuleCmHostSub.where(:sidfmd_rule_c_match_host_id => sub_cond.id).count == 1 then
                #この条件を削除すると条件がなくなるならルールを削除
                if rule.sidfmd_rule_conds.size == 1 then
                  delete_rule_ids << rule.id
                  rule_messages[rule.name] = {
                    :item_name => "ルール",
                    :content => "ルールが削除されます。",
                    :reason => "ルールの条件が「#{host_cond_name}」のみで、「#{host_cond_name}」の条件に設定されているホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                  }
                else
                  delete_cond_ids << cond.id
                  rule_messages[rule.name] = {
                    :item_name => "条件",
                    :content => "「#{host_cond_name}」が削除されます。",
                    :reason => "ホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                  }
                end
              else
              #ホストが２つ以上あればホスト限定・削除から対象のホストを外す
                host_cond_ids << host_cond.id
                t_host = SidfmdHost.find(host_cond.sidfmd_host_id)
                rule_messages[rule.name] = {
                  :item_name => "条件",
                  :content => "「#{host_cond_name}」から" + ' "' + "#{t_host.ext_name ? (t_host.ext_name + '(ホスト名: ' + t_host.hostname + ')') : t_host.hostname}" + '" ' + "が削除されます。",
                  :reason => "ホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                }
              end
            end
          end
          r = {
            :delete_rule_ids => delete_rule_ids,
            :delete_cond_ids => delete_cond_ids,
            :delete_host_cond_ids => host_cond_ids,
            :rule_messages => rule_messages
          }
          return r
        }
        host_cond_ids = []
        host_cond_hostnames = {}
        if action_idx == "delete_one_host" then
          del_info = get_del_info.(delete_id, delete_rule_ids, host_cond_ids, host_cond_hostnames, rule_messages)
          delete_rule_ids = del_info[:delete_rule_ids]
          delete_cond_ids = del_info[:delete_cond_ids]
          host_cond_ids = del_info[:delete_host_cond_ids]
          rule_messages = del_info[:rule_messages]
        else
          host_ids = JSON.parse(f_params[:host_ids])
          host_ids.each do |host_id|
            get_del_info.(host_id.to_i, delete_rule_ids, host_cond_ids, host_cond_hostnames, rule_messages)
          end
        end
        render :layout => false, 
               :json => { 
                 :delete_rule_ids => delete_rule_ids, 
                 :delete_cond_ids => delete_cond_ids, 
                 :delete_host_cond_ids => host_cond_ids,
                 :rule_messages => rule_messages,
                 :hostid => f_params[:hostid]
               }
      end
    elsif page_name == "category" then #カテゴリ設定ページでカテゴリ削除時のチェック
      #ルールの「ルールを適用するカスタムカテゴリ」にカテゴリが設定されていれば、ルールを削除の対象にする
      SidfmdRule.where(:sidfmd_category_id => delete_id).each do |rule|
        delete_rule_ids << rule.id
        rule_messages[rule.name] = {
          :item_name => "ルール",
          :content => "ルールが削除されます。",
          :reason => "「ルールを適用するカスタムカテゴリ」にこのカテゴリが設定されているため。"
        }
      end
      #脆弱性同期時実行のタイミングで、対象カテゴリに削除するカテゴリが設定されていれば、そのルールを削除の対象にする
      SidfmdRuleTVulsync.where(:sidfmd_category_id => delete_id).each do |vulsync_trigger|
        rule = vulsync_trigger.sidfmd_rule_trigger.sidfmd_rule
        if !delete_rule_ids.include?(rule.id) then
          delete_rule_ids << rule.id
          rule_messages[rule.name] = {
            :item_name => "ルール",
            :content => "ルールが削除されます。",
            :reason => "タイミング設定にこのカテゴリが設定されているため。"
          }
        end
      end 
      #カテゴリ限定・除外に削除するカテゴリが設定されていればそのカテゴリを外す
      category_cond_ids = []
      host_cond_ids = []
      del_host_cond_names = {}
      host_cond_hostnames = {}
      SidfmdRuleCmCategorySub.where(:sidfmd_category_id => delete_id).each do |category|
        sub_cond = category.sidfmd_rule_c_match_category
        cond = sub_cond.sidfmd_rule_cond
        rule = cond.sidfmd_rule
        if !delete_rule_ids.include?(rule.id) then
          cat_cond_name = (sub_cond.inv ? cond_names['category-unmatch'] : cond_names['category-match'])
          #カテゴリが１つだったら条件を削除の対象にする
          if SidfmdRuleCmCategorySub.where(:sidfmd_rule_c_match_category_id => sub_cond.id).count == 1 then
            #この条件を削除すると条件がなくなるならルールを削除
            if rule.sidfmd_rule_conds.size == 1 then
              delete_rule_ids << rule.id
              rule_messages[rule.name] = {
                :item_name => "ルール",
                :content => "ルールが削除されます。",
                :reason => "ルールの条件が「#{cat_cond_name}」のみで、このカテゴリを削除すると条件数が0になるため。"
              }
            else
              delete_cond_ids << cond.id
              rule_messages[rule.name] = {
                :item_name => "条件",
                :content => "「#{cat_cond_name}」が削除されます。",
                :reason => "「#{cat_cond_name}」に設定されているカテゴリがこのカテゴリのみのため。"
              }
            end
          else
            #カテゴリが２つ以上あればカテゴリ限定・削除から対象のカテゴリを外す
            category_cond_ids << category.id
            rule_messages[rule.name] = {
              :item_name => "条件",
              :content => "「#{cat_cond_name}」からこのカテゴリが削除されます。",
              :reason => "このカテゴリが削除されるため。"
            }
            #カテゴリ限定の場合、ホスト限定・ホスト除外の条件が設定されているかチェック
            if !sub_cond.inv then
              rule_host_conds = SidfmdRuleCond.where(:sidfmd_rule_id => rule.id).select{|r_c| r_c.detail.class.table_name == "sidfmd_rule_c_match_hosts"}
              if rule_host_conds.size > 0 then 
                new_rule_cats = sub_cond.sidfmd_rule_cm_category_subs.map{|sub_c| sub_c.sidfmd_category_id}
                new_rule_cats.delete(delete_id) 
                rule_host_conds.each do |h_cond|
                  detail = h_cond.detail
                  host_cond_name = (detail.inv ? cond_names['host-unmatch'] : cond_names['host-match'])
                  subs = detail.sidfmd_rule_cm_host_subs
                  sub_h_ids = []
                  subs.each do |sub_h_cond|
                    host_cats = SidfmdCategoriesHost.where(:sidfmd_host_id => sub_h_cond.sidfmd_host_id).map{|h_c| h_c.sidfmd_category_id}
                    if (new_rule_cats - host_cats).size == new_rule_cats.size then
                      host_cond_ids << sub_h_cond.id
                      host_cond_hostnames[rule.name] = {} unless host_cond_hostnames[rule.name]
                      host_cond_hostnames[rule.name][host_cond_name] = [] unless host_cond_hostnames[rule.name][host_cond_name]
                      t_host = SidfmdHost.find(sub_h_cond.sidfmd_host_id)
                      host_cond_hostnames[rule.name][host_cond_name] << '"' + "#{t_host.ext_name ? (t_host.ext_name + '(ホスト名: ' + t_host.hostname + ')') : t_host.hostname}" + '"'
                      sub_h_ids << sub_h_cond.id
                      #全てのホストが条件から外れるなら条件を削除
                      if subs.size == sub_h_ids.size then
                        delete_cond_ids << h_cond.id
                        host_cond_ids = host_cond_ids - sub_h_ids
                        host_cond_hostnames.delete(rule.name)
                        del_host_cond_names[rule.name] = [] unless del_host_cond_names[rule.name]
                        del_host_cond_names[rule.name] << "「#{host_cond_name}」"
                      end
                    end
                  end
                end
              end
            end
          end
        end
      end
      #メール送信の操作に関連するカテゴリがあれば操作を削除の対象にする
      send_mail_cat_action_name = action_names['send-mail-category']
      SidfmdRuleAMailCategory.where(:sidfmd_category_id => delete_id).each do |mail_action|
        action = mail_action.sidfmd_rule_action
        rule = action.sidfmd_rule
        if !delete_rule_ids.include?(rule.id) then
          #この操作を削除すると操作がなくなるならルールを削除
          if rule.sidfmd_rule_actions.size == 1 then
            delete_rule_ids << rule.id
            rule_messages[rule.name] = {
              :item_name => "ルール",
              :content => "ルールが削除されます。",
              :reason => "ルールの操作が「#{send_mail_cat_action_name}」のみで、送信先カテゴリにこのカテゴリが設定されているため。"
            }
          else
            delete_action_ids << action.id
            if rule_messages[rule.name] then
              rule_m = rule_messages[rule.name]
              item_name = rule_m[:item_name] + "・操作"
              content = rule_m[:content] + "\n「#{send_mail_cat_action_name}」が削除されます。"
              reason = rule_m[:reason] + "\n「#{send_mail_cat_action_name}」の送信先カテゴリにこのカテゴリが設定されているため。"
            else
              item_name = "操作"
              content = "「#{send_mail_cat_action_name}」が削除されます。"
              reason = "「#{send_mail_cat_action_name}」の送信先カテゴリにこのカテゴリが設定されているため。"
            end
            rule_messages[rule.name] = {
              :item_name => item_name,
              :content => content,
              :reason => reason
            }
          end
        end
      end
      #カテゴリを削除するとカスタムカテゴリが無くなるホストを取得して、そのホストに関連するルールを削除・変更の対象にする
      target_host_ids = []
      related_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => delete_id).map{|a| a.sidfmd_host_id}
      related_host_ids.each do |h_id|
        h_cat_ids = SidfmdCategoriesHost.where(:sidfmd_host_id => h_id).map{|c| c.sidfmd_category_id}.uniq
        h_cat_ids.delete(delete_id)
        if h_cat_ids.size == 1 then
          target_host_ids << h_id
        end
      end
      #host_cond_ids = []
      #del_host_cond_names = {}
      #host_cond_hostnames = {}
      if target_host_ids.size > 0 then
        SidfmdRuleTVulsync.where(:sidfmd_host_id => target_host_ids).each do |t|
          rule = t.sidfmd_rule_trigger.sidfmd_rule
          delete_rule_ids << rule.id
          rule_messages[rule.name] = {
            :item_name => "ルール",
            :content => "ルールが削除されます。",
            :reason => "タイミング設定に設定されているホストのカスタムカテゴリが、このカテゴリのみのため。"
          }
        end
        SidfmdRuleCmHostSub.where(:sidfmd_host_id => target_host_ids).each do |host_cond|
          sub_cond = host_cond.sidfmd_rule_c_match_host
          cond = sub_cond.sidfmd_rule_cond
          rule = cond.sidfmd_rule
          if !delete_rule_ids.include?(rule.id) then
            #このホストを削除するとホストが無くなるなら条件削除
            sub_cond_ids = SidfmdRuleCmHostSub.where(:sidfmd_rule_c_match_host_id => sub_cond.id).map{|sub_c| sub_c.id}
            host_cond_name = (sub_cond.inv ? cond_names['host-unmatch'] : cond_names['host-match'])
            if sub_cond_ids.count == 1 then
              #条件を削除すると条件が１つもなくなるならルールを削除
              if rule.sidfmd_rule_conds.size == 1 then
                delete_rule_ids << rule.id
                rule_messages[rule.name] = {
                  :item_name => "ルール",
                  :content => "ルールが削除されます。",
                  :reason => "ルールの条件が「#{host_cond_name}」のみで、「#{host_cond_name}」の条件に設定されているホストのカテゴリが、このカテゴリのみのため。"
                }
              else
                delete_cond_ids << cond.id
                del_host_cond_names[rule.name] = [] unless del_host_cond_names[rule.name]
                del_host_cond_names[rule.name] << "「#{host_cond_name}」"
              end
            else
              #ホスト限定・削除条件からホストを抜く
              if !host_cond_ids.include?(host_cond.id) then
                host_cond_ids << host_cond.id
                host_cond_hostnames[rule.name] = {} unless host_cond_hostnames[rule.name]
                host_cond_hostnames[rule.name][host_cond_name] = [] unless host_cond_hostnames[rule.name][host_cond_name]
                t_host = SidfmdHost.find(host_cond.sidfmd_host_id)
                host_cond_hostnames[rule.name][host_cond_name] << '"' + "#{t_host.ext_name ? (t_host.ext_name + '(ホスト名: ' + t_host.hostname + ')') : t_host.hostname}" + '"'
              end
              #全てのホストが条件から外れるなら条件を削除
              if (sub_cond_ids - host_cond_ids).size == 0 then
                delete_cond_ids << cond.id unless delete_cond_ids.include?(cond.id)
                host_cond_ids = host_cond_ids - sub_cond_ids
                host_cond_hostnames.delete(rule.name)
                del_host_cond_names[rule.name] = [] unless del_host_cond_names[rule.name]
                del_host_cond_names[rule.name] << "「#{host_cond_name}」"
              end
            end
          end
        end
      end
      delete_rule_ids = delete_rule_ids.uniq

      match_name = cond_names['host-match']
      unmatch_name = cond_names['host-unmatch']
      host_cond_hostnames.each do |rule_name, cond_data|
        message = ""
        match_hostnames = cond_data[match_name]
        unmatch_hostnames = cond_data[unmatch_name]
        if match_hostnames then
          message += "「#{match_name}」から #{match_hostnames.join(', ')} が削除されます。"
        end
        if unmatch_hostnames then
          message += "「#{unmatch_name}」から #{unmatch_hostnames.join(', ')} が削除されます。"
        end
        if rule_messages[rule_name] then
          rule_m = rule_messages[rule_name]
          item_name = (rule_m[:item_name].index("操作") ? "条件・操作" : "条件")
          content = rule_m[:content] + "\n" + message
          reason = rule_m[:reason] + "\nホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
        else 
          item_name = "条件"
          content = message
          reason = "ホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
        end
        if match_hostnames && unmatch_hostnames then
          reason += "\nホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
        end
        rule_messages[rule_name] = {
          :item_name => item_name,
          :content => content,
          :reason => reason
        }
      end

      #カテゴリを無くすとカスタムカテゴリがなくなるホストがいる時のみチェック
      if target_host_ids.size > 0 then
        #条件を削除するとルールの条件がなくなるならルールを削除
        checked_rule_ids = []
        delete_cond_ids.each do |cond_id|
          rule = SidfmdRuleCond.find(cond_id).sidfmd_rule
          rule_id = rule.id
          if !checked_rule_ids.include?(rule_id) then
            checked_rule_ids << rule_id
            cond_ids = rule.sidfmd_rule_conds.map{|c| c.id}
            if (cond_ids - delete_cond_ids).size == 0 then
              delete_rule_ids << rule_id
              delete_cond_ids = delete_cond_ids - cond_ids
              del_host_cond_names.delete(rule.name)
              rule_messages[rule.name] = {
                :item_name => "ルール",
                :content => "ルールが削除されます。",
                :reason => "ルールの全ての条件にこのカテゴリが関係しており、このカテゴリを削除するとルールの条件設定が無効になるため。"
              }
            end
          end
        end
      end
  
      del_host_cond_names.each do |rule_name, cond_names|
        cond_names = cond_names.uniq
        if rule_messages[rule_name] then
          rule_m = rule_messages[rule_name]
          item_name = (rule_m[:item_name].index("操作") ? "条件・操作" : "条件")
          content = rule_m[:content] + "\n#{cond_names.join("と")}が削除されます。"
          reason = rule_m[:reason] + "\nホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
        else
          item_name = "条件"
          content = "#{cond_names.join("と")}が削除されます。"
          reason = "ホストのカテゴリが、ルールのカテゴリと一致しなくなるため。"
        end
        rule_messages[rule_name] = {
          :item_name => item_name,
          :content => content,
          :reason => reason
        }
      end
      render :layout => false,
             :json => {
               :delete_rule_ids => delete_rule_ids,
               :delete_cond_ids => delete_cond_ids,
               :delete_action_ids => delete_action_ids,
               :delete_category_cond_ids => category_cond_ids,
               :delete_host_cond_ids => host_cond_ids,
               :rule_messages => rule_messages
             }
    elsif page_name == "user" then
      action_idx = f_params[:action_idx]
      send_mail_cat_action_name = action_names['send-mail-category'];
      send_mail_user_action_name = action_names['send-mail-user'];
      #send_mail_user_action_ids = []
      if action_idx == "delete" then #ユーザ削除の場合
        #メール送信の操作の送信元が削除するユーザだったら、操作を削除の対象にする
        SidfmdRuleAMailCategory.where(:from_user_id => delete_id).each do |mail_action|
          action = mail_action.sidfmd_rule_action
          rule = action.sidfmd_rule
          #この操作を削除すると操作が無くなるならルールを削除
          if rule.sidfmd_rule_actions.size === 1 then
            delete_rule_ids << rule.id
            rule_messages[rule.name] = {
              :item_name => "ルール",
              :content => "ルールが削除されます。",
              :reason => "ルールの操作が「#{send_mail_cat_action_name}」のみで、「#{send_mail_cat_action_name}」の送信元にこのユーザが設定されているため。"
            }
          else
            delete_action_ids << action.id
            rule_messages[rule.name] = {
              :item_name => "操作",
              :content => "「#{send_mail_cat_action_name}」が削除されます。",
              :reason => "「#{send_mail_cat_action_name}」の送信元にこのユーザが設定されているため。"
            }
          end
        end
        SidfmdRuleAMailUser.where("from_user_id = ? OR user_id = ?", delete_id, delete_id).each do |mail_action|
          action = mail_action.sidfmd_rule_action
          rule = action.sidfmd_rule
          if !delete_rule_ids.include?(rule.id) then
            rule_actions = rule.sidfmd_rule_actions
            rule_actions_except_mail_users = rule_actions.select{|a| a.detail.class.table_name != "sidfmd_rule_a_mail_users"}
            #メール送信の操作の送信元が削除するユーザだったら、操作を削除の対象にする
            #この操作を削除すると操作がなくなるならルールを削除
            if mail_action.from_user_id == delete_id then
              if rule_actions_except_mail_users.size == 0 ||
                (rule_actions_except_mail_users.size == 1 && delete_action_ids.include?(rule_actions_except_mail_users[0].id))
              then
                delete_rule_ids << rule.id
                if rule_actions_except_mail_users.size == 0 then
                  reason = "ルールの操作が「#{send_mail_user_action_name}」のみで、「#{send_mail_user_action_name}」の送信元にこのユーザが設定されているため。"
                else
                  reason = "ルールの操作が「#{send_mail_user_action_name}」と「#{send_mail_cat_action_name}」で、その両方の送信元にこのユーザが設定されているため。"
                end 
                rule_messages[rule.name] = {
                  :item_name => "ルール",
                  :content => "ルールが削除されます。",
                  :reason => reason
                }
                if rule_actions_except_mail_users.size == 1 then
                  delete_action_ids.delete(rule_actions_except_mail_users[0].id)
                end
              else
                delete_action_ids << action.id
                if rule_messages[rule.name] then
                  rule_m = rule_messages[rule.name]
                  content = rule_m[:content] + "\n「#{send_mail_user_action_name}」が削除されます。"
                  reason = rule_m[:reason] + "\n「#{send_mail_user_action_name}」の送信元にこのユーザが設定されているため。"
                else
                  content = "「#{send_mail_user_action_name}」が削除されます。"
                  reason = "「#{send_mail_user_action_name}」の送信元にこのユーザが設定されているため。"
                end
                rule_messages[rule.name] = {
                  :item_name => "操作",
                  :content => content,
                  :reason => reason
                }
              end
            else
              #送信先に削除するユーザが設定されていれば、送信先から外す
              #送信先ユーザが削除するユーザのみだったら操作削除
              rule_actions = rule.sidfmd_rule_actions
              if rule_actions.select{|a| a.detail.class.table_name == "sidfmd_rule_a_mail_users"}.size == 1 then
                #この操作を削除すると操作が無くなるならルール削除
                if rule_actions.size == 1 then
                  delete_rule_ids << rule.id
                  rule_messages[rule.name] = {
                    :item_name => "ルール",
                    :content => "ルールが削除されます。",
                    :reason => "ルールの操作が「#{send_mail_user_action_name}」のみで、送信先ユーザにこのユーザのみが設定されているため。"
                  }
                elsif rule_actions_except_mail_users.size == 1 && delete_action_ids.include?(rule_actions_except_mail_users[0].id) then
                  delete_rule_ids << rule.id
                  delete_action_ids.delete(rule_actions_except_mail_users[0].id)
                  rule_messages[rule.name] = {
                    :item_name => "ルール",
                    :content => "ルールが削除されます。",
                    :reason => "ルールの操作が「#{send_mail_user_action_name}」と「#{send_mail_cat_action_name}」で、「#{send_mail_user_action_name}」の送信先ユーザにこのユーザのみが設定されており、「#{send_mail_cat_action_name}」の送信元にこのユーザが設定されているため。"
                  }
                else
                  delete_action_ids << action.id
                  if rule_messages[rule.name] then
                    rule_m = rule_messages[rule.name]
                    content = rule_m[:content] + "\n「#{send_mail_user_action_name}」が削除されます。"
                    reason = rule_m[:reason] + "\n「#{send_mail_user_action_name}」の送信先にこのユーザのみが設定されているため。"
                  else
                    content = "「#{send_mail_user_action_name}」が削除されます。"
                    reason = "「#{send_mail_user_action_name}」の送信先にこのユーザのみが設定されているため。"
                  end
                  rule_messages[rule.name] = {
                    :item_name => "操作",
                    :content => content,
                    :reason => reason
                  }
                end
              else
                delete_action_ids << action.id
                if rule_messages[rule.name] then
                  rule_m = rule_messages[rule.name]
                  content = rule_m[:content] + "\n「#{send_mail_user_action_name}」の送信先からこのユーザが削除されます。"
                  reason = rule_m[:reason] + "\n「#{send_mail_user_action_name}」の送信先にこのユーザが設定されているため。"
                else
                  content = "「#{send_mail_user_action_name}」の送信先からこのユーザが削除されます。"
                  reason = "「#{send_mail_user_action_name}」の送信先にこのユーザが設定されているため。"
                end
                rule_messages[rule.name] = {
                  :item_name => "操作",
                  :content => content,
                  :reason => reason
                }
              end
            end
          end
        end
        render :layout => false,
               :json => {
                 :delete_rule_ids => delete_rule_ids,
                 :delete_action_ids => delete_action_ids,
                 :rule_messages => rule_messages
               }
      elsif action_idx == "category_change" || action_idx == "bulk_category_change" then #ユーザのカテゴリ変更の場合
        #message_prefix = (action_idx == "category_change" ? "この" : "選択中の")
        user_ids = JSON.parse(f_params[:user_ids])
        category_ids = JSON.parse(f_params[:category_ids])
        #is_user_deleted_form_address = false 送信先ユーザから外すユーザがいるかどうか
        #メール送信の操作の送信元に変更するユーザが設定されていて、
        #ユーザのカテゴリがそのルールの「ルールを適用するカスタムカテゴリ」のカテゴリで無くなる場合、ルールを削除
        SidfmdRuleAMailCategory.where(:from_user_id => user_ids).each do |mail_action|
          action =  mail_action.sidfmd_rule_action
          rule = action.sidfmd_rule
          rule_category = rule.sidfmd_category_id
          if (rule_category && !category_ids.include?(rule_category)) || (category_ids.empty?) then
            #この操作を削除すると操作がなくなるならルールを削除
            if rule.sidfmd_rule_actions.size == 1 then
              delete_rule_ids << rule.id
              rule_messages[rule.name] = {
                :item_name => "ルール",
                :content => "ルールが削除されます。",
                :reason => "ルールの操作が「#{send_mail_cat_action_name}」のみで、「#{send_mail_cat_action_name}」の送信元に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
              }
            else
              delete_action_ids << action.id
              rule_messages[rule.name] = {
                :item_name => "操作",
                :content => "「#{send_mail_cat_action_name}」が削除されます。",
                :reason => "「#{send_mail_cat_action_name}」の送信元に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
              }
            end
          end
        end
        address_user_names = {}
        checked_rule_ids_for_message = []
        user_ids.each do |user_id|
          SidfmdRuleAMailUser.where("from_user_id = ? OR user_id = ?", user_id, user_id).each do |mail_action|
            action =  mail_action.sidfmd_rule_action
            rule = action.sidfmd_rule
            if !delete_rule_ids.include?(rule.id) then
              rule_actions = rule.sidfmd_rule_actions
              rule_category = rule.sidfmd_category_id
              rule_actions_except_mail_users = rule_actions.select{|a| a.detail.class.table_name != "sidfmd_rule_a_mail_users"}
              if (rule_category && !category_ids.include?(rule_category)) || (category_ids.empty?) then
                #メール送信(ユーザ)の送信元ユーザをチェック
                if user_ids.include?(mail_action.from_user_id) then
                  #この操作を削除すると操作がなくなるならルールを削除
                  if rule_actions_except_mail_users.size == 0 then
                    delete_rule_ids << rule.id
                    rule_messages[rule.name] = {
                      :item_name => "ルール",
                       :content => "ルールが削除されます。",
                       :reason => "ルールの操作が「#{send_mail_user_action_name}」のみで、「#{send_mail_user_action_name}」の送信元に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                    }
                  else
                    if rule_actions_except_mail_users.size == 1 && 
                      delete_action_ids.include?(rule_actions_except_mail_users[0].id)
                    then 
                      delete_rule_ids << rule.id
                      rule_messages[rule.name] = {
                        :item_name => "ルール",
                        :content => "ルールが削除されます。",
                        :reason => "ルールの操作が「#{send_mail_user_action_name}」と「#{send_mail_cat_action_name}」で、その両方の送信元に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                      }
                      delete_action_ids.delete(rule_actions_except_mail_users[0].id)
                    else
                      delete_action_ids << action.id
                      if !checked_rule_ids_for_message.include?(rule.id) then
                        if rule_messages[rule.name] then
                          rule_m = rule_messages[rule.name]
                          content = rule_m[:content] + "\n「#{send_mail_user_action_name}」が削除されます。"
                          reason = rule_m[:reason] + "\n「#{send_mail_user_action_name}」の送信元に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                        else
                          content = "「#{send_mail_user_action_name}」が削除されます。"
                          reason = "「#{send_mail_user_action_name}」の送信元に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                        end
                        rule_messages[rule.name] = {
                          :item_name => "操作",
                          :content => content,
                          :reason => reason
                        }
                        checked_rule_ids_for_message << rule.id
                      end
                    end
                  end
                else
                  #送信先チェック
                  #送信先ユーザがこのユーザのみなら操作削除
                  #この操作を削除すると操作がなくなるならルールを削除
                  rule_a_mail_users_actions = rule_actions.select{|a| a.detail.class.table_name == "sidfmd_rule_a_mail_users"}
                  if rule_actions_except_mail_users.size == 0 &&
                    rule_actions.size == 1 
                  then
                    delete_rule_ids << rule.id
                    rule_messages[rule.name] = {
                      :item_name => "ルール",
                      :content => "ルールが削除されます。",
                      :reason => "ルールの操作が「#{send_mail_user_action_name}」のみで、「#{send_mail_user_action_name}」の送信先に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                    }
                  elsif rule_a_mail_users_actions.size == 1 then
                    if rule_actions_except_mail_users.size == 1 &&
                      delete_action_ids.include?(rule_actions_except_mail_users[0].id) 
                    then
                      delete_rule_ids << rule.id
                      rule_messages[rule.name] = {
                        :item_name => "ルール",
                        :content => "ルールが削除されます。",
                        :reason => "ルールの操作が「#{send_mail_user_action_name}」と「#{send_mail_cat_action_name}」で、「#{send_mail_user_action_name}」の送信先に設定されているユーザのカテゴリと、「#{send_mail_cat_action_name}」の送信元に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                      }
                    else
                      delete_action_ids << action.id
                      if rule_messages[rule.name] then
                        rule_m = rule_messages[rule.name]
                        content = rule_m[:content] + "\n「#{send_mail_user_action_name}」が削除されます。"
                        reason = rule_m[:reason] + "\n「#{send_mail_user_action_name}」の送信先に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                      else
                        content = "「#{send_mail_user_action_name}」が削除されます。"
                        reason = "「#{send_mail_user_action_name}」の送信先に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"  
                      end
                      rule_messages[rule.name] = {
                        :item_name => "操作",
                        :content => content,
                        :reason => reason
                      }
                    end
                  else
                    delete_action_ids << action.id
                    if (rule_a_mail_users_actions.map{|a| a.id} - delete_action_ids).size == 0 then
                      address_user_names.delete(rule.name)
                      if rule_messages[rule.name] then
                        rule_m = rule_messages[rule.name]
                        content = rule_m[:content] + "\n「#{send_mail_user_action_name}」が削除されます。"
                        reason = rule_m[:reason] + "\n「#{send_mail_user_action_name}」の送信先に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                      else
                        content = "「#{send_mail_user_action_name}」が削除されます。"
                        reason = "「#{send_mail_user_action_name}」の送信先に設定されているユーザのカテゴリが、ルールのカテゴリと一致しなくなるため。"
                      end
                      rule_messages[rule.name] = {
                        :item_name => "操作",
                        :content => content,
                        :reason => reason
                      }
                    else
                      target_user = User.find(mail_action.user_id)  
                      address_user_names[rule.name] = [] unless address_user_names[rule.name]
                      address_user_names[rule.name] << '"' + target_user.lastname + target_user.firstname + '"'
                    end
                  end
                end
              end
            end
          end
        end
        #カテゴリ一括変更の時にチェック。操作を削除するとルールの操作が無くなるならルールを削除
        if action_idx == "bulk_category_change" then
          checked_rule_ids = []
          SidfmdRuleAction.where(:id => delete_action_ids).each do |action|
            rule = action.sidfmd_rule
            rule_id = rule.id
            if !checked_rule_ids.include?(rule_id) then
              checked_rule_ids << rule_id
              rule_action_ids = rule.sidfmd_rule_actions
                                    .map{|a| a.id}
              if (rule_action_ids - delete_action_ids).size == 0 then
                delete_rule_ids << rule.id
                delete_action_ids = (delete_action_ids - rule_action_ids)
                address_user_names.delete(rule.name)
                rule_messages[rule.name] = {
                  :item_name => "ルール",
                  :content => "ルールが削除されます。",
                  :reason => "ルールの操作が「#{send_mail_user_action_name}」または「#{send_mail_cat_action_name}」、もしくはその両方で、ユーザのカテゴリを変更すると全ての操作の設定が無効になるため。"
                }
              end
            end
          end
        end
        address_user_names.each do |rule_name, user_names|
          if rule_messages[rule_name] then
            rule_m = rule_messages[rule_name]
            content = rule_m[:content] + "\n「#{send_mail_user_action_name}」の送信先から #{user_names.join(', ')} が削除されます。"
            reason = rule_m[:reason] + "\nユーザのカテゴリとルールのカテゴリが一致しなくなるため。"
          else
            content = "「#{send_mail_user_action_name}」の送信先から #{user_names.join(', ')} が削除されます。"
            reason = "ユーザのカテゴリとルールのカテゴリが一致しなくなるため。"
          end
          rule_messages[rule_name] = {
            :item_name => "操作",
            :content => content,
            :reason => reason
          }
        end
        render :layout => false,
               :json => {
                 :delete_rule_ids => delete_rule_ids,
                 :delete_action_ids => delete_action_ids,
                 :rule_messages => rule_messages
               }
      end 
    end
  end

  #カテゴリ削除時に、対象のカテゴリに関連するホストで、
  #所属するカスタムカテゴリが無くなるホストを返す
  def api_check_category_related_hosts
    f_params = params.permit(:category_id)
    category_id = f_params[:category_id].to_i
    r = SidfmdHost.where(
      :id => SidfmdCategoriesHost.group(:sidfmd_host_id)
                                 .having('COUNT(DISTINCT sidfmd_category_id) = 2')
                                 .select(:sidfmd_host_id)
    ).where(
      :id => SidfmdCategoriesHost.where(:sidfmd_category_id => category_id).select(:sidfmd_host_id)
    ).where(:flg_delete => false)
     .order(%|
       CASE 
       WHEN ext_name IS NULL THEN
         hostname
       ELSE
         ext_name
       END
     |)
     .order(:hostname)
     .select(:hostname, :ext_name)
    render :layout => false, :json => r
  end

  #カテゴリ削除時に、対象のカテゴリに関連するユーザで、
  #所属するカスタムカテゴリが無くなるユーザ返す
  def api_check_category_related_users
    f_params = params.permit(:category_id)
    category_id = f_params[:category_id].to_i
    r = User.where(
      :id => SidfmdCategoriesUser.group(:user_id)
                                 .having('COUNT(DISTINCT sidfmd_category_id) = 1')
                                 .select(:user_id)
    ).where(
      :id => SidfmdCategoriesUser.where(:sidfmd_category_id => category_id).select(:user_id)
    ).order(:login)
     .select(:login, :firstname, :lastname)
    render :layout => false, :json => r
  end

  def api_get_rule_selectable_users 
    f_params = params.permit(:category_id)
    category_id = f_params[:category_id]
    users_info = []
    if @role_admin then
      admin_id = Role.find_by(name: 'SIDfmVM管理者').id
      admin_users = User.joins("LEFT OUTER JOIN members ON members.project_id = #{@sidfm_project.id} AND members.user_id = users.id")
                        .joins("LEFT OUTER JOIN member_roles ON member_roles.member_id = members.id")
                        .joins("LEFT OUTER JOIN roles ON roles.id = member_roles.role_id").where("roles.id = ?", admin_id)
      if category_id == "all" then
        userids = (SidfmdCategoriesUser.pluck(:user_id) + admin_users.pluck(:id)).uniq
      else
        userids = SidfmdCategoriesUser.where(:sidfmd_category_id => category_id).pluck(:user_id)
      end
    else
      if category_id == "all" || !@user_category_ids.include?(category_id.to_i) then 
        userids = [] #ユーザのカテゴリが操作中に変更されない限りは、入らないスコープ
      else
        userids = SidfmdCategoriesUser.where(:sidfmd_category_id => category_id).pluck(:user_id)
      end
    end
    users = User.where(:id => userids).order(:login => :asc)
    users.each do |user|
      users_info.push({
          :user_id => user.id,
          :login => user.login,
          :lastname => user.lastname,
          :firstname => user.firstname,
          :address => user.mail,
          :categories => if @role_admin then
                           user.sidfmd_categories.map{|c| c.name}.sort
                         else
                           user.sidfmd_categories.select{|c| @user_category_ids.include?(c.id)}.map{|c| c.name}.sort
                         end
      })
    end
    render :layout => false, :json => users_info
  end

  #ユーザの登録・変更をするときに、ユーザ情報が登録可能かどうかを返す
  #ユーザ管理画面で使用する
  def api_check_user_info_registerable
    f_params = params.permit(:user_id, :login, :mail_address, :password, :func)
    user_id = f_params[:user_id].to_i
    login = f_params[:login]
    address = f_params[:mail_address]
    func = f_params[:func]
    password = f_params[:password]
    err = {}
    if !login.match(/\A[a-z0-9_\-@\.]*\z/i) then
      err[:login] = "ログイン名に使用できない文字が含まれています。\n※半角英数字及び _.-@ のみ使用可能です"
    elsif (func != "mod" && User.find_by_login(login)) or (func == "mod" && User.where.not(:id => user_id).where('LOWER(login) = ?', login.downcase).first) then
      err[:login] = "同じログイン名が登録されています"
    end
    if !address.match(/\A[a-zA-Z0-9_\#!$%&`'*+\-{|}~^\/=?\.]+@[a-zA-Z0-9][a-zA-Z0-9\.-]+\z/) then
      err[:address] = "メールアドレスに誤りがあります"
    elsif func != "mod" && EmailAddress.find_by(:address => address) then
      err[:address] = "同じメールアドレスが登録されています"
    elsif (func != "mod" && User.joins(:email_addresses).where('LOWER(email_addresses.address) = ?', address.downcase).first) or 
          (func == "mod" && User.joins(:email_addresses).where('LOWER(email_addresses.address) = ?', address.downcase).where.not('users.id = ?', user_id).exists?) then
      err[:address] = "同じメールアドレスが登録されています"
    elsif address.size > User::MAIL_LENGTH_LIMIT then
      err[:address] = "メールアドレスは#{User::MAIL_LENGTH_LIMIT}文字以内で入力してください。"
    end
    if func != 'mod' || password.length > 0 then
      user_obj = User.new()
      validate_password_symbols = user_obj.validate_password_symbols password
      validate_password_char_types = user_obj.validate_password_char_types password
      validate_password_same_as_username = user_obj.validate_password_same_as_username password, login
      validate_password_includes_username = user_obj.validate_password_includes_username password, login
      if !validate_password_symbols || !validate_password_char_types || !validate_password_same_as_username || !validate_password_includes_username then
        err[:password] = 'パスワードポリシーに違反しています。'
      end
    end
    render :layout => false, :json => err
  end

  #ホストの表示名を返す
  def api_get_hosts_display_name
    f_params = params.permit(:hostnames)
    hostnames = JSON.parse(f_params[:hostnames])
    data = {}
    SidfmdHost.where(:hostname => hostnames).each do |host|
      data[host.hostname] = host.ext_name
    end
    render :layout => false, :json => data
  end

  #productからそのproductidに紐づくversionを返す
  def api_get_product_versions
    f_params = params.permit(:product_id, :lproduct_id)
    prod_id = f_params[:product_id]
    lprod_id = f_params[:lproduct_id]
    versions = []
    prod = nil
    lversions = []
    lprod = nil
    if prod_id then
      prod = SidfmdProduct.find_by(:pid => prod_id)
      if prod then
        versions = prod.sidfmd_versions.where.not(:version => 'ALL').order(:position => :desc)
        versions = versions.sort{|a, b|
          r = (a.is_support_end ? 1 : 0) <=> (b.is_support_end ? 1 : 0) 
        }
      end
    end
    if lprod_id then
      lprod = SidfmdLproduct.where(:flg_enable => true).where(:id => lprod_id).first
      if lprod then
        lversions = lprod.sidfmd_lversions.where(:flg_enable => true).order(:priority)
      end  
    end
    if prod then
      data = {:product => prod, :versions => versions}
    else
      data = {:lproduct => lprod, :lversions => lversions}
    end
    render :layout => false, :json => data
  end

  #使用できないキーワードが含まれているかチェックする
  def api_check_keywords
    f_params = params.permit(:item_name, :text)
    item_name = f_params[:item_name]
    text = f_params[:text]
    is_pass = false
    if item_name == 'syslog_message' then
      is_pass = SidfmdRuleASendSyslog.validate_keywords text
    end
    render :layout => false, :json => is_pass
  end

  #HostView, VulViewのupdate
  def update_view
    f_params = params.permit(:host_ids, :html_ids)
    host_ids = JSON.parse(f_params[:host_ids] || '[]')
    html_ids = JSON.parse(f_params[:html_ids] || '[]')
    target_host_ids = host_ids.shift(100)
    executed_size = target_host_ids.size
    target_html_ids = html_ids.shift(100)
    executed_size += target_html_ids.size
    SidfmdHostView.sync_data(target_host_ids.uniq) if target_host_ids.any?
    SidfmdVulView.sync_data(target_html_ids.uniq) if target_html_ids.any?
    r = {
      :executed_size => executed_size,
      :unexecuted_host_ids => host_ids,
      :unexecuted_html_ids => html_ids
    }
    render :layout => false, :json => r
  end

  #SidfmdHostView.relate_htmlidsを取得する
  #SidfmdHostView.relate_htmlidsが処理に時間がかかるため、可能な部分では非同期でhtmlidsを取得する
  def api_get_host_view_related_htmlids
    f_params = params.permit(:host_ids, :hostnames)
    host_ids = []
    if f_params[:host_ids] then
      host_ids = JSON.parse(f_params[:host_ids])
    end
    if f_params[:hostnames] then
      hostnames = JSON.parse(f_params[:hostnames])
      host_ids += SidfmdHost.where(:hostname => hostnames).pluck(:id)
    end 
    target_html_ids = SidfmdHostView.relate_htmlids(host_ids)
    render :layout => false, :json => target_html_ids
  end

  def api_get_host_ids_by_hostnames
    f_params = params.permit(:hostnames)
    hostnames = JSON.parse(f_params[:hostnames])
    host_ids = SidfmdHost.where(:hostname => hostnames).pluck(:id)
    render :layout => false, :json => host_ids
  end

  #対象のホストのHostViewのOSのみをupdateする 
  def api_update_host_view_os
    f_params = params.permit(:host_ids)
    host_ids = JSON.parse(f_params[:host_ids])
    target_host_ids = host_ids.shift(100)
    executed_size = target_host_ids.size
    SidfmdHostView.update_os(target_host_ids)
    r = { :executed_size => executed_size, :unexecuted_host_ids => host_ids }
    render :layout => false, :json => r
  end

  #脆弱性に関連するホストを取得
  def api_get_vul_hosts
    f_params = params.permit(:html_id)
    html_id = f_params[:html_id].to_i
    html_id = html_id.to_i if html_id
    vul_hosts = if @role_admin then
                  SidfmdVvHost.where(
                    :sidfmd_vul_view_id => SidfmdVulView.where(:html_id => html_id).select(:id)
                  )
                else
                  SidfmdVvHost.where(
                    :sidfmd_vul_view_id => SidfmdVulView.where(:html_id => html_id).select(:id)
                  ).where(:sidfmd_host_id => @user_host_ids)
                end
    vul_hosts = vul_hosts.distinct(:sidfmd_host_id)
                         .select(
                           :sidfmd_host_id,                  
                           :hostname,
                           :ext_name,
                           :display_name ,
                           'LOWER(display_name COLLATE "C")'
                         )
                         .order('LOWER(display_name COLLATE "C")')
    render :layout => false, :json => { :hosts => vul_hosts, :html_id => html_id }
  end  

  #脆弱性もしくはホストに紐づくカテゴリを取得
  def api_get_hosts_vuls_related_categories
    f_params = params.permit(:host_id, :html_id, :page_name)
    host_id = f_params[:host_id]
    html_id = f_params[:html_id]
    host_id = host_id.to_i if host_id
    html_id = html_id.to_i if html_id
    page_name = f_params[:page_name]
    data = []
    #Viewで使うので、Viewから取得する
    if host_id then
      if @role_admin || @user_host_ids.include?(host_id) then
        host = SidfmdHostView.find_by(:sidfmd_host_id => host_id) 
        if host then
          t_cat = SidfmdCategory.find_by(:id => host.system_category_id)
          data << { 
            :system_category_name => host.system_category_name,
            :desc => t_cat ? t_cat.desc : '',
            :url => case page_name 
                    when 'vulnerabilities' then
                      url_for(
                        :controller => :sidfm_vulnerabilities,
                        :action => :index,
                        :'pages[category_id]' => host.system_category_id
                      )
                    when 'hosts' then
                      url_for(
                        :controller => :sidfm_hosts,
                        :action => :index,
                        :'pages[category_id]' => host.system_category_id
                      )
                    when 'vulnerability' then
                      url_for(
                        :controller => :sidfm_vulnerability,
                        :action => :index,
                        :id => html_id,
                        :'pages[category_id]' => host.system_category_id
                      )
                    end
          }

          cats = if @role_admin then
                   SidfmdHvCategory.where(:sidfmd_host_view_id => host.id)
                 else
                   SidfmdHvCategory.where(:sidfmd_host_view_id => host.id)
                                   .where(:sidfmd_category_id => @user_category_ids)
                 end
          cats.order('LOWER(name COLLATE "C")').each do |cat|
            t_cat = SidfmdCategory.find_by(:id => cat.sidfmd_category_id)
            data << {
              :custom_category_name => cat.name,
              :desc => t_cat ? t_cat.desc : '',
              :url => case page_name 
                      when 'vulnerabilities' then 
                        url_for(
                          :controller => :sidfm_vulnerabilities,
                          :action => :index,
                          :'pages[category_id]' => cat.sidfmd_category_id
                        )
                      when 'hosts' then
                        url_for(
                          :controller => :sidfm_hosts,
                          :action => :index,
                          :'pages[category_id]' => cat.sidfmd_category_id
                        )
                      when 'vulnerability' then
                        url_for(
                          :controller => :sidfm_vulnerability,
                          :action => :index,
                          :id => html_id,
                          :'pages[category_id]' => cat.sidfmd_category_id
                        )
                      end
            }
          end
        end     
      end
    elsif html_id then
      data = SidfmdVvCategory.where(
               :sidfmd_vul_view_id => SidfmdVulView.where(:html_id => html_id).select(:id)
             )
      if !@role_admin then
        data = data.where('is_system = true OR sidfmd_category_id IN (?)', @user_category_ids)
        if data.where(:sidfmd_category_id => @user_category_ids).count == 0 then
          data = data.where(:id => 0) #カスタムカテゴリが属するものがなければ表示させない
        end
      end
  
      data = data.order(
        "CASE WHEN is_system IS TRUE THEN sidfmd_category_id ELSE NULL END"
      ) 
      .order('LOWER(name COLLATE "C")')
      .select(
        :sidfmd_category_id,
        :name
      )
      .map{|cat|
        t_cat = SidfmdCategory.find_by(:id => cat.sidfmd_category_id)
        {
          :custom_category_name => cat.name,
          :desc => t_cat ? t_cat.desc : '',
          :url => if page_name == 'vulnerabilities' then 
                    url_for(
                      :controller => :sidfm_vulnerabilities,
                      :action => :index,
                      :'pages[category_id]' => cat.sidfmd_category_id
                    )
                  elsif page_name == 'hosts' then
                    url_for(
                      :controller => :sidfm_hosts,
                      :action => :index,
                      :'pages[category_id]' => cat.sidfmd_category_id
                    )
                  end 
        }
      }
    end
    r = {
      :host_id => host_id, 
      :html_id => html_id,
      :categories => data
    }
    render :layout => false, :json => r
  end

  def api_get_vul_view_heat_map_data
    f_params = params.permit(:index, :host_id, :category_id)
    index = f_params[:index].to_sym
    host_id = f_params[:host_id]
    category_id = f_params[:category_id].to_i
    data = []
    cat_host_ids = nil
    vul_view = SidfmdVulView.user_vul_view(@role_admin, @user_host_ids)
    if category_id > 0 then
      vul_view = vul_view.where(
        :id => SidfmdVvCategory.where(:sidfmd_category_id => category_id).select(:sidfmd_vul_view_id)
      )
    end
    if category_id != 0 && SidfmdCategory.not_system.where(:id => category_id).any? then
      if @role_admin then
        cat_host_ids = SidfmdHost.where(
          :id => SidfmdCategoriesHost.where(:sidfmd_category_id => category_id).select(:sidfmd_host_id)
        ).pluck(:id)
      else
        cat_host_ids = SidfmdHost.where(
          :id => SidfmdCategoriesHost.where(:sidfmd_category_id => category_id)
                                     .where(:sidfmd_host_id => @user_host_ids)
                                     .select(:sidfmd_host_id)
        ).pluck(:id)
      end
    else
      if !@role_admin then
        cat_host_ids = @user_host_ids
      end
    end

    if host_id.size > 0 then
      #ホスト詳細画面の場合
      vul_view = vul_view.where(
                   :id => SidfmdVvHost.where(:sidfmd_host_id => host_id)
                                      .distinct
                                      .select(:sidfmd_vul_view_id)
                 )
      cat_host_ids = [ host_id ]
    end
    vul_view.order(:html_id).order('LOWER(title COLLATE "C")').select(
      :id,
      :title,
      :html_id,
      :cvss_label,
      :cvss_score
    ).each do |vul|
      if cat_host_ids then
        vul_scsids = SidfmdVvScsid.where(:sidfmd_vul_view_id => vul.id).where(:sidfmd_host_id => cat_host_ids)
      else
        vul_scsids = SidfmdVvScsid.where(:sidfmd_vul_view_id => vul.id)
      end
      if index == :sri then
        if vul_scsids.any? && SidfmdIssue::STATUS_UNFINISHED.include?(vul_scsids.minimum(:status_id)) then
          risk_id = vul_scsids.maximum(:sri)
          risk_name = SidfmdIssueSri::RISKS[risk_id]
        else
          risk_id = SidfmdIssueSri::VALUE_FIXED
          risk_name = SidfmdIssueSri::RISK_FIXED
        end
      else
        if vul_scsids.any? && SidfmdIssue::STATUS_UNFINISHED.include?(vul_scsids.minimum(:status_id)) then
          max_cvss = vul_scsids.where.not(:cvss => nil).order(:cvss).last
          max_cvss_score = max_cvss ? max_cvss.cvss : nil
          cvss_label = SidfmdIssueCve.labelling(max_cvss_score)
          risk_id = vul.cvss_label.to_s
          risk_name = SidfmdIssueCve.display_cvss(
            { :label => cvss_label, :score => max_cvss_score }
          )
        else
          risk_id = :fixed
          risk_name = SidfmdIssueCve::RISK_FIXED
        end
      end
      title = if vul.title.split(' / ').size != vul.title.split(' / ').uniq.size then
                vul.title.split(' / ').uniq.join(' / ')
              else
                vul.title.split(' / ').first
              end
      data << {
        :riskid => risk_id,
        :riskname => risk_name,
        :title => title,
        :htmlid => vul.html_id
      }
    end
    render :layout => false, :json => data
  end

  #脆弱性状況ページで使用するvul_viewを利用した「担当者にメール送信」で使用するキーワード変換のためのデータ
  def api_get_vul_view_send_mail_data
    f_params = params.permit(:html_ids, :index, :category_id)

    begin 
      html_ids = JSON.parse(f_params[:html_ids])
      index = f_params[:index]
      category_id = f_params[:category_id].to_i
  
      vul_view = SidfmdVulView.user_vul_view(@role_admin, @user_host_ids)
      cat_host_ids = nil

      if category_id > 0 then
        vul_view = vul_view.where(
          :id => SidfmdVvCategory.where(:sidfmd_category_id => category_id).select(:sidfmd_vul_view_id)
        )
      end
      if category_id != 0 && SidfmdCategory.not_system.where(:id => category_id).any? then
        if @role_admin then
          cat_host_ids = SidfmdHost.where(
            :id => SidfmdCategoriesHost.where(:sidfmd_category_id => category_id).select(:sidfmd_host_id)
          ).pluck(:id)
        else
          cat_host_ids = SidfmdHost.where(
            :id => SidfmdCategoriesHost.where(:sidfmd_category_id => category_id)
                                       .where(:sidfmd_host_id => @user_host_ids)
                                       .select(:sidfmd_host_id)
          ).pluck(:id)
        end
      else
        if !@role_admin then
          cat_host_ids = @user_host_ids
        end
      end
  
      vul_view = vul_view.where(:html_id => html_ids)
                         .order(:html_id)
                         .order('LOWER(title COLLATE "C")')
      data = []

      vul_view.each do |vul|
        if cat_host_ids then
          vul_scsids = SidfmdVvScsid.where(:sidfmd_vul_view_id => vul.id).where(:sidfmd_host_id => cat_host_ids)
        else
          vul_scsids = SidfmdVvScsid.where(:sidfmd_vul_view_id => vul.id)
        end
        max_sri = vul_scsids.where(:status_id => SidfmdIssue::STATUS_UNFINISHED).maximum(:sri)
        max_sri = SidfmdIssueSri::VALUE_NONE unless max_sri
        max_cvss = vul_scsids.where.not(:cvss => nil).order(:cvss).last
        max_cvss_score = max_cvss ? max_cvss.cvss : nil
        cvss_label = SidfmdIssueCve.labelling(max_cvss_score)
        if index == :sri then
          if vul_scsids.any? && SidfmdIssue::STATUS_UNFINISHED.include?(vul_scsids.minimum(:status_id)) then
            risk_id = max_sri
            risk_name = SidfmdIssueSri::RISKS[risk_id]
          else
            risk_id = SidfmdIssueSri::VALUE_FIXED
            risk_name = SidfmdIssueSri::RISK_FIXED
          end
          taiou = vul.warning_sri ? '要' : '不要'
        else
          if vul_scsids.any? && SidfmdIssue::STATUS_UNFINISHED.include?(vul_scsids.minimum(:status_id)) then
            risk_id = cvss_label.to_s
            risk_name = SidfmdIssueCve.display_cvss(
              { :label => cvss_label, :score => max_cvss_score }
            )
          else
            risk_id = :fixed
            risk_name = SidfmdIssueCve::RISK_FIXED
          end
          taiou = vul.warning_cvss ? '要' : '不要'
        end
 
        hosts = if @role_admin then
                  vul.sidfmd_vv_hosts
                else
                  vul.sidfmd_vv_hosts.where(:sidfmd_host_id => @user_host_ids)
                end
        title = if vul.title.split(' / ').size != vul.title.split(' / ').uniq.size then
                  vul.title.split(' / ').uniq.join(' / ')
                else
                  vul.title.split(' / ').first
                end
        min_created_on = vul_scsids.minimum(:created_on)
        min_created_on_display = min_created_on ? min_created_on.to_datetime.localtime.strftime('%Y/%m/%d') : '---'
        min_elapsed_created_on = vul_scsids.where(:status_id => SidfmdIssue::STATUS_UNFINISHED).minimum(:created_on)
        min_elapsed_created_on_display = min_elapsed_created_on ? ((Time.current - min_elapsed_created_on) / 24.0 / 60.0 / 60.0).to_i : '---'

        vul_status_ids = vul_scsids.pluck(:status_id)
        data << {
          :html_id => vul.html_id,
          :title => title,
          :risk_id => risk_id,
          :risk_name => risk_name,
          :vulname => vul.html_id < 0 ? "[KB#{-vul.html_id}]#{title}" : "[#{vul.html_id}]#{title}",
          :detected_date => min_created_on_display,
          :elapsed_days => min_elapsed_created_on_display,
          :taiou => taiou,
          :sri => max_sri,
          :cvss => cvss_label == :notreg ? '---' : sprintf('%.1f', max_cvss_score),
          :area_id => vul.softek_area_id,
          :exploit => cvss_label != :notreg ? (vul.exploitable ? 'あり' : 'なし') : '---',
          :system_category => vul.sidfmd_vv_categories.where(:is_system => true).order(:sidfmd_category_id).pluck(:name),
          :custom_category => if @role_admin then
                                vul.sidfmd_vv_categories.where(:is_system => false)
                                                        .order('LOWER(name COLLATE "C")')
                                                        .pluck(:name)
                              else
                                vul.sidfmd_vv_categories.where(:is_system => false)
                                                        .where(:sidfmd_category_id => @user_category_ids)
                                                        .order('LOWER(name COLLATE "C")')
                                                        .pluck(:name)
                              end,
          :hostnames => hosts.order('LOWER(hostname COLLATE "C")').pluck(:hostname),
          :tackle_date => if @role_admin then
                            display_percent(vul.duedates_percent)
                          else
                            duedates_percent = if vul_scsids.count != 0 then
                                                 display_percent(
                                                   (vul_scsids.where(:due_date => nil).count.to_f / vul_scsids.count.to_f * 100)
                                                 )
                                               else
                                                 display_percent(0.0)
                                               end                     
                          end,
          :tackle_status => if @role_admin then
                              display_percent(vul.fixed_percent)
                            else
                              display_finished_percent(vul_status_ids)
                            end,
          :status_ids => vul_status_ids,
          :display_names => hosts.order('LOWER(display_name COLLATE "C")').pluck(:display_name)
        }
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'データの取得に失敗しました。画面をリロードします。' } and return 
    end
    render :layout => false, :json => { :data => data }
  end

  def api_get_host_view_heat_map_data
    f_params = params.permit(:index, :html_id, :category_id)
    index = f_params[:index].to_sym
    html_id = f_params[:html_id]
    category_id = f_params[:category_id].to_i

    data = []

    if html_id.size > 0 then
      #脆弱性詳細ページの場合
      view = SidfmdVulView.user_vul_view(@role_admin, @user_host_ids).where(:html_id => html_id).joins(:sidfmd_vv_scsids)
      unless @role_admin then
        if @user_host_ids.any? then
          view = view.where(%|
            sidfmd_vv_scsids.sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}]))
          |)
        end
      end
    
      if category_id != 0 then
        cat = SidfmdCategory.find_by(:id => category_id)
        if cat then
          view = view.where(%|
            sidfmd_vv_scsids.sidfmd_host_id IN (SELECT sidfmd_host_id FROM sidfmd_categories_hosts WHERE sidfmd_category_id = #{category_id})
          |)
        end
      end
      view.distinct('sidfmd_vv_scsids.sidfmd_host_id').select(
        'sidfmd_vv_scsids.sidfmd_host_id',
        :sri,
        :status_id,
        'cvss AS cvss_score'
      ).each do |row|
        host = SidfmdHostView.find_by(:sidfmd_host_id => row.sidfmd_host_id)
        if host then
          if index == :sri then
            if SidfmdIssue::STATUS_UNFINISHED.include?(row.status_id) then
              risk_id = row.sri
              risk_name = SidfmdIssueSri::RISKS[risk_id]
            else
              risk_id = SidfmdIssueSri::VALUE_FIXED
              risk_name = SidfmdIssueSri::RISK_FIXED
            end
          else
            if SidfmdIssue::STATUS_UNFINISHED.include?(row.status_id) then
              cvss_label = SidfmdIssueCve.labelling(row.cvss_score)
              risk_id = cvss_label.to_s
              risk_name = SidfmdIssueCve.display_cvss(
                            { :label => cvss_label, :score => row.cvss_score }
                          )
            else
              risk_id = :fixed
              risk_name = SidfmdIssueCve::RISK_FIXED
            end
          end
          data << {
            :displayname => host.display_name,
            :riskid => risk_id,
            :riskname => risk_name,
            :hostid => row.sidfmd_host_id
          }
        end
      end
      data = data.sort_by{|d| d[:displayname].downcase }
    else
      #ホスト別状況ページの場合
      host_view = SidfmdHostView.user_host_view(@role_admin, @user_host_ids)
      if category_id != 0 then
        cat = SidfmdCategory.find_by(:id => category_id)
        if cat then
          if cat.is_system then
            host_view = host_view.where(:system_category_id => category_id)
          else
            host_view = host_view.where(
                          :id => SidfmdHvCategory.where(:sidfmd_category_id => category_id)
                                                 .distinct
                                                 .select(:sidfmd_host_view_id)
                        )
          end
        end
      end
      host_view.order('LOWER(display_name COLLATE "C")').select(
        :display_name,
        :sidfmd_host_id,
        'min_status_id AS status_id',
        'max_sri AS sri',
        'max_cvss_label AS cvss_label',
        'max_cvss_score AS cvss_score'
      ).each do |host|
        if index == :sri then
          if SidfmdIssue::STATUS_UNFINISHED.include?(host.status_id) then
            risk_id = host.sri
            risk_name = SidfmdIssueSri::RISKS[risk_id]
          else
            risk_id = SidfmdIssueSri::VALUE_FIXED
            risk_name = SidfmdIssueSri::RISK_FIXED
          end
        else
          if SidfmdIssue::STATUS_UNFINISHED.include?(host.status_id) then
            risk_id = host.cvss_label
            risk_name = SidfmdIssueCve.display_cvss(
                          { :label => host.cvss_label.to_sym, :score => host.cvss_score }
                        )
          else
            risk_id = :fixed
            risk_name = SidfmdIssueCve::RISK_FIXED
          end
        end
        data << {
          :displayname => host.display_name,
          :riskid => risk_id,
          :riskname => risk_name,
          :hostid => host.sidfmd_host_id
        }
      end
    end

    render :layout => false, :json => data
  end

  #host_viewを利用した「担当者にメール送信」で使用するキーワード変換のためのデータ
  def api_get_host_view_send_mail_data
    f_params = params.permit(:host_ids, :index)

    begin
      host_ids = JSON.parse(f_params[:host_ids])
      index = f_params[:index].to_sym

      hosts = SidfmdHostView.user_host_view(@role_admin, @user_host_ids)
                            .where(:sidfmd_host_id => host_ids)
                            .order('LOWER(display_name COLLATE "C")')
      data = []

      hosts.each do |host|
        if index == :sri then
          if SidfmdIssue::STATUS_UNFINISHED.include?(host.min_status_id) then
            risk_id = host.max_sri
            risk_name = SidfmdIssueSri::RISKS[risk_id]
          else
            risk_id = SidfmdIssueSri::VALUE_FIXED
            risk_name = SidfmdIssueSri::RISK_FIXED
          end
          taiou = host.warning_sri ? '要' : '不要'
        else
          if SidfmdIssue::STATUS_UNFINISHED.include?(host.min_status_id) then
            risk_id = host.max_cvss_label
            risk_name = SidfmdIssueCve.display_cvss(
                          { :label => host.max_cvss_label.to_sym, :score => host.max_cvss_score }
                        )
          else
            risk_id = :fixed
            risk_name = SidfmdIssueCve::RISK_FIXED
          end
          taiou = host.warning_cvss ? '要' : '不要'
        end
  
        status_ids = host.sidfmd_hv_scsids.pluck(:status_id)
        data << {
          :host_id => host.sidfmd_host_id,
          :hostname => host.hostname,
          :display_name => host.display_name,
          :risk_id => risk_id,
          :risk_name => risk_name,
          :taiou => taiou,
          :status_ids => status_ids,
          :sri => host.max_sri,
          :cvss => (host.max_cvss_label == 'notreg' || !host.max_cvss_label) ? '---' : sprintf('%.1f', host.max_cvss_score ),
          :area_id => host.max_area_id,
          :exploit => status_ids.size > 0 ? (host.max_cvss_label != 'notreg' ? (host.exploitable ? 'あり' : 'なし') : '---') : '---',
          :system_category => host.system_category_name ? host.system_category_name : 'none',
          :custom_category => @role_admin ? host.sidfmd_hv_categories.order('LOWER(name COLLATE "C")').pluck(:name) : host.sidfmd_hv_categories.where(:sidfmd_category_id => @user_category_ids).order('LOWER(name COLLATE "C")').pluck(:name),
          :osname => host.osname ? host.osname : '不明',
          :tackle_date => host.duedates_percent ? display_percent(host.duedates_percent) : '---',
          :tackle_status => host.fixed_percent ? display_percent(host.fixed_percent) : '---'
        }
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'データの取得に失敗しました。画面をリロードします。' } and return
    end
    render :layout => false, :json => { :data => data }
  end

  #ホスト詳細画面の担当者にメール送信のキーワード変換で使用するデータを取得
  def api_get_host_send_mail_data
    f_params = params.permit(:host_id, :html_ids, :index)

    begin
      host_id = f_params[:host_id]
      html_ids = JSON.parse(f_params[:html_ids])
      index = f_params[:index]

      if !host_id || !index then
        render :layout => false, :json => [] and return 
      end

      host_id = host_id.to_i
 
      if !@role_admin && !@user_host_ids.include?(host_id) then
        render :layout => false, :json => [] and return
      end

      host = SidfmdHost.enable_hosts.where(:id => host_id).first
      unless host then
        render :layout => false, :json => { :err_message => 'ホストが削除されています。' } and return
      end

      host_vuls = host.create_content(:corresp_required => Sidfm.corresp_required).sort{|a, b|
        r = a[:html_id] <=> b[:html_id]
        if r == 0 then
          r = a[:title] <=> b[:title]
        end              
        r
      } 

      index = index.to_sym
      data = []
      host_vuls.each do |vul|
        next unless html_ids.include?(vul[:html_id])
        case index
        when :sri then
          if SidfmdIssue::STATUS_UNFINISHED.include?(vul[:status_id]) then
            risk_id = vul[:sri]
            risk_name = SidfmdIssueSri::RISKS[risk_id]
          else
            risk_id = SidfmdIssueSri::VALUE_FIXED
            risk_name = SidfmdIssueSri::RISK_FIXED
          end
        when :cvss then
          if SidfmdIssue::STATUS_UNFINISHED.include?(vul[:status_id]) then
            risk_id = vul[:cvss][:label].to_s
            risk_name = SidfmdIssueCve.display_cvss(vul[:cvss])
          else
            risk_id = :fixed
            risk_name = SidfmdIssueCve::RISK_FIXED
          end
        end
        data << {
          :html_id => vul[:html_id],
          :count_affpkgs => vul[:count_affpkgs],
          :title => vul[:title],
          :risk_id => risk_id,
          :risk_name => risk_name,
          :vulname => vul[:html_id] < 0 ? "[KB#{-vul[:html_id]}]#{vul[:title]}" : "[#{vul[:html_id]}]#{vul[:title]}",
          :detected_date => vul[:created_on][:display],
          :elapsed_days => vul[:created_on][:elapsed_days]  > 0 ? "#{vul[:created_on][:elapsed_days]}日" : "---",
          :date => vul[:due_date] ? vul[:due_date].strftime('%Y/%m/%d') : '未設定',
          :taiou => vul[:warning][index] ? '要' : '不要',
          :sri => vul[:sri].to_i,
          :cvss => vul[:cvss][:label] == :notreg ? '---' : sprintf('%.1f', (vul[:cvss][:score])),
          :area_id => vul[:softek][:area_id],
          :exploit => vul[:cvss][:label] != :notreg ? (vul[:exploitable] ? 'あり' : 'なし') : '---',
          :status_id => vul[:status_id]  
        }       
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'データの取得に失敗しました。画面をリロードします。' } and return
    end    
    render :layout => false, :json => { :data => data }
  end

  #脆弱性詳細画面の担当者にメール送信のキーワード変換で使用するデータを取得
  def api_get_vul_send_mail_data
    f_params = params.permit(:html_id, :host_ids, :category_id, :index)

    begin
      html_id = f_params[:html_id]
      host_ids = JSON.parse(f_params[:host_ids])
      index = f_params[:index]
      category_id = f_params[:category_id]

      if !html_id || !index || !category_id then
        render :layout => false, :json => [] and return
      end
    
      data = []
      if html_id.size > 0 then 
        category_id = category_id.to_i
        hosts = SidfmdIssue.create_content(:category_id => category_id, :role_admin => @role_admin, :user_host_ids => @user_host_ids, :user_category_ids => @user_category_ids, :html_id => html_id.to_i, :corresp_required => Sidfm.corresp_required)

        if hosts.empty? then
          render :layout => false, :json => { :err_message => '脆弱性が削除されたか、ホストの変更があったため、データを取得できませんでした。' } and return 
        end

        index = index.to_sym

        hosts.sort_by{|host| host[:display_name].downcase }.each do |host|
          next unless host_ids.include?(host[:sidfmd_host_id])
          case index
          when :sri then
            if SidfmdIssue::STATUS_UNFINISHED.include?(host[:status_id]) then
              risk_id = host[:sri]
              risk_name = SidfmdIssueSri::RISKS[risk_id]
            else
              risk_id = SidfmdIssueSri::VALUE_FIXED
              risk_name = SidfmdIssueSri::RISK_FIXED
            end
          when :cvss then
            if SidfmdIssue::STATUS_UNFINISHED.include?(host[:status_id]) then
              risk_id = host[:cvss][:label].to_s
              risk_name = SidfmdIssueCve.display_cvss(host[:cvss])
            else
              risk_id = :fixed
              risk_name = SidfmdIssueCve::RISK_FIXED
            end
          end

          data << {
            :host_id => host[:sidfmd_host_id],
            :hostname => host[:hostname],
            :display_name => host[:display_name],
            :risk_id => risk_id,
            :risk_name => risk_name,
            :taiou => host[:warning][index] ? '要' : '不要',
            :status_id => host[:status_id],
            :sri => host[:sri].to_i,
            :cvss => host[:cvss][:label] == :notreg ? (host[:html_id] < 0 ? '情報無し' : '未登録') : sprintf('%.1f', (host[:cvss][:score] ? host[:cvss][:score] : 0).to_f),
            :area_id => host[:softek][:area_id],
            :exploit => host[:cvss][:label] != :notreg ? (host[:exploitable] ? 'あり' : 'なし') : '---',
            :elapsed_days => host[:elapsed_days],
            :system_category => host[:system_category] ? host[:system_category][:name] : 'none',
            :custom_category => host[:no_system_categories] ? host[:no_system_categories].map{|cc| cc[:name]} : [],
            :osname => host[:os] ? (host[:os][:osname] ? host[:os][:osname] : '不明') : '不明',
            :tackle_date => host[:due_date] ? host[:due_date] : '---'
          }
        end
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'データの取得に失敗しました。画面をリロードします。' } and return
    end
    render :layout => false, :json => { :data => data }
  end

  #Playbook出力で表示するホストの脆弱性データ
  def api_get_host_playbook_data
    f_params = params.permit(:host_id, :html_ids, :is_windows)
    host_id = f_params[:host_id]
    category_id = f_params[:category_id].to_i
    html_ids = JSON.parse(f_params[:html_ids])
    data = []
    vul_view = SidfmdVulView.user_vul_view(@role_admin, @user_host_ids)
                            .where(:html_id => html_ids)
                            .where(
                              :id => SidfmdVvHost.where(:sidfmd_host_id => host_id)
                                                 .distinct
                                                 .select(:sidfmd_vul_view_id)
    
                        )
    if f_params[:is_windows] then
      is_all_status_completed = vul_view.joins(:sidfmd_vv_scsids)
                                        .where(:sidfmd_vv_scsids => { :status_id => SidfmdIssue::STATUS_UNFINISHED })
                                        .empty?
      render :layout => false, :json => { :is_all_status_completed => is_all_status_completed }
    else
      if !@role_admin && !@user_host_ids.include?(host_id.to_i) then
        render :layout => false, :json => [] and return
      end
      affpkgs = SidfmdIssueAffpkg.joins(:sidfmd_issue).where(:sidfmd_issues => { :html_id => html_ids,:sidfmd_host_id => host_id })
      count_affpkgs = {}
      affpkgs.each do |p|
        i_id = p.sidfmd_issue.html_id.to_i
        if count_affpkgs[i_id] then
          count_affpkgs[i_id] += 1
        else
          count_affpkgs[i_id] = 1
        end
      end

      vul_view.order(:html_id).order('LOWER(title COLLATE "C")').select(
        :id,
        :title,
        :html_id
      ).each do |vul|
        title = if vul.title.split(' / ').size != vul.title.split(' / ').uniq.size then
                  vul.title.split(' / ').uniq.join(' / ')
                else
                  vul.title.split(' / ').first
                end
        data << {
          :title => title,
          :html_id => vul.html_id,
          :status_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul.id).where(:sidfmd_host_id => host_id).first.status_id,
          :count_affpkgs => count_affpkgs[vul.html_id] ? count_affpkgs[vul.html_id] : 0 #affpkgs.where('sidfmd_issues.html_id = ?', vul.html_id).count
        }
      end
      render :layout => false, :json => data
    end
  end

  #SidfmdVulViewの脆弱性に紐づくCVE情報を返す
  def api_get_vul_view_cve_info
    f_params = params.permit(:vul_id)
    vul_id = f_params[:vul_id].to_i
    err = false
    r = {}
    vul = SidfmdVulView.find_by(:id => vul_id)
    if vul then
      if !@role_admin then
        if vul.sidfmd_vv_categories.where(:sidfmd_category_id => @user_category_ids).empty? then
          err = true
        end
      end
    else
      err = true
    end
    
    if !err then
      #CVEIDが同じものがある場合は、
      #c3_update_datetimeがあればc3_update_datetime, 
      #c2_update_datetimeがあればc2_update_datetimeで最も新しいもの1つを採用する。
      #両方無い場合はidが最も大きいものを採用する。
      cves = vul.sidfmd_vv_cves.order('cve_id COLLATE "C"')
=begin      cves = vul.sidfmd_vv_cves.where(%|
          id = (
            SELECT id
            FROM sidfmd_vv_cves AS cve_table2
            WHERE cve_table.sidfmd_vul_view_id = cve_table2.sidfmd_vul_view_id AND cve_table.cve_id = cve_table2.cve_id
            ORDER BY 
            CASE
            WHEN c3_update_datetime IS NOT NULL THEN
              100
            WHEN c2_update_datetime IS NOT NULL THEN
              200
            ELSE
              300
            END,
            c3_update_datetime,
            c2_update_datetime,
            id
             LIMIT 1
           )
      |).select('c2_av AS AV', 'c2_ac AS AC', 'c2_au AS Au', 'c2_c AS C', 'c2_i AS I', 'c2_a AS A')
=end      
      if cves.any? then
        filtered_cves = []
        target_cve = nil
        create_cve_data = ->(cve){
          cvss_v2 = if cve.c2_av then
                      { 
                        :AV => cve.c2_av,
                        :AC => cve.c2_ac,
                        :Au => cve.c2_au,
                        :C => cve.c2_c,
                        :I => cve.c2_i,
                        :A => cve.c2_a 
                      }
                    else
                      nil
                    end
          cvss_v3 = if cve.c3_av then
                      {
                        :AV => cve.c3_av,
                        :AC => cve.c3_ac,
                        :PR => cve.c3_pr,
                        :UI => cve.c3_ui,
                        :S => cve.c3_s,
                        :C => cve.c3_c,
                        :I => cve.c3_i,
                        :A => cve.c3_a
                      }
                    else
                      nil
                    end
          return { 
            :cve_id => cve.cve_id,
            :cvss_v2 => cvss_v2,
            :cvss_v3 => cvss_v3,
          }
        }
        cves.each_with_index do |cve, idx|
          if idx == 0 then
            target_cve = cve
          else
            if target_cve.cve_id == cve.cve_id then
              if cve.c3_update_datetime then
                if !target_cve.c3_update_datetime then
                  target_cve = cve
                else
                  target_cve = cve if target_cve.c3_update_datetime < cve.c3_update_datetime
                end
              elsif !target_cve.c3_update_datetime && cve.c2_update_datetime then
                if !target_cve.c2_update_datetime then
                  target_cve = cve
                else
                  target_cve = cve if target_cve.c2_update_datetime < cve.c2_update_datetime
                end
              elsif !target_cve.c3_update_datetime && !target_cve.c2_update_datetime then
                target_cve = cve if target_cve.id < cve.id
              end
            else 
              filtered_cves << create_cve_data.(target_cve)
              target_cve = cve
            end
          end
          if idx == cves.size - 1 then
            filtered_cves << create_cve_data.(target_cve)
          end
        end
        r[:cves] = filtered_cves
      else
        err = true
      end
    end
    r[:err] = err
    render :layout => false, :json => r 
  end

  #ホストの脆弱性のcve情報を返す。
  #ホスト詳細画面のcve取得で使用する
  def api_get_vul_view_cve_info_of_host
    f_params = params.permit(:html_id, :host_id)
    html_id = f_params[:html_id].to_i
    host_id = f_params[:host_id].to_i
    r = { :err => false, :cves => [] }

    if !@role_admin then
      if !@user_host_ids.include?(host_id) then
        r[:err] = true
      end
    end
    if !r[:err] then
      cves = SidfmdIssueCve.where(
        :sidfmd_issue_id => SidfmdIssue.where(:html_id => html_id, :sidfmd_host_id => host_id).select(:id)
      ).select(:cve_id, :c2_vector, :c3_vector)
      cves.each do |cve|
        c2_vector = cve.c2_vector
        c3_vector = cve.c3_vector
        cvss_v2 = nil
        cvss_v3 = nil
        if c2_vector then
          cvss_v2 = c2_vector.split('/').map{|v| v.split(':') }.to_h.symbolize_keys
        end
        if c3_vector then
          cvss_v3 = c3_vector.split('/').map{|v| v.split(':') }.to_h.symbolize_keys
        end
        r[:cves] << {
          :cve_id => cve.cve_id,
          :cvss_v2 => cvss_v2,
          :cvss_v3 => cvss_v3
        }
      end
    end
    render :layout => false, :json => r
  end

  TIMELINE_SPAN = 6
  TIMELINE_SPAN_UNIT = :month

  # ホスト詳細画面・脆弱性詳細画面のタイムラインのデータを返す
  def api_get_timeline_data
    f_params = params.permit(:host_ids, :html_id, :category_id, :idx)
    host_ids = JSON.parse(f_params[:host_ids])
    html_id = f_params[:html_id] ? f_params[:html_id].to_i : nil
    idx = f_params[:idx].to_sym
    host_ids = host_ids.map{|h_id| h_id.to_i}
    if !@role_admin then
      if (host_ids - @user_host_ids).any? then
        render :layout => false, :json => { :err => true } and return
      end
    end
    hosts = SidfmdHost.where(:id => host_ids)
    if host_ids.any? and !hosts.exists? then
      render :layout => false, :json => { :err => true } and return
    end
    content = if html_id then
                SidfmdIssue.create_content(:category_id => f_params[:category_id].to_i, :role_admin => @role_admin, :user_host_ids => @user_host_ids, :user_category_ids => @user_category_ids, :html_id => html_id, :corresp_required => Sidfm.corresp_required) 
              else
                hosts.first.create_content(:corresp_required => Sidfm.corresp_required)
              end
    ret = {}
    ret[:now] = Time.current
    ret[:localtime] = ret[:now].localtime
    ret[:timescale] = []
    time_condition = ''
    TIMELINE_SPAN.times do |i|
      z = ret[:localtime].ago((TIMELINE_SPAN - 1 - i).send(TIMELINE_SPAN_UNIT))
      time_condition = z if i == TIMELINE_SPAN - 1
      r = {
        :time => z,
        :year => z.year,
        :month => z.month,
        :step => TIMELINE_SPAN - 1 - i,
        :is_first => (i == 0),
        :is_last => (i == (TIMELINE_SPAN - 1)),
        :name => z.strftime('%Y/%m'),
      }
      ret[:timescale] << r
    end
    ret[:status_counts] = {
      SidfmdIssue::STATUS_NOTCONFIRMED => [0] * ret[:timescale].size,
      SidfmdIssue::STATUS_ONGOING      => [0] * ret[:timescale].size,
      SidfmdIssue::STATUS_FIXED        => [0] * ret[:timescale].size,
      SidfmdIssue::STATUS_NOTAFFECT    => [0] * ret[:timescale].size,
    }
    ret[:status_names] = {}
    SidfmdIssue::STATUS_ALL.each do |s|
      ret[:status_names][s] = SidfmdIssue.status_id_to_s(s)
    end
    case idx
    when :sri then
      ret[:risk_syms] = {
        0 => 'none', 1 => 'low', 2 => 'medium', 3 => 'high', 4 => 'critical', 5 => 'fixed',
      }
      ret[:risk_names] = {
        0 => '無', 1 => '低', 2 => '中', 3 => '高', 4 => '重大', 5 => '済',
      }
      ret[:risk_counts] = ret[:risk_syms].each_key.collect {|x|
        [0] * ret[:timescale].size
      }
    when :cvss then
      ret[:risk_labels] = [ :notreg, :d4, :d3, :d2, :d1, :fixed ]
      ret[:risk_names] = {
        :notreg => '未登録',
        :d4 => '10.0〜9.0',
        :d3 => '8.9〜7.0',
        :d2 => '6.9〜4.0',
        :d1 => '3.9〜0.0',
        :fixed => '済',
      }
      ret[:risk_counts] = {}
      ret[:risk_labels].each do |i|
        ret[:risk_counts][i] = [0] * ret[:timescale].size
      end
    end
   
    if html_id and idx == :sri then
      #SidfmdIssue#create_contenttのSriは正しい値ではないため、取得し直す
      host_sri = {}
      SidfmdVulView.joins(:sidfmd_vv_scsids)
                   .where(:sidfmd_vul_views => { :html_id => html_id })
                   .where(:sidfmd_vv_scsids => { :sidfmd_host_id => content.map{|h| h[:sidfmd_host_id]} })
                   .select(:sidfmd_host_id, :sri)
                   .each do |data|
                     host_sri[data.sidfmd_host_id] = data.sri  
                   end
    end
    count_issue_risks = ->(item, status_id, count_index, idx){
      ret[:status_counts][status_id][count_index] += 1
      if SidfmdIssue::STATUS_UNFINISHED.include?(status_id) then
        case idx
        when :sri then
          if html_id then
            sri = host_sri[item[:sidfmd_host_id]]
            ret[:risk_counts][sri][count_index] += 1 if sri and ret[:risk_counts][sri] 
          else
            ret[:risk_counts][item[:sri]][count_index] += 1
          end
        when :cvss then
          ret[:risk_counts][item[:cvss][:label]][count_index] += 1
        end
      else
        case idx
        when :sri then
          ret[:risk_counts][5][count_index] += 1
        when :cvss then
          ret[:risk_counts][:fixed][count_index] += 1
        end
      end
    }

    issue_journal_status_ids = {}
   
    SidfmdIssue.joins(:issue)
               .joins(%| LEFT OUTER JOIN journals ON journals.journalized_id = issues.id |)
               .joins(%| LEFT OUTER JOIN journal_details ON journals.id = journal_details.journal_id |)
               .where(%|
                 sidfmd_issues.sidfmd_host_id IN (?) AND
                 issues.created_on IS NOT NULL AND
                 issues.created_on <= ? AND
                 journals.created_on <= ?
                 #{html_id ? "AND sidfmd_issues.html_id = #{html_id}" : ''}
               |, host_ids, time_condition, time_condition)
               .where(:journal_details => { :prop_key => 'status_id' })
               .order("journals.created_on DESC")
               .order("journals.id DESC")
               .select(
                 "sidfmd_issues.id AS sidfmd_issue_id", 
                 "journal_details.value AS journal_value",
                 "journals.created_on AS journal_created_on"
               )
               .each do |issue|
                   issue_journal_status_ids[issue.sidfmd_issue_id] = [] unless issue_journal_status_ids[issue.sidfmd_issue_id]
                   issue_journal_status_ids[issue.sidfmd_issue_id] << {
                   :status_id => issue.journal_value.to_i,
                   :journal_created_on => issue.journal_created_on
                 }
               end

    content.each do |item|
      ret[:timescale].each do |step|
        next if item[:created_on][:value] > step[:time]
        issue_status_id = item[:status_id]
        count_index = TIMELINE_SPAN - 1 - step[:step]
        journal_status_id = nil
        journals = issue_journal_status_ids[item[:sidfmd_issue_id]]
        if journals then
          temp_j = journals.find{|j| j[:journal_created_on] <= step[:time]}
          journal_status_id = temp_j[:status_id] if temp_j
        end

        if journal_status_id then
          count_issue_risks.(item, journal_status_id, count_index, idx)
        elsif issue_status_id then
          count_issue_risks.(item, issue_status_id, count_index, idx)
        end
      end
    end
    render :layout => false, :json => { :timeline_data => ret }
  end

  #脆弱性詳細画面で使用する
  #選択されたホストの内、対応状況が未対策・対策中のもののみを抽出する
  def api_get_unfinished_hostids
    f_params = params.permit(:host_ids, :html_id)
    host_ids = JSON.parse(f_params[:host_ids])
    html_id = f_params[:html_id].to_i
    target_hostids = SidfmdVulView.joins(:sidfmd_vv_scsids)
                                  .where(:sidfmd_vul_views => {:html_id => html_id})
                                  .where(:sidfmd_vv_scsids => {:sidfmd_host_id => host_ids, :status_id => SidfmdIssue::STATUS_UNFINISHED})
                                  .distinct
                                  .pluck(:sidfmd_host_id)
    not_target_hostids = host_ids - target_hostids
    render :layout => false, :json => {:target_ids => target_hostids, :not_target_ids => not_target_hostids}               
  end

  #ホスト詳細画面で使用する
  #選択された脆弱性の内、対応状況が未対策・対策中のもののみを抽出する
  def api_get_unfinished_htmlids
    f_params = params.permit(:host_id, :html_ids)
    host_id = f_params[:host_id].to_i
    html_ids = JSON.parse(f_params[:html_ids])
    target_htmlids = SidfmdVulView.joins(:sidfmd_vv_scsids)
                                  .where(:sidfmd_vul_views => {:html_id => html_ids})
                                  .where(:sidfmd_vv_scsids => {:sidfmd_host_id => host_id, :status_id => SidfmdIssue::STATUS_UNFINISHED})
                                  .distinct
                                  .pluck(:html_id)
    not_target_htmlids = html_ids - target_htmlids
    render :layout => false, :json => {:target_ids => target_htmlids, :not_target_ids => not_target_htmlids}
  end

  #文字列の部分一致もしくはidの配列で、履歴情報画面で使うフィルターのデータを取得し返す
  def api_get_history_filter_data
    f_params = params.permit(:str, :ids, :item, :filter_item)
    str = f_params[:str]
    ids = JSON.parse(f_params[:ids] || '[]')
    r = []
    case f_params[:filter_item]
    when 'users' then
      users = if !@role_admin then
                User.joins(:email_addresses).where(%|
                  users.id IN (
                    SELECT user_id 
                    FROM sidfmd_categories_users
                    WHERE sidfmd_categories_users.sidfmd_category_id IN (SELECT unnest(ARRAY[#{@user_category_ids.join(', ')}]))
                  ) OR users.login = 'sidfmadm'
                |).where.not(:type => 'AnonymousUser')
              else
                User.joins(:email_addresses).where.not(:type => 'AnonymousUser')
              end
      if ids.any? then
        users = users.where(:id => ids)
      elsif str and str.size > 0 then
        case f_params[:item]
        when 'login' then
          users = users.where('login ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, str)}%")
        when 'email' then
          users = users.where(
                              :id => EmailAddress.where('address ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, str)}%").select(:user_id)
          )
        when 'name' then
          users = users.where('firstname ILIKE ? OR lastname ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, str)}%", "%#{ActiveRecord::Base.send(:sanitize_sql_like, str)}%")
        end
      else
        users = users.where(:id => 0)
      end

      r = users.order('LOWER(login COLLATE "C")').select(:id, :login, :address, :firstname, :lastname)
    when 'hosts' then
      hosts = if !@role_admin then
                SidfmdHost.where(:id => @user_host_ids)
              else
                SidfmdHost.all
              end
      if ids.any? then
        hosts = hosts.where(:id => ids)
      elsif str and str.size > 0 then
        case f_params[:item]
        when 'hostname' then
          hosts = hosts.where('hostname ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, str)}%")
        when 'display_name' then
          hosts = hosts.where('ext_name ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, str)}%")
        end
      else
        hosts = hosts.where(:id => 0)
      end

      r = hosts.order(%| 
        CASE 
        WHEN ext_name IS NULL THEN
          LOWER(hostname COLLATE "C")
        ELSE
          LOWER(ext_name COLLATE "C")
        END
      |).select(
        :id, 
        :hostname,
        :ext_name
      )
    when 'categories' then
      categories = if !@role_admin then
                     SidfmdCategory.where(%| is_system = true OR id IN (?) |, @user_category_ids)
                   else
                     SidfmdCategory.all
                   end
      if ids.any? then
        categories = categories.where(:id => ids)
      elsif str and str.size > 0 then
        categories = categories.where('name ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, str)}%")
      else
        categories = categories.where(:id => 0)
      end
   
      r = categories.order(%|
        CASE WHEN is_system = true THEN 10 ELSE 100 END
      |)
      .order(%|
        CASE WHEN is_system = true THEN CAST(id AS text) ELSE LOWER(name COLLATE "C") END
      |) 
      .select(:id, :name)
    end

    render :layout => false, :json => r
  end

  def api_get_download_max_size
    f_params = params.permit(:target)
    r = nil
    case f_params[:target]
    when 'history_json_tsv' then
      r = Sidfm.get_history_max_download_size
    end
    render :layout => false, :json => { :max_size => r }
  end

  def api_get_has_vuls_hostids
    f_params = params.permit(:host_ids)
    hostids = JSON.parse(f_params[:host_ids])
    hosts = SidfmdVvHost.where(:sidfmd_host_id => hostids)
    unless @role_admin then
      hosts = hosts.where(:sidfmd_host_id => @user_host_ids)
    end
    render :layout => false, :json => hosts.distinct.pluck(:sidfmd_host_id)
  end

end
