#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class CreateSidfmdVulViews < ActiveRecord::Migration

  def up
    # 脆弱性一覧参照テーブル
    create_table :sidfmd_vul_views do |t|

      # :sri_cvss_status_ids
      # sidfmd_vv_scsidsテーブルへ

      # :html_id
      # < 0 : Windows用VM Agentから取得したKB番号の逆数
      # SidfmdLocalVul::MIN/MAX_HTMLIDの範囲: ローカル脆弱性のhtml_id
      # 上記以外: SIDfmID
      t.integer :html_id

      # :title
      t.text :title

      # :softek
      t.integer :softek_risk_id
      t.integer :softek_area_id

      # :short_description
      t.text :short_description

      # :exploitable
      t.boolean :exploitable

      # :hosts
      # sidfmd_vv_hostsテーブルへ

      # :categories
      # sidfmd_vv_categoriesテーブルへ

      # :status_ids
      # sidfmd_vv_scsidsテーブルから取得
      # ex. self.sidfmd_vv_scsids.map(&:status_id)

      # :max_sri
      t.integer :max_sri

      # :min_status_id
      t.integer :min_status_id

      # :cvss
      t.float :cvss_score
      t.text :cvss_label

      # :warning_orig
      t.boolean :warning_orig_sri
      t.boolean :warning_orig_cvss

      # :warning
      t.boolean :warning_sri
      t.boolean :warning_cvss

      # :fixed_warning
      t.boolean :fixed_warning_sri
      t.boolean :fixed_warning_cvss

      # :not_fixed_warning
      t.boolean :not_fixed_warning_sri
      t.boolean :not_fixed_warning_cvss

      # :not_fixed_hosts
      t.integer :not_fixed_hosts

      # 対応予定日設定率
      t.float :duedates_percent, index: true

      # 対処率
      t.float :fixed_percent, index: true

      # :due_dates
      # sidfmd_vv_due_datesテーブルへ

      # :due_dates_unfinished
      # sidfmd_vv_unf_due_datesテーブルへ

      # :created_ons
      # sidfmd_vv_scsidsテーブルから取得
      # ex. self.sidfmd_vv_scsids.map(&:created_on)

      # :min_created_on
      # elapsed_days(経過日数)は、後述の方法で取得時に算出します。
      t.datetime :min_created_on
      t.text :min_created_on_display

      # 以下は、経過日数(elapsed_days)を生成するために利用するのみなので、
      # 一部のみ実装します。
      # 足りないものがあれば、ご確認下さい。
      # :elapsed_created_ons
      # :min_elapsed_created_on
      #
      # 以下は、実際にelapsed_daysの値を生成している実装です。
      # 
      # temp_cu = item[:created_ons].compact.uniq
      # item[:min_created_on] = { :value => temp_cu.min }
      # item[:min_created_on][:display] = item[:min_created_on][:value] ? item[:min_created_on][:value].localtime.strftime('%Y/%m/%d') : '---'
      # item[:min_created_on][:elapsed_days] = item[:min_created_on][:value] ? ((Time.current - item[:min_created_on][:value]) / 24.0 / 60.0 / 60.0).to_i : -1
      # item[:elapsed_created_ons] = item[:sri_cvss_status_ids].collect {|x| SidfmdIssue::STATUS_UNFINISHED.include?(x[:status_id]) ? x[:created_on] : nil }.compact
      # item[:min_elapsed_created_on] = { :value => item[:elapsed_created_ons].compact.uniq.min }
      # item[:min_elapsed_created_on][:display] = item[:min_elapsed_created_on][:value] ? ((Time.current - item[:min_elapsed_created_on][:value]) / 24.0 / 60.0 / 60.0).to_i : -1
      #
      # 必要な時に以下の値からelapsed_daysを生成します。
      t.datetime :min_elapsed_created_on
      t.text :min_elapsed_created_on_display

      t.timestamps null: false
    end

    # :sri_cvss_status_ids
    # status_id と cvss と sri をセットで保持しているテーブル。
    # 実際には、sidfmd_host_id, due_date, created_onsも保持しており、脆弱性に
    # 対して関連付けられている各ホストの状態がほぼ全て確認可能。
    # 「特定のstatus_idについて、cvssの最高値を取得」のような時に利用する。
    # FIXME: max_css, max_sri等はこれから生成しているので、集約していく予定。
    create_table :sidfmd_vv_scsids do |t|
      t.references :sidfmd_vul_view, foreign_key: true, index: true, null: false

      t.references :sidfmd_host, foreign_key: true, index: true, null: false
      t.integer :status_id
      t.integer :sri
      t.float :cvss
      t.date :due_date
      t.datetime :created_on
    end

    # :hosts
    # 関連づけられているホストのホスト名・表示名の一覧。
    # ステータス等でフィルタする場合はsidfmd_vv_scsidsと併用する。
    create_table :sidfmd_vv_hosts do |t|
      t.references :sidfmd_vul_view, foreign_key: true, index: true, null: false

      t.references :sidfmd_host, foreign_key: true, index: true, null: false
      t.text :hostname
      t.text :ext_name
      t.text :display_name
    end

    # :categories
    # 関連づけられているホストのカテゴリ一覧
    create_table :sidfmd_vv_categories do |t|
      t.references :sidfmd_vul_view, foreign_key: true, index: true, null: false

      t.integer :sidfmd_category_id
      t.boolean :is_system
      t.text :name
    end

    # :due_dates
    create_table :sidfmd_vv_due_dates do |t|
      t.references :sidfmd_vul_view, foreign_key: true, index: true, null: false

      t.date :dt
    end

    # :due_dates_unfinished
    create_table :sidfmd_vv_unf_due_dates do |t|
      t.references :sidfmd_vul_view, foreign_key: true, index: true, null: false

      t.date :dt
    end
  end

  def down
    drop_table :sidfmd_vv_unf_due_dates
    drop_table :sidfmd_vv_due_dates
    drop_table :sidfmd_vv_categories
    drop_table :sidfmd_vv_hosts
    drop_table :sidfmd_vv_scsids
    drop_table :sidfmd_vul_views
  end

end
